#include "assert.h"

#include "string.h"
#include "math.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_rotozoom.h"
#include "SDL_ttf.h"

#include "List.h"

#include "auxiliar.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "interfaces.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activemap.h"
#include "ActiveINTERFACE.h"
#include "activemog2.h"
#include "actionscript.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int surface_flags;


ActiveINTERFACE::ActiveINTERFACE(INTERFACE *interfp,ActiveMOG2 *pgame)
{
	back_sfc=0;
      
	interf=interfp;
    game=pgame;
	interf_focusing_x=0;
	interf_focusing_y=0;

	interface_variables.Copy(interf->interface_variables);

	{
        int i;
		List<PLACED_OBJECT> l;
		PLACED_OBJECT *o;

        for(i=0;i<MAX_OBJECT_PLANES;i++) {
            l.Instance(interf->objects[i]);
            l.Rewind();
			while(l.Iterate(o)) {
				if (!o->object->multiroom_flag ||
					(o->object->multiroom_flag && game->find_object(o->ID)==0)) {
					if (i!=0 || !o->object->passive()) {
						ActiveOBJECT *ao;
						ao=new ActiveOBJECT(*o,0,this);
						
						objects[i].Add(ao);
						if (!ao->object->passive()) nonpassive_objects[i].Add(ao);
					} /* if */ 
				} /* if */ 
            } /* while */ 
        } /* for */ 
	}

	if (interf->interface_action!=0) {
		interface_action=new ActionSCRIPT(*interf->interface_action);
		interface_action->init();
	} else {
		interface_action=0;
	} /* if */ 
	script_foreign_object=0;
        
} /* ActiveINTERFACE::ActiveINTERFACE */ 


ActiveINTERFACE::~ActiveINTERFACE(void)
{
	int i;

    game=0;
	if (back_sfc!=0) SDL_FreeSurface(back_sfc);
	back_sfc=0;

	for(i=0;i<MAX_OBJECT_PLANES;i++) while(!nonpassive_objects[i].EmptyP()) nonpassive_objects[i].ExtractIni();
	while(!objects_to_change_plane.EmptyP()) objects_to_change_plane.ExtractIni();

	if (interface_action!=0) delete interface_action;
	interface_action=0;
} /* ActiveINTERFACE::~ActiveINTERFACE */ 


bool ActiveINTERFACE::cycle(KEYBOARDSTATE **LVK,int n_LVKs)
{
	List<ActiveOBJECT> l,todelete[MAX_OBJECT_PLANES];
	ActiveOBJECT *o;
    int i;

	draw_commands.Delete();

	/* INTERFACE SCRIPT: */ 
	if (interface_action!=0 &&
		interface_action->cycle(0,0,0,game,this,game->app,&script_foreign_object,LOCATION_INTERFACE,LVK,n_LVKs)) {
		delete interface_action;
		interface_action=0;
	} /* if */ 

	/* OBJECT PRECYCLES: */ 
    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        l.Instance(nonpassive_objects[i]);
        l.Rewind();
        while(l.Iterate(o)) o->precycle();
    } /* for */ 

	/* OBJECT CYCLES: */ 
    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        l.Instance(nonpassive_objects[i]);
        l.Rewind();
        while(l.Iterate(o)) {
			o->cycle(LVK,n_LVKs);
			if (o->terminate) todelete[i].Add(o);
        } /* while */ 
    } /* for */ 

	/* DELETE TERMINATED OBJECTS: */ 
    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        while(!todelete[i].EmptyP()) {
            ActiveOBJECT *o=todelete[i].ExtractIni();
            objects[i].DeleteElement(o);
			nonpassive_objects[i].DeleteElement(o);		
			if (objects_to_change_plane.MemberRefP(o)) objects_to_change_plane.DeleteElement(o);
            delete o;
        } /* while */ 
    } /* for */ 

    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        l.Instance(objects[i]);
        l.Rewind();
        while(l.Iterate(o)) o->delete_terminated_children();
    } /* for */ 

	/* ADD THE NEW OBJECTS: */ 
    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        while(!objects_toadd[i].EmptyP()) {
            o=objects_toadd[i].ExtractIni();
            objects[i].Add(o);
			if (!o->passive()) {
				nonpassive_objects[i].Add(o);
			} else {
				if (i==0 && back_sfc!=0) {
					/* Delete the back image since the background surface has to be recomputed: */ 
					SDL_FreeSurface(back_sfc);
					back_sfc=0;
				} /* if */ 
			} /* if */ 
        } /* while */ 
    } /* for */ 

	/* CHANGE OBJECTS OF PLANE: */ 
	while(!objects_to_change_plane.EmptyP()) {
		o=objects_to_change_plane.ExtractIni();
		int new_p=o->object_plane;
		o->object_plane=-1;
		o->get_object_plane();
		objects[o->object_plane].DeleteElement(o);
		objects[new_p].Add(o);

		if (!o->passive()) {
			nonpassive_objects[o->object_plane].DeleteElement(o);
			nonpassive_objects[new_p].Add(o);
		} /* if */ 
		o->object_plane=new_p;
	} /* whiel */ 

	return true;
} /* ActiveINTERFACE::cycle */ 


void ActiveINTERFACE::draw(SDL_Surface *sfc,SDL_Rect vp)
{
	SDL_Surface *rendering_sfc=sfc;
	SDL_Rect rendering_vp;
	int viewx=0;
	int viewy=0;
	float scale=1.0F;
	int sizex=interf->dx;
	int sizey=interf->dy;

//	if (vp.w>sizex) vp.w=sizex;
//	if (vp.h>sizey) vp.h=sizey;

	if (back_sfc==0 ||
		back_sfc->w!=sizex ||
		back_sfc->h!=sizey) {

		/* The backgorund sfc doesn't have any APHA channel: */ 
		if (back_sfc!=0) SDL_FreeSurface(back_sfc);
		back_sfc=SDL_CreateRGBSurface(SDL_SRCALPHA|surface_flags,sizex,sizey,32,0,0,0,0);

		/* Draw the background: */ 
		SDL_FillRect(back_sfc,0,0);
		{
			List<ActiveOBJECT> l2;
			ActiveOBJECT *ao;
			List<PLACED_OBJECT> l;
			PLACED_OBJECT *o;
			List<PLACED_TILE> l1;
			PLACED_TILE *t;
			SDL_Rect r;

            l1.Instance(interf->tiles[0]);
            l1.Rewind();
            while(l1.Iterate(t)) t->draw(0,0,back_sfc);

			/* Draw also Static objects with only 1 bitmap */ 
			l.Instance(interf->objects[0]);
			l.Rewind();
			while(l.Iterate(o)) {
				if (o->object->passive()) o->draw(0,0,back_sfc);
			} /* while */ 

			/* The static objects that have been created after creating the interface: */ 
			l2.Instance(objects[0]);
			l2.Rewind();
			while(l2.Iterate(ao)) {
				if (ao->object->passive()) {
					r.x=0;
					r.y=0;
					r.w=sizex;
					r.h=sizey;
					ao->draw(0,0,back_sfc,&r);
				} /* if */ 
			} /* while */ 
		}
	} /* if */ 

	/* Compute the viewport depending on the display mode: */ 
	/* 0 - default  */ 
	/* 1 - centered	*/ 
	/* 2 - scale	*/ 
	/* 3 - scale up	*/ 
	/* 4 - scale down	*/ 
	/* 5 - scroll (centrado en interf_focusing_x,interf_focusing_y) */ 
	switch(interf->display_mode) {
	case 0: viewx=0;
			viewy=0;
			scale=1.0F;
			break;
	case 1: viewx=(interf->dx-vp.w)/2;
			viewy=(interf->dy-vp.h)/2;	
			scale=1.0F;
			break;
	case 2: {
				float scale_x=1.0F,scale_y=1.0F;

				scale_x=float(vp.w)/float(interf->dx);
				scale_y=float(vp.h)/float(interf->dy);

				viewx=0;
				viewy=0;
				scale=(scale_x<scale_y ? scale_x:scale_y);	
			}
			break;
	case 3: {
				float scale_x=1.0F,scale_y=1.0F;

				scale_x=float(vp.w)/float(interf->dx);
				scale_y=float(vp.h)/float(interf->dy);

				viewx=0;
				viewy=0;
				scale=(scale_x<scale_y ? scale_x:scale_y);	

				if (scale<=1.0F) {
					viewx=(interf->dx-vp.w)/2;
					viewy=(interf->dy-vp.h)/2;	
					scale=1.0F;
				} /* if */ 
			}
			break;
	case 4: {
				float scale_x=1.0F,scale_y=1.0F;

				scale_x=float(vp.w)/float(interf->dx);
				scale_y=float(vp.h)/float(interf->dy);

				viewx=0;
				viewy=0;
				scale=(scale_x<scale_y ? scale_x:scale_y);	

				if (scale>=1.0F) {
					viewx=(interf->dx-vp.w)/2;
					viewy=(interf->dy-vp.h)/2;	
					scale=1.0F;
				} /* if */ 
			}
			break;
	case 5: if (interf->dx<vp.w) {
				viewx=(interf->dx-vp.w)/2;
			} else {
				viewx=interf_focusing_x-vp.w/2;
				if (viewx<0) viewx=0;
				if (viewx>(interf->dx-vp.w)) viewx=interf->dx-vp.w;
			} /* if */ 

			if (interf->dy<vp.h) {
				viewy=(interf->dy-vp.h)/2;
			} else {
				viewy=interf_focusing_y-vp.h/2;
				if (viewy<0) viewy=0;
				if (viewy>(interf->dy-vp.h)) viewy=interf->dy-vp.h;
			} /* if */ 
			scale=1.0F;
			
			break;
	} /* switch */ 

	if (scale==1.0F) {
		rendering_sfc=sfc;
		rendering_vp.x=vp.x;
		rendering_vp.y=vp.y;
		rendering_vp.w=vp.w;
		rendering_vp.h=vp.h;
	} else {
		rendering_sfc=SDL_CreateRGBSurface(surface_flags,interf->dx,interf->dy,32,0,0,0,0);
		SDL_FillRect(rendering_sfc,0,0);
		rendering_vp.x=0;
		rendering_vp.y=0;
		rendering_vp.w=interf->dx;
		rendering_vp.h=interf->dy;
	} /* if */ 

	/* Compute the Clipping rectangle: */ 
	{
		SDL_Rect old_clip,new_clip,interf_rect;

		SDL_GetClipRect(rendering_sfc,&old_clip);

		interf_rect.x=rendering_vp.x-viewx;
		interf_rect.y=rendering_vp.y-viewy;
		interf_rect.w=interf->dx;
		interf_rect.h=interf->dy;
		new_clip.x=rendering_vp.x;
		new_clip.y=rendering_vp.y;
		new_clip.w=rendering_vp.w;
		new_clip.h=rendering_vp.h;
		if (new_clip.x<interf_rect.x) new_clip.x=interf_rect.x;
		if (new_clip.y<interf_rect.y) new_clip.y=interf_rect.y;
		if (new_clip.x+new_clip.w>interf_rect.x+interf_rect.w) new_clip.w=(interf_rect.x+interf_rect.w)-new_clip.x;
		if (new_clip.y+new_clip.h>interf_rect.y+interf_rect.h) new_clip.h=(interf_rect.y+interf_rect.h)-new_clip.y;

		SDL_SetClipRect(rendering_sfc,&new_clip);


		/* Draw the interface: */ 
		/* Background: */ 
		{
			List<ActiveOBJECT> l;
			ActiveOBJECT *o;
			SDL_Rect r,d;

			r.x=viewx;
			r.y=viewy;
			r.w=rendering_vp.w;
			r.h=rendering_vp.h;

			d.x=rendering_vp.x;
			d.y=rendering_vp.y;
			d.w=rendering_vp.w;
			d.h=rendering_vp.h;

			SDL_BlitSurface(back_sfc,&r,rendering_sfc,&d);

			l.Instance(nonpassive_objects[0]);
			l.Rewind();
			while(l.Iterate(o)) o->draw(rendering_vp.x-viewx,rendering_vp.y-viewy,rendering_sfc,&rendering_vp);
		}

		/* Foreground: */ 
		{
            int i;
			List<ActiveOBJECT> l;
			ActiveOBJECT *o;
			List<PLACED_TILE> l1;
			PLACED_TILE *t;

            for(i=1;i<MAX_OBJECT_PLANES;i++) {
                l1.Instance(interf->tiles[i]);
                l1.Rewind();
                while(l1.Iterate(t)) t->draw(rendering_vp.x-viewx,rendering_vp.y-viewy,rendering_sfc,rendering_vp);

                l.Instance(objects[i]);
                l.Rewind();
                while(l.Iterate(o)) o->draw(rendering_vp.x-viewx,rendering_vp.y-viewy,rendering_sfc,&rendering_vp);
            } /* for */ 
		}

		/* Extra drawing elements: */ 
		{
			List<DRAWCOMMAND> l;
			DRAWCOMMAND *dc;

			SDL_LockSurface(rendering_sfc);
			l.Instance(draw_commands);
			l.Rewind();
			while(l.Iterate(dc)) {
				switch(dc->command) {
				case 0:
					putpixel(rendering_sfc,dc->x1+rendering_vp.x-viewx,dc->y1+rendering_vp.x-viewx,SDL_MapRGBA(rendering_sfc->format,dc->r,dc->g,dc->b,dc->a));
					break;
				case 1:
					draw_line(rendering_sfc,dc->x1+rendering_vp.x-viewx,dc->y1+rendering_vp.x-viewx,dc->x2+rendering_vp.x-viewx,dc->y2+rendering_vp.x-viewx,SDL_MapRGBA(rendering_sfc->format,dc->r,dc->g,dc->b,dc->a));
					break;
				case 2:
					{
						SDL_Rect r;

						r.x=dc->x1+rendering_vp.x-viewx;
						r.y=dc->y1+rendering_vp.y-viewy;
						r.w=dc->x2-dc->x1;
						r.h=dc->y2-dc->y1;
						if (dc->x2>dc->x1 && dc->y2>dc->y1) {
							SDL_FillRect(rendering_sfc,&r,SDL_MapRGBA(rendering_sfc->format,dc->r,dc->g,dc->b,dc->a));
						} /* if */ 
					}
					break;
				case 3:
					{
						TILE *t;
				
						t=game->active_game->tiles[(dc->x2)%TILECATEGORIES][dc->y2];
						
						SDL_UnlockSurface(rendering_sfc);
						if (t!=0) t->draw(dc->x1+rendering_vp.x-viewx,dc->y1+rendering_vp.y-viewy,sfc);
						SDL_LockSurface(rendering_sfc);
					}
					break;
				} /* switch */ 
			} /* while */ 
			SDL_UnlockSurface(rendering_sfc);
		}

		SDL_SetClipRect(rendering_sfc,&old_clip);
	}

	if (scale==1.0F) {

	} else {
		SDL_Surface *tmp;
		SDL_Rect r;
		if (scale>=2.0) tmp=zoomSurface(rendering_sfc, scale, scale, 0);
				   else tmp=zoomSurface(rendering_sfc, scale, scale, 1);

		r.x=vp.x+(vp.w-tmp->w)/2;
		r.y=vp.y+(vp.h-tmp->h)/2;
		r.w=tmp->w;
		r.h=tmp->h;
		SDL_BlitSurface(tmp,0,sfc,&r);

		SDL_FreeSurface(tmp);
		SDL_FreeSurface(rendering_sfc);		
	} /* if */ 

} /* ActiveINTERFACE::draw */ 



int ActiveINTERFACE::get_variable_value(char *vname,int *ival,char **sval,ActiveOBJECT **oval)
{
	List<VARIABLE> l;
	VARIABLE *v;

	if (strcmp(vname,"interf_focusing_x")==0) {
		*ival=interf_focusing_x;
	} else if (strcmp(vname,"interf_focusing_y")==0) {
		*ival=interf_focusing_y;
	} else {
		l.Instance(interface_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			if (v->cmp(vname)) return v->get(ival,sval,oval);
		} /* while */ 
	} /* if */ 
	return VALUE_NONE;
} /* ActiveINTERFACE::get_variable_value */ 


void ActiveINTERFACE::set_variable_value(char *vname,int value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	if (strcmp(vname,"interf_focusing_x")==0) {
		interf_focusing_x=value;
	} else if (strcmp(vname,"interf_focusing_y")==0) {
		interf_focusing_y=value;
	} else {
		l.Instance(interface_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			if (v->cmp(vname)) {
				v->set(value);
				return;
			} /* if */ 
		} /* while */ 
	} /* if */ 
} /* ActiveINTERFACE::set_variable_value */ 


void ActiveINTERFACE::set_variable_value(char *vname,char *value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(interface_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveINTERFACE::set_variable_value */ 


void ActiveINTERFACE::set_variable_value(char *vname,ActiveOBJECT *value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(interface_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveINTERFACE::set_variable_value */ 


bool ActiveINTERFACE::find_object(ActiveOBJECT *object)
{
        int i;
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;

    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        l.Instance(objects[i]);
        l.Rewind();
        while(l.Iterate(o)) {
                if (o->offspringp(object)) return true;
        } /* while */ 
    } /* for */ 

	return false;
} /* ActiveINTERFACE::find_object */ 
