#include "assert.h"
#include "stdio.h"
#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "activeobject.h"
#include "activeroom.h"
#include "ActiveMAP.h"
#include "activemog2.h"
#include "actionscript.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

ActiveMAP::ActiveMAP(MAP *pmap,ActiveMOG2 *pgame)
{
	game=pgame;
	map=pmap;

	if (pmap->map_action!=0) {
		map_action=new ActionSCRIPT(*pmap->map_action);
		map_action->init();
	} else {
		map_action=0;
	} /* if */ 
} /* ActiveMAP */ 


ActiveMAP::~ActiveMAP(void)
{
	game=0;
	map=0;

	if (map_action!=0) delete map_action;
	map_action=0;
} /* ActiveMAP */ 

/*

bool ActiveMAP::save(FILE *fp);
bool ActiveMAP::load(FILE *fp);

*/ 


bool ActiveMAP::cycle(KEYBOARDSTATE **LVK,int n_LVKs)
{
	int i;
	bool any_active_object;
	List<ActiveROOM> l,rooms_to_delete;
	List<ROOM> *tmp1,*lp;
	List<ActiveOBJECT> l2,objects_to_delete;
	ActiveROOM *r,*r2;
	ActiveOBJECT *o,*o2;
	ROOM *room;
	bool found;
	int x1,x2,y1,y2;
	int ax1,ax2,ay1,ay2;

	/* Lists of new rooms to be added: */ 
	nrooms_object.Delete();
	nrooms_to_add.Delete();

	/* MAP SCRIPT: */ 
	if (map_action!=0 &&
		map_action->cycle(0,0,this,game,0,game->app,0,LOCATION_MAP,LVK,n_LVKs)) {
		delete map_action;
		map_action=0;
	} /* if */ 

	focusingobject_in_room_edge=false;

	l.Instance(active_rooms);
	l.Rewind();

	/* Active rooms cycles: */ 
	while(l.Iterate(r)) {
		r->cycle(LVK,n_LVKs);

		/* Test for objects changing of room: */ 
		any_active_object=false;

		for(i=0;i<MAX_OBJECT_PLANES;i++) {
			l2.Instance(r->nonpassive_objects[i]);
			l2.Rewind();
			while(l2.Iterate(o)) {
				if (!o->terminate && 
					(o->object->multiroom_flag || 
					 o->room->room->horizontal_circularity ||
					 o->room->room->vertical_circularity)) {

					if (o->object->active_flag) any_active_object=true;

					o->get_family_multiroom_bbox(&x1,&y1,&x2,&y2);
					if (x1<0 || y1<0 ||
						x2>r->room->dx*TILESIZE ||
						y2>r->room->dy*TILESIZE) {
						if (game->focusing_object(o)) focusingobject_in_room_edge=true;

						/* Look for active rooms for 'o': */ 
						tmp1=new List<ROOM>;
						o->get_family_multiroom_absbbox(&ax1,&ay1,&ax2,&ay2);
						if (o->object->multiroom_flag) find_rooms_for_object(ax1,ay1,ax2-ax1,ay2-ay1,tmp1);
												  else tmp1->Add(o->room->room);
						nrooms_object.Add(o);
						nrooms_to_add.Add(tmp1);
					} /* if */ 

				} /* if */ 
			} /* while */ 

		} /* for */ 

		if (!any_active_object) rooms_to_delete.Add(r);
	} /* while */ 

	/* Add the new active rooms: */ 
	while(!nrooms_object.EmptyP()) {
		o=nrooms_object.ExtractIni();
		lp=nrooms_to_add.ExtractIni();
		while(!lp->EmptyP()) {
			room=lp->ExtractIni();
	
			/* Test if the active room already exists: */ 
			found=false;
            l.Instance(active_rooms);
			l.Rewind();
			while(!found && l.Iterate(r)) {
				if (r->room==room) {
					r2=r;
					found=true;
				} /* if */ 
			} /* while */ 

            if (!found) {
                l.Instance(inactive_rooms);
                l.Rewind();
                while(!found && l.Iterate(r)) {
                    if (r->room==room) {
                        r2=r;
                        found=true;
                    } /* if */ 
                } /* while */ 
            } /* if */ 

            if (found) {
				List<ActiveOBJECT> found_ol;
				ActiveOBJECT *found_o;
				/* Test if the room was going to be deleted: */ 
				rooms_to_delete.DeleteElement(r2);
                if (inactive_rooms.MemberRefP(r2)) {
                    inactive_rooms.DeleteElement(r2);
                    active_rooms.Add(r2);
                } /* if */ 

				/* Test if there is an instance of the object on that room: */ 
				found=false;
				for(i=0;(!found || r2->room->horizontal_circularity || 
								   r2->room->vertical_circularity) && i<MAX_OBJECT_PLANES;i++) {
					l2.Instance(r2->nonpassive_objects[i]);
					l2.Rewind();
					while((!found || r2->room->horizontal_circularity || 
									 r2->room->vertical_circularity) && l2.Iterate(o2)) {
						if (o->cmp(o2)) {
							found=true;
							found_ol.Add(o2);
							assert(o2->object->cmp(o->object));
						} /* if */ 
					} /* while */ 
				} /* for */ 

				if (!found) {
					int absx,absy;
					/* Create an instance of the object to be added to the room: */ 
					o->absolute_hotspot_coordinates(&absx,&absy);

					o2=new ActiveOBJECT(absx-r2->room->x*TILESIZE,absy-r2->room->y*TILESIZE,o->ID,o->object,r2,0);
					o2->is_instance(o);
					o2->copy_state(o);
					r2->objects[o->get_object_plane()].Add(o2);
					if (!o2->passive()) r2->nonpassive_objects[o->get_object_plane()].Add(o2);
					o2->get_object_plane();
				} else {
					if (r2->room->horizontal_circularity ||
						r2->room->vertical_circularity) {
						int rx,ry,rx2,ry2;
						ActiveOBJECT *o3=0;
						int absx,absy;
						
						rx=r2->room->x*TILESIZE;
						ry=r2->room->y*TILESIZE;
						rx2=(r2->room->x+r2->room->get_dx())*TILESIZE;
						ry2=(r2->room->y+r2->room->get_dy())*TILESIZE;

						found_o=found_ol[0];
						found_o->get_family_multiroom_absbbox(&ax1,&ay1,&ax2,&ay2);
						found_o->absolute_hotspot_coordinates(&absx,&absy);

						if (ax1<rx && r2->room->horizontal_circularity) {
							/* Add the object HORIZONTAL-1 */ 
							found=false;
							found_ol.Rewind();
							while(found_ol.Iterate(o3)) {
								if (o3->horizontal_circularity_count==found_o->horizontal_circularity_count-1) found=true;
							} /* if */ 
							if (!found) {
								o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE+r2->room->get_dx()*TILESIZE,absy-r2->room->y*TILESIZE,found_o->ID,found_o->object,r2,0);
								o3->horizontal_circularity_count=found_o->horizontal_circularity_count-1;
								o3->is_instance(found_o);
								o3->copy_state(found_o);
								r2->objects[found_o->get_object_plane()].Add(o3);
								if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
								o3->get_object_plane();
							} /* if */ 

							if (ay1<ry && r2->room->vertical_circularity) {
								/* Add the object VERTICAL-1 */ 
								found=false;
								found_ol.Rewind();
								while(found_ol.Iterate(o3)) {
									if (o3->vertical_circularity_count==found_o->vertical_circularity_count-1) found=true;
								} /* if */ 
								if (!found) {
									o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE,absy-r2->room->y*TILESIZE+r2->room->get_dy()*TILESIZE,found_o->ID,o->object,r2,0);
									o3->vertical_circularity_count=found_o->vertical_circularity_count-1;
									o3->is_instance(found_o);
									o3->copy_state(found_o);
									r2->objects[found_o->get_object_plane()].Add(o3);
									if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
									o3->get_object_plane();
								} /* if */ 

								/* Add the object HORIZONTAL-1,VERTICAL-1 */ 
								found=false;
								found_ol.Rewind();
								while(found_ol.Iterate(o3)) {
									if (o3->horizontal_circularity_count==found_o->horizontal_circularity_count-1 &&
										o3->vertical_circularity_count==found_o->vertical_circularity_count-1) found=true;
								} /* if */ 
								if (!found) {
									o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE+r2->room->get_dx()*TILESIZE,absy-r2->room->y*TILESIZE+r2->room->get_dy()*TILESIZE,found_o->ID,found_o->object,r2,0);
									o3->horizontal_circularity_count=found_o->horizontal_circularity_count-1;
									o3->vertical_circularity_count=found_o->vertical_circularity_count-1;
									o3->is_instance(found_o);
									o3->copy_state(found_o);
									r2->objects[found_o->get_object_plane()].Add(o3);
									if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
									o3->get_object_plane();
								} /* if */ 
							} else {
								if (ay2>ry2 && r2->room->vertical_circularity) {
									/* Add the object VERTICAL+1 */ 
									found=false;
									found_ol.Rewind();
									while(found_ol.Iterate(o3)) {
										if (o3->vertical_circularity_count==found_o->vertical_circularity_count+1) found=true;
									} /* if */ 
									if (!found) {
										o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE,absy-r2->room->y*TILESIZE-r2->room->get_dy()*TILESIZE,found_o->ID,found_o->object,r2,0);
										o3->vertical_circularity_count=found_o->vertical_circularity_count+1;
										o3->is_instance(found_o);
										o3->copy_state(found_o);
										r2->objects[found_o->get_object_plane()].Add(o3);
										if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
										o3->get_object_plane();
									} /* if */ 

									/* Add the object HORIZONTAL-1,VERTICAL+1 */ 
									found=false;
									found_ol.Rewind();
									while(found_ol.Iterate(o3)) {
										if (o3->horizontal_circularity_count==found_o->horizontal_circularity_count-1 &&
											o3->vertical_circularity_count==found_o->vertical_circularity_count+1) found=true;
									} /* if */ 
									if (!found) {
										o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE+r2->room->get_dx()*TILESIZE,absy-r2->room->y*TILESIZE-r2->room->get_dy()*TILESIZE,found_o->ID,found_o->object,r2,0);
										o3->horizontal_circularity_count=found_o->horizontal_circularity_count-1;
										o3->vertical_circularity_count=found_o->vertical_circularity_count+1;
										o3->is_instance(found_o);
										o3->copy_state(found_o);
										r2->objects[found_o->get_object_plane()].Add(o3);
										if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
										o3->get_object_plane();
									} /* if */ 
								} /* if */ 
							} /* if */ 
						} else {
							if (ax2>rx2 && r2->room->horizontal_circularity) {
								/* Add the object HORIZONTAL+1 */ 
								found=false;
								found_ol.Rewind();
								while(found_ol.Iterate(o3)) {
									if (o3->horizontal_circularity_count==found_o->horizontal_circularity_count+1) found=true;
								} /* if */ 
								if (!found) {
									o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE-r2->room->get_dx()*TILESIZE,absy-r2->room->y*TILESIZE,found_o->ID,found_o->object,r2,0);
									o3->horizontal_circularity_count=found_o->horizontal_circularity_count+1;
									o3->is_instance(found_o);
									o3->copy_state(found_o);
									r2->objects[found_o->get_object_plane()].Add(o3);
									if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
									o3->get_object_plane();
								} /* if */ 
								if (ay1<ry && r2->room->vertical_circularity) {
									/* Add the object VERTICAL-1 */ 
									found=false;
									found_ol.Rewind();
									while(found_ol.Iterate(o3)) {
										if (o3->vertical_circularity_count==found_o->vertical_circularity_count-1) found=true;
									} /* if */ 
									if (!found) {
										o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE,absy-r2->room->y*TILESIZE+r2->room->get_dy()*TILESIZE,found_o->ID,found_o->object,r2,0);
										o3->vertical_circularity_count=found_o->vertical_circularity_count-1;
										o3->is_instance(found_o);
										o3->copy_state(found_o);
										r2->objects[found_o->get_object_plane()].Add(o3);
										if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
										o3->get_object_plane();
									} /* if */ 

									/* Add the object HORIZONTAL+1,VERTICAL-1 */ 
									found=false;
									found_ol.Rewind();
									while(found_ol.Iterate(o3)) {
										if (o3->horizontal_circularity_count==found_o->horizontal_circularity_count+1 &&
											o3->vertical_circularity_count==found_o->vertical_circularity_count-1) found=true;
									} /* if */ 
									if (!found) {
										o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE-r2->room->get_dx()*TILESIZE,absy-r2->room->y*TILESIZE+r2->room->get_dy()*TILESIZE,found_o->ID,found_o->object,r2,0);
										o3->horizontal_circularity_count=found_o->horizontal_circularity_count+1;
										o3->vertical_circularity_count=found_o->vertical_circularity_count-1;
										o3->is_instance(found_o);
										o3->copy_state(found_o);
										r2->objects[found_o->get_object_plane()].Add(o3);
										if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
										o3->get_object_plane();
									} /* if */ 
								} else {
									if (ay2>ry2 && r2->room->vertical_circularity) {
										/* Add the object VERTICAL+1 */ 
										found=false;
										found_ol.Rewind();
										while(found_ol.Iterate(o3)) {
											if (o3->vertical_circularity_count==found_o->vertical_circularity_count+1) found=true;
										} /* if */ 
										if (!found) {
											o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE,absy-r2->room->y*TILESIZE-r2->room->get_dy()*TILESIZE,found_o->ID,found_o->object,r2,0);
											o3->vertical_circularity_count=found_o->vertical_circularity_count+1;
											o3->is_instance(found_o);
											o3->copy_state(found_o);
											r2->objects[found_o->get_object_plane()].Add(o3);
											if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
											o3->get_object_plane();
										} /* if */ 

										/* Add the object HORIZONTAL+1,VERTICAL+1 */ 
										found=false;
										found_ol.Rewind();
										while(found_ol.Iterate(o3)) {
											if (o3->horizontal_circularity_count==found_o->horizontal_circularity_count+1 &&
												o3->vertical_circularity_count==found_o->vertical_circularity_count+1) found=true;
										} /* if */ 
										if (!found) {
											o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE-r2->room->get_dx()*TILESIZE,absy-r2->room->y*TILESIZE-r2->room->get_dy()*TILESIZE,found_o->ID,found_o->object,r2,0);
											o3->horizontal_circularity_count=found_o->horizontal_circularity_count+1;
											o3->vertical_circularity_count=found_o->vertical_circularity_count+1;
											o3->is_instance(found_o);
											o3->copy_state(found_o);
											r2->objects[found_o->get_object_plane()].Add(o3);
											if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
											o3->get_object_plane();
										} /* if */ 
									} /* if */ 
								} /* if */ 
							} else {
								if (ay1<ry && r2->room->vertical_circularity) {
									/* Add the object VERTICAL-1 */ 
									found=false;
									found_ol.Rewind();
									while(found_ol.Iterate(o3)) {
										if (o3->vertical_circularity_count==found_o->vertical_circularity_count-1) found=true;
									} /* if */ 
									if (!found) {
										o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE,absy-r2->room->y*TILESIZE+r2->room->get_dy()*TILESIZE,found_o->ID,found_o->object,r2,0);
										o3->vertical_circularity_count=found_o->vertical_circularity_count-1;
										o3->is_instance(found_o);
										o3->copy_state(found_o);
										r2->objects[found_o->get_object_plane()].Add(o3);
										if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
										o3->get_object_plane();
									} /* if */ 
								} else {
									if (ay2>ry2 && r2->room->vertical_circularity) {
										/* Add the object VERTICAL+1 */ 
										found=false;
										found_ol.Rewind();
										while(found_ol.Iterate(o3)) {
											if (o3->vertical_circularity_count==found_o->vertical_circularity_count+1) found=true;
										} /* if */ 
										if (!found) {
											o3=new ActiveOBJECT(absx-r2->room->x*TILESIZE,absy-r2->room->y*TILESIZE-r2->room->get_dy()*TILESIZE,found_o->ID,found_o->object,r2,0);
											o3->vertical_circularity_count=found_o->vertical_circularity_count+1;
											o3->is_instance(found_o);
											o3->copy_state(found_o);
											r2->objects[found_o->get_object_plane()].Add(o3);
											if (!o3->passive()) r2->nonpassive_objects[found_o->get_object_plane()].Add(o3);
											o3->get_object_plane();
										} /* if */ 
									} else {
										/* nothing at all (couldn't happen) */ 
									} /* if */ 
								} /* if */ 
							} /* if */ 
						} /* if */ 

					} /* if */ 
				} /* if */ 

				while(!found_ol.EmptyP()) found_ol.ExtractIni();
			} else {
                int i;
                bool found;
				r=new ActiveROOM(room,this,game);

				/* Create an instance of the object to be added to the room: */ 
				int absx,absy;

				o->absolute_hotspot_coordinates(&absx,&absy);

				o2=new ActiveOBJECT(absx-r->room->x*TILESIZE,absy-r->room->y*TILESIZE,o->ID,o->object,r,0);
				o2->is_instance(o);
				o2->copy_state(o);
				
                found=false;
                for(i=0;!found && i<MAX_OBJECT_PLANES-1;i++) {
                    if (o->planep(i)) {
                       r->objects[i].Add(o2);
					   if (!o2->passive()) r->nonpassive_objects[i].Add(o2);
                       found=true;
                    } /* if */ 
                } /* for */ 
                if (!found) {
					r->objects[MAX_OBJECT_PLANES-1].Add(o2);
					if (!o2->passive()) r->nonpassive_objects[MAX_OBJECT_PLANES-1].Add(o2);
				} /* if */ 

				active_rooms.Add(r);
			} /* if */ 
		} /* while */ 
		delete lp;
	} /* while */ 

	/* Delete the objects that are not inside the visible part of a room: */ 
	l.Instance(active_rooms);
	l.Rewind();
	while(l.Iterate(r)) {
		int x1,y1,x2,y2;
		for(i=0;i<MAX_OBJECT_PLANES;i++) {
			l2.Instance(r->nonpassive_objects[i]);
			l2.Rewind();
			while(l2.Iterate(o)) {
				o->get_family_multiroom_bbox(&x1,&y1,&x2,&y2);
				if (//o->object->multiroom_flag &&
					!game->focusing_object(o) &&
					(x2<0 ||
					 x1>r->room->dx*TILESIZE ||
					 y2<0 ||
					 y1>r->room->dy*TILESIZE)) {
					objects_to_delete.Add(o);
				} /* if */ 
			} /* while */ 

			while(!objects_to_delete.EmptyP()) {
				o=objects_to_delete.ExtractIni();
				r->objects[i].DeleteElement(o);
				if (!o->passive()) r->nonpassive_objects[i].DeleteElement(o);
				assert(!game->focusing_object(o));
				delete o;
			} /* while */ 
		} /* for */ 
	} /* while */ 


	/* Move all the rooms without an active objet to the inactive rooms list: */ 
	while(!rooms_to_delete.EmptyP()) {
		r=rooms_to_delete.ExtractIni();
		active_rooms.DeleteElement(r);
        inactive_rooms.Add(r);
	} /* while */ 
       

	/* Delete the rooms that have remained inactive for more than 16 cycles: */ 
    l.Instance(inactive_rooms);
    l.Rewind();
	while(l.Iterate(r)) if (r->last_active_cycle<game->get_current_cycle()-16) rooms_to_delete.Add(r);

	while(!rooms_to_delete.EmptyP()) {
		r=rooms_to_delete.ExtractIni();
		inactive_rooms.DeleteElement(r);

		/* Eliminar todos los CHILDs no multiroom de los active objects que estn en la habitacin 'r': */ 
		l.Instance(active_rooms);
		l.Rewind();
		while(l.Iterate(r2)) {
			for(i=0;i<2;i++) {
				l2.Instance(r2->objects[i]);
				l2.Rewind();
				while(l2.Iterate(o)) o->delete_children_in_room(r);
			} /* for */ 
		} /* while */ 

		delete r;
	} /* while */ 

	return true;
} /* ActiveMAP::cycle */ 



ActiveOBJECT *ActiveMAP::find_object(int ID)
{
        int i;
	List<ActiveROOM> l;
	ActiveROOM *r;
	List<ActiveOBJECT> l2;
	ActiveOBJECT *o;

	/* ID = -1 means an object that wasn't placed using the editor. */ 
	if (ID==-1) return 0;

	l.Instance(active_rooms);
	l.Rewind();
	while(l.Iterate(r)) {
        for(i=0;i<MAX_OBJECT_PLANES;i++) {
            l2.Instance(r->objects[i]);
            l2.Rewind();
            while(l2.Iterate(o)) {
				if (o->ID==ID) {
					return o;
				} /* if */ 
            } /* while */ 
        } /* for */ 
	} /* while */ 

	return 0;
} /* ActiveMAP::find_object */ 


bool ActiveMAP::find_object(ActiveOBJECT *object)
{
	List<ActiveROOM> l;
	ActiveROOM *r;

	l.Instance(active_rooms);
	l.Rewind();
	while(l.Iterate(r)) {
		if (r->find_object(object)) return true;
	} /* while */ 

	return false;
} /* ActiveMAP::find_object */ 


void ActiveMAP::init_map(MAP *m)
{
	int i;
	List<ROOM> l;
	ROOM *r;
	List<PLACED_OBJECT> l2;
	PLACED_OBJECT *o;
	bool add_this_room=false;

	map=m;

	map_variables.Delete();
	map_variables.Copy(map->map_variables);


	l.Instance(map->rooms);
	l.Rewind();
	while(l.Iterate(r)) {
		add_this_room=false;

        for(i=0;i<MAX_OBJECT_PLANES;i++) {
            l2.Instance(r->objects[i]);
            l2.Rewind();
            while(l2.Iterate(o)) {
				if (o->object->active_flag) add_this_room=true; 
            } // while 
        } // for 
                
		if (add_this_room) add_room(r);
	} // while 

} /* ActiveMAP::init_map */ 


void ActiveMAP::find_rooms_for_object(int x,int y,int dx,int dy,List<ROOM> *l)
{
	List<ROOM> rl;
	ROOM *r;

	rl.Instance(map->rooms);
	rl.Rewind();
	while(rl.Iterate(r)) {
		if (x+dx>r->x*TILESIZE &&
			x<(r->x+r->dx)*TILESIZE &&
			y+dy>r->y*TILESIZE &&
			y<(r->y+r->dy)*TILESIZE) {
			l->Add(r);
		} else {
			if (r->horizontal_circularity &&
				x+dx>(r->x-r->dx)*TILESIZE &&
				x<(r->x+r->dx*2)*TILESIZE &&
				y+dy>r->y*TILESIZE &&
				y<(r->y+r->dy)*TILESIZE) {
				l->Add(r);
			} else {
				if (r->vertical_circularity &&
					x+dx>r->x*TILESIZE &&
					x<(r->x+r->dx)*TILESIZE &&
					y+dy>(r->y-r->dy)*TILESIZE &&
					y<(r->y+r->dy*2)*TILESIZE) {
					l->Add(r);
				} else {
					if (r->horizontal_circularity &&
						r->vertical_circularity &&
						x+dx>(r->x-r->dx)*TILESIZE &&
						x<(r->x+r->dx*2)*TILESIZE &&
						y+dy>(r->y-r->dy)*TILESIZE &&
						y<(r->y+r->dy*2)*TILESIZE) {
						l->Add(r);
					} /* if */ 
				} /* if */  
			} /* if */ 
		} /* if */ 
	} /* while */ 
} /* ActiveMAP::find_rooms_for_object */ 



ActiveROOM *ActiveMAP::add_room(ROOM *room)
{
	/* Test if the room is actually in the list of active rooms: */ 
	List<ActiveROOM> l;
	ActiveROOM *ar,*r;
	bool found=false;

	l.Instance(active_rooms);
	l.Rewind();
	while(!found && l.Iterate(ar)) {
		if (ar->room==room) {
			r=ar;
			found=true;
		} /* if */ 
	} /* while */ 

	/* If it is not, create a new active room: */ 
	if (!found) {
		r=new ActiveROOM(room,this,game);
		active_rooms.Add(r);
	} /* if */ 

	return r;
} /* ActiveMAP::add_room */ 


int ActiveMAP::object_collision(ActiveOBJECT *o,bool hierarchical)
{
	int ax=0,ay=0;

	o->absolute_hotspot_coordinates(&ax,&ay);
	return object_collision(o,ax,ay,hierarchical);
} /* ActiveMAP::object_collision */ 


int ActiveMAP::object_collision(ActiveOBJECT *o,int ax,int ay,bool hierarchical)
{
	int i;
	int rel_x,rel_y;
	List<ROOM> l;
	List<ActiveROOM> l2;
	ActiveROOM *ar;
	List<ActiveOBJECT> l3;
	ActiveOBJECT *o2;
	ROOM *r;
	bool found;
	int x1,x2,y1,y2;
	int ax1,ax2,ay1,ay2;
	int tmp_res=0;

	rel_x=ax-o->room->room->get_x()*TILESIZE;
	rel_y=ay-o->room->room->get_y()*TILESIZE;

	if (hierarchical) {
		int rax,ray;
		o->absolute_hotspot_coordinates(&rax,&ray);
		o->get_family_absbbox(&ax1,&ay1,&ax2,&ay2);
		ax1+=ax-rax;
		ay1+=ay-ray;
		ax2+=ax-rax;
		ay2+=ay-ray;
		x1=ax1-o->room->room->get_x()*TILESIZE;
		y1=ay1-o->room->room->get_y()*TILESIZE;
		x2=ax2-o->room->room->get_x()*TILESIZE;
		y2=ay2-o->room->room->get_y()*TILESIZE;
	} else {
		int rax,ray;
		o->absolute_hotspot_coordinates(&rax,&ray);
		o->get_absbbox(&ax1,&ay1,&ax2,&ay2);
		ax1+=ax-rax;
		ay1+=ay-ray;
		ax2+=ax-rax;
		ay2+=ay-ray;
		x1=ax1-o->room->room->get_x()*TILESIZE;
		y1=ay1-o->room->room->get_y()*TILESIZE;
		x2=ax2-o->room->room->get_x()*TILESIZE;
		y2=ay2-o->room->room->get_y()*TILESIZE;
	} /* if */ 

	tmp_res|=o->room->room_object_collision(o,rel_x,rel_y,hierarchical);
	if ((tmp_res&CONS_SOLID)!=0) return tmp_res;

	/* Test for other instances of the object in the same room (for circularities): */ 
	{
		List<ActiveOBJECT> oil;
		ActiveOBJECT *oi;

		oil.Instance(o->other_instances);
		oil.Rewind();
		while(oil.Iterate(oi)) {
			if (oi!=o && oi->room==o->room) {
				tmp_res|=oi->room->room_object_collision(oi,oi->x+(rel_x-o->x),oi->y+(rel_y-o->y),hierarchical);
				if ((tmp_res&CONS_SOLID)!=0) return tmp_res;
			} /* if */ 
		} /* while */ 
	}

	if (o->object->multiroom_flag &&
		(x1<0 || y1<0 ||
		 x2>(o->room->room->get_dx()*TILESIZE) ||
		 y2>(o->room->room->get_dy()*TILESIZE))) {
		/* The object has some part outside the room, find other rooms for testing collision: */ 

		find_rooms_for_object(ax1,ay1,ax2-ax1,ay2-ay1,&l);

		/* Test if some part of the object is in an area without room: */ 
		{
			int i,j;
			int rx=o->room->room->get_x()*TILESIZE;
			int ry=o->room->room->get_y()*TILESIZE;
			int rx2=(o->room->room->get_dx()+o->room->room->get_x())*TILESIZE;
			int ry2=(o->room->room->get_dy()+o->room->room->get_y())*TILESIZE;
			SDL_Surface *tmp_sfc=SDL_CreateRGBSurface(SDL_SWSURFACE,(ax2-ax1),(ay2-ay1),32,0,0,0,0);
			SDL_FillRect(tmp_sfc,0,0);
			o->draw_collision_mask(ax-ax1-o->x,ay-ay1-o->y,tmp_sfc,0,CONS_ALL);

			if (o->room->room->horizontal_circularity) {
				rx-=o->room->room->get_dx()*TILESIZE;
				rx2+=o->room->room->get_dx()*TILESIZE*2;
			} /* if */ 
			if (o->room->room->vertical_circularity) {
				ry-=o->room->room->get_dy()*TILESIZE;
				ry2+=o->room->room->get_dy()*TILESIZE*2;
			} /* if */ 

			for(i=ax1;i<ax2;i++) {
				if (ay1<ry || ay2>=ry2 || i<rx || i>=rx2) {
					for(j=ay1;j<ay2;j++) {
						if (getpixel(tmp_sfc,i-ax1,j-ay1)!=0) {
							found=false;
							l.Rewind();
							while(!found && l.Iterate(r)) {
								if (i>=(r->x-(r->horizontal_circularity ? r->get_dx():0))*TILESIZE && 
									i<(r->x+(r->horizontal_circularity ? r->get_dx()*2:r->get_dx()))*TILESIZE &&
									j>=(r->y-(r->vertical_circularity ? r->get_dy():0))*TILESIZE && 
									j<(r->y+(r->vertical_circularity ? r->get_dy()*2:r->get_dy()))*TILESIZE) {

									found=true;
								} /* if */ 
							} /* while */ 

							/* Pixel out of all the rooms: collision! */ 
							if (!found) {
								while(!l.EmptyP()) l.ExtractIni();
								SDL_FreeSurface(tmp_sfc);

								return tmp_res|CONS_SOLID;
							} /* if */ 
						} /* if */ 
					} /* for */ 
				} /* if */ 
			} /* for */ 

			SDL_FreeSurface(tmp_sfc);
		}

	} /* if */ 

	/* Test with multiroom objects of the rooms not found, because they can have changed of room
	   on this cycle, and therefore can collide with the current object:							*/ 
	l2.Instance(active_rooms);
	l2.Rewind();
	while(l2.Iterate(ar)) {
		
		if (ar==o->room) found=true;
				    else found=false;
		l.Rewind();
		while(!found && l.Iterate(r)) {
			if (r==ar->room) found=true;
		} /* while */ 

		if (!found) {
			for(i=0;i<MAX_OBJECT_PLANES;i++) {
				l3.Instance(ar->nonpassive_objects[i]);
				l3.Rewind();
				while(l3.Iterate(o2)) {
					if (o2->object->active_flag &&
						o2->object->multiroom_flag &&
						o2!=o &&
						!o2->other_instances.MemberRefP(o)) {
						if (o2->object->constitution==CONS_SOLID_SURFACE) {
							if (o->collision(rel_x,rel_y,o2,hierarchical)) {
								tmp_res|=CONS_SOLID_SURFACE;						
							} /* if */ 
						} /* if */ 
						if (o2->object->constitution==CONS_SOLID) {
							if (o->collision(rel_x,rel_y,o2,hierarchical)) {
								while(!l.EmptyP()) l.ExtractIni();
								return tmp_res|=CONS_SOLID;							
							} /* if */ 
						} /* if */ 
					} /* if */ 
				} /* while */ 
			} /* for */ 
		} /* if */ 
	} /* while */ 

	while(!l.EmptyP()) {
		r=l.ExtractIni();
		if (o->room->room!=r) {

			found=false;
			l2.Instance(active_rooms);
			l2.Rewind();
			while(!found && l2.Iterate(ar)) {
				if (ar->room==r) {	
					tmp_res|=ar->room_object_collision(o,ax-r->get_x()*TILESIZE,ay-r->get_y()*TILESIZE,hierarchical);
					if ((tmp_res&CONS_SOLID)!=0) {
						while(!l.EmptyP()) l.ExtractIni();
						return tmp_res;
					} /* if */ 
					found=true;
				} /* if */ 
			} /* while */ 
                        
            if (!found) {
                l2.Instance(inactive_rooms);
                l2.Rewind();
                while(!found && l2.Iterate(ar)) {
                    if (ar->room==r) {					
                        tmp_res|=ar->room_object_collision(o,ax-r->get_x()*TILESIZE,ay-r->get_y()*TILESIZE,hierarchical);
						if ((tmp_res&CONS_SOLID)!=0) {
							while(!l.EmptyP()) l.ExtractIni();
							return tmp_res;
                        } /* if */ 
                        found=true;
                    } /* if */ 
                } /* while */ 
            } /* if */ 

			if (!found) {
				/* Create a temporal active room just for testing the collisions: */ 
				ar=new ActiveROOM(r,this,game);
                                inactive_rooms.Add(ar);
                                ar->last_active_cycle=game->get_current_cycle();
                                
				tmp_res|=ar->room_object_collision(o,ax-r->get_x()*TILESIZE,ay-r->get_y()*TILESIZE,hierarchical);
				if ((tmp_res&CONS_SOLID)!=0) {
					while(!l.EmptyP()) l.ExtractIni();
					return tmp_res;
				} /* if */ 
			} /* if */ 
		} /* if */ 
	} /* while */ 

	return tmp_res;
} /* ActiveMAP::object_collision */ 


ActiveOBJECT *ActiveMAP::object_collision2(ActiveOBJECT *o,int ax,int ay,int constitution,ConditionSCRIPT *s,ActiveOBJECT **foreign_object,bool hierarchical)
{
	int i;
	int rel_x,rel_y;
	List<ROOM> l;
	List<ActiveROOM> l2;
	ActiveROOM *ar;
	List<ActiveOBJECT> l3;
	ActiveOBJECT *o2,*tmp;
	ROOM *r;
	bool found;
	int x1,x2,y1,y2;
	int ax1,ax2,ay1,ay2;

	rel_x=ax-o->room->room->get_x()*TILESIZE;
	rel_y=ay-o->room->room->get_y()*TILESIZE;

	if (hierarchical) {
		int rax,ray;
		o->absolute_hotspot_coordinates(&rax,&ray);
		o->get_family_absbbox(&ax1,&ay1,&ax2,&ay2);
		ax1+=ax-rax;
		ay1+=ay-ray;
		ax2+=ax-rax;
		ay2+=ay-ray;
		x1=ax1-o->room->room->get_x()*TILESIZE;
		y1=ay1-o->room->room->get_y()*TILESIZE;
		x2=ax2-o->room->room->get_x()*TILESIZE;
		y2=ay2-o->room->room->get_y()*TILESIZE;
	} else {
		int rax,ray;
		o->absolute_hotspot_coordinates(&rax,&ray);
		o->get_absbbox(&ax1,&ay1,&ax2,&ay2);
		ax1+=ax-rax;
		ay1+=ay-ray;
		ax2+=ax-rax;
		ay2+=ay-ray;
		x1=ax1-o->room->room->get_x()*TILESIZE;
		y1=ay1-o->room->room->get_y()*TILESIZE;
		x2=ax2-o->room->room->get_x()*TILESIZE;
		y2=ay2-o->room->room->get_y()*TILESIZE;
	} /* if */ 

	tmp=o->room->room_object_collision2(o,rel_x,rel_y,constitution,s,foreign_object,hierarchical);
	if (tmp!=0) return tmp;

	/* Test for other instances of the object in the same room (for circularities): */ 
	{
		List<ActiveOBJECT> oil;
		ActiveOBJECT *oi;

		oil.Instance(o->other_instances);
		oil.Rewind();
		while(oil.Iterate(oi)) {
			if (oi!=o && oi->room==o->room) {
				tmp=oi->room->room_object_collision2(oi,oi->x+(rel_x-o->x),oi->y+(rel_y-o->y),constitution,s,foreign_object,hierarchical);
				if (tmp!=0) return tmp;
			} /* if */ 
		} /* while */ 
	}


//	return false;

	if (o->object->multiroom_flag &&
		(x1<0 || y1<0 ||
		 x2>(o->room->room->get_dx()*TILESIZE) ||
		 y2>(o->room->room->get_dy()*TILESIZE))) {
		/* The object has some part outside the room, find other rooms for testing collision: */ 

		find_rooms_for_object(ax1,ay1,ax2-ax1,ay2-ay1,&l);
	} /* if */ 

	/* Test with multiroom objects of the rooms not found, because they can have changed of room
	   on this cycle, and therefore can collide with the current object:							*/ 
	l2.Instance(active_rooms);
	l2.Rewind();
	while(l2.Iterate(ar)) {
		
		if (ar==o->room) found=true;
				    else found=false;
		l.Rewind();
		while(!found && l.Iterate(r)) {
			if (r==ar->room) found=true;
		} /* while */ 

		if (!found) {
			for(i=0;i<MAX_OBJECT_PLANES;i++) {
				l3.Instance(ar->objects[i]);
				l3.Rewind();
				while(l3.Iterate(o2)) {
					if (o2->object->active_flag &&
						o2->object->multiroom_flag &&
						o2!=o &&
						!o2->other_instances.MemberRefP(o)) {

						tmp=o->collision2(rel_x,rel_y,o2,o->object->constitution|CONS_SOLID|CONS_SOLID_SURFACE,constitution,s,foreign_object,hierarchical);
						if (tmp!=0) {
							while(!l.EmptyP()) l.ExtractIni();
							return tmp;
						} /* if */ 

					} /* if */ 
				} /* while */ 
			} /* for */ 
		} /* if */ 
	} /* while */ 

	while(!l.EmptyP()) {
		r=l.ExtractIni();
		if (o->room->room!=r) {

			found=false;
			l2.Instance(active_rooms);
			l2.Rewind();
			while(!found && l2.Iterate(ar)) {
				if (ar->room==r) {					
					tmp=ar->room_object_collision2(o,ax-r->get_x()*TILESIZE,ay-r->get_y()*TILESIZE,constitution,s,foreign_object,hierarchical);
					if (tmp!=0) {
						while(!l.EmptyP()) l.ExtractIni();
						return tmp;
					} /* if */ 
					found=true;
				} /* if */ 
			} /* while */ 
            if (!found) {
                l2.Instance(inactive_rooms);
                l2.Rewind();
                while(!found && l2.Iterate(ar)) {
                    if (ar->room==r) {					
                        tmp=ar->room_object_collision2(o,ax-r->get_x()*TILESIZE,ay-r->get_y()*TILESIZE,constitution,s,foreign_object,hierarchical);
                        if (tmp!=0) {
                            while(!l.EmptyP()) l.ExtractIni();
                            return tmp;
                        } /* if */ 
                        found=true;
                    } /* if */ 
                } /* while */ 
            } /* if */ 

			if (!found) {
				/* Create a temporal active room just for testing the collisions: */ 
				ar=new ActiveROOM(r,this,game);
                inactive_rooms.Add(ar);
                ar->last_active_cycle=game->get_current_cycle();

				tmp=ar->room_object_collision2(o,ax-r->get_x()*TILESIZE,ay-r->get_y()*TILESIZE,constitution,s,foreign_object,hierarchical);
				if (tmp!=0) {
					while(!l.EmptyP()) l.ExtractIni();
					return tmp;
				} /* if */ 
			} /* if */ 
		} /* if */ 
	} /* while */ 

	return 0;
} /* ActiveMAP::object_collision2 */ 


int ActiveMAP::get_variable_value(char *vname,int *ival,char **sval,ActiveOBJECT **oval)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(map_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) return v->get(ival,sval,oval);
	} /* while */ 
	return VALUE_NONE;
} /* ActiveMAP::get_variable_value */ 


void ActiveMAP::set_variable_value(char *vname,int value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(map_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveMAP::set_variable_value */ 


void ActiveMAP::set_variable_value(char *vname,char *value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(map_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveMAP::set_variable_value */ 



void ActiveMAP::set_variable_value(char *vname,ActiveOBJECT *value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(map_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveMAP::set_variable_value */ 




void ActiveMAP::remove_object_references(ActiveOBJECT *o)
{
	int pos=0;
	List<ActiveOBJECT> l1;
	ActiveOBJECT *o1;
	List<ROOM> *l2;

	List<int> to_delete;
	int *p;

	l1.Instance(nrooms_object);
	l1.Rewind();

	while(l1.Iterate(o1)) {
		if (o1==o) {
			to_delete.Add(new int(pos));
		} /* if */ 
		pos++;
	} /* while */ 

	while(!to_delete.EmptyP()) {
		p=to_delete.ExtractIni();
		nrooms_object.DeletePosition(*p);
		l2=nrooms_to_add[*p];
		nrooms_to_add.DeletePosition(*p);
		while(!l2->EmptyP()) l2->ExtractIni();
		delete l2;
		delete p;
	} /* while */ 
} /* ActiveMAP::remove_object_references */ 

