#include "assert.h"

#include "string.h"
#include "math.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_rotozoom.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "List.h"

#include "auxiliar.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "interfaces.h"
#include "tiles.h"
#include "samples.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activeMAP.h"
#include "activemog2.h"
#include "activeinterface.h"
#include "activemog2application.h"

#include "actionscript.h"

#include "randomc.h"


#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif


extern bool SOUND_ENABLED;
extern int surface_flags;

ActiveMOG2::ActiveMOG2(MOG2 *game)
{
	app=0;

	active_game=game;

	game_variables.Delete();
	game_variables.Copy(game->game_variables);

	if (game->game_action!=0) {
		game_action=new ActionSCRIPT(*game->game_action);
	} else {
		game_action=0;
	} /* if */ 

	paused=false;

} /* ActiveMOG2::ActiveMOG2 */ 


ActiveMOG2::~ActiveMOG2(void)
{
	while(!player_objects.EmptyP()) player_objects.ExtractIni();

	active_game=0;

	if (game_action!=0) delete game_action;
	game_action=0;
} /* ActiveMOG2::ActiveMOG2 */ 


ActiveOBJECT *ActiveMOG2::add_player(INTERFACE *interf,ROOM *room,char *character,char *playername,SDL_Rect *vp,float zoom,int LVK_ID)
{
	List<OBJECT> l;
	OBJECT *o,*chosen=0;
	ActiveROOM *r;
	bool found=false;
	char lowerstr1[256],lowerstr2[256];

	/* Test if the room is actually in the list of active rooms: */ 
	r=add_room(room);

	strcpy(lowerstr1,character);
	strlwr(lowerstr1);

	l.Instance(active_game->objects);
	l.Rewind();
	while(!found && l.Iterate(o)) {
		if (o->constitution==CONS_CHARACTER) {
			chosen=o;

			strcpy(lowerstr2,o->name);
			strlwr(lowerstr2);

			if (strcmp(lowerstr1,lowerstr2)==0) found=true;
		} /* if */ 
	} /* while */ 

	if (chosen!=0) {
		/* place character: */ 
		VIEWPORT *v;
		ActiveOBJECT *ao;

		v=new VIEWPORT();
		v->vp=*vp;
		ao=new ActiveOBJECT(0,0,-1,o,r,0);
		r->place_character_in_safe_place(ao);
		if (app!=0) {
			r->objects[app->player_plane].Add(ao);
			r->nonpassive_objects[app->player_plane].Add(ao);
		} else {
			int i=0;
			List<VARIABLE> l;
			VARIABLE *var;

			l.Instance(active_game->application_variables);
			l.Rewind();
			while(l.Iterate(var)) {
				if (var->cmp("player_plane")) {
					var->get(&i,0,0);
				} /* if */ 
			} /* while */ 
			r->objects[i].Add(ao);
			r->nonpassive_objects[i].Add(ao);
		} /* if */ 
		if (interf!=0) v->current_interface=new ActiveINTERFACE(interf,this);
				  else v->current_interface=0;
		v->zoom=zoom;
		ao->LVK_ID=LVK_ID;
		viewports.Add(v);

		player_objects.Add(ao);
		{
			char *tmp=0;
			if (playername!=0) {
				tmp=new char[strlen(playername)+1];
				strcpy(tmp,playername);
			} /* if */ 
			player_names.Add(tmp);
		}

		return ao;
	} /* if */ 

	return 0;
} /* ActiveMOG2::add_player */ 


ActiveOBJECT *ActiveMOG2::add_player(INTERFACE *interf,ROOM *room,char *character,char *playername,int x,int y,SDL_Rect *vp,float zoom,int LVK_ID)
{
	ActiveOBJECT *o;
	o=add_player(interf,room,character,playername,vp,zoom,LVK_ID);

	if (o!=0) {
		o->x=x;
		o->y=y;
	} /* if */ 

	return o;
} /* ActiveMOG2::add_player */ 



void ActiveMOG2::init_map(MAP *m)
{
	List<ActiveMAP> l;
	ActiveMAP *am;
	bool found=false;

	l.Instance(active_maps);
	l.Rewind();
	while(!found && l.Iterate(am)) {
		if (am->map==m) found=true;
	} /* while */ 

	if (!found) {
		am=new ActiveMAP(m,this);
		active_maps.Add(am);
	} /* if */ 

	am->init_map(m);
	
} /* ActiveMOG2::init_map */ 



bool ActiveMOG2::cycle(KEYBOARDSTATE **LVK,int n_LVKs)
{
	int i;
	List<ActiveMAP> l;
	ActiveMAP *m;
	List<ActiveOBJECT> l3;
	List<int> todelete;
	ActiveOBJECT *o;
	List<VIEWPORT> lvp;
	VIEWPORT *v;

	if (!paused) {
		/* GAME SCRIPT: */ 
		if (game_action!=0 &&
			game_action->cycle(0,0,0,this,0,app,0,LOCATION_GAME,LVK,n_LVKs)) {
			delete game_action;
			game_action=0;
		} /* if */ 

		l.Instance(active_maps);
		l.Rewind();
		while(l.Iterate(m)) {
			deleted_objects=false;
			m->cycle(LVK,n_LVKs);

			/* Check if the focusingroom has to be changed: */ 
			if (m->focusingobject_in_room_edge || deleted_objects) {

				for(i=0;i<player_objects.Length();i++) {
					if (player_objects[i]!=0 &&
						player_objects[i]->get_map()==m) {
						o=find_activeinstance(player_objects[i],m);
						if (o!=0) {
							player_objects.GoTo(i);
							player_objects.SetObj(o);
						} else {
							todelete.Add(new int(i));
						} /* if */ 
					} /* if */ 
				} /* for */ 
				while(!todelete.EmptyP()) {
					int *i;
					i=todelete.ExtractIni();
					player_objects.DeletePosition(*i);
					player_names.DeletePosition(*i);
					delete i;
				} /* if */ 
		
				lvp.Instance(viewports);
				lvp.Rewind();
				while(lvp.Iterate(v)) {
					if (v->current_interface!=0) {

						for(i=0;i<MAX_OBJECT_PLANES;i++) {
							l3.Instance(v->current_interface->objects[i]);
							l3.Rewind();
							while(l3.Iterate(o)) {
								if (o->object->control==CTRL_GAMEWINDOW) {
									int i;
									for(i=0;i<o->gw_focusing.Length();i++) {
										o->gw_focusing.GoTo(i);
										o->gw_focusing.SetObj(find_activeinstance(o->gw_focusing[i],o->gw_focusing[i]->room->map));
									} /* for */ 
								} /* if */ 
							} /* while */ 
						} /* for */ 
					} /* if */ 
				} /* while */ 

			} /* if */ 
		} /* while */ 
	} /* if */ 

	/* The interfaces are executed even when the game is paused: */ 
	{
		List<VIEWPORT> l;
		VIEWPORT *v;

		l.Instance(viewports);
		l.Rewind();
		while(l.Iterate(v)) if (v->current_interface!=0) v->current_interface->cycle(LVK,n_LVKs);
	}

	return true;
} /* ActiveMOG2::cycle */ 


ActiveOBJECT *ActiveMOG2::find_activeinstance(ActiveOBJECT *o,ActiveMAP *m) 
{
	List<ActiveOBJECT> l3;
	ActiveOBJECT *o2;

	if (m->find_object(o)) {
		if (o->x>=0 && o->y>=0 &&
			o->x<o->room->room->get_dx()*TILESIZE &&
			o->y<o->room->room->get_dy()*TILESIZE) {
			return o;
		} /* if */ 

		l3.Instance(o->other_instances);
		l3.Rewind();
		while(l3.Iterate(o2)) {
			if (o2->x>=0 && o2->y>=0 &&
				o2->x<o2->room->room->get_dx()*TILESIZE &&
				o2->y<o2->room->room->get_dy()*TILESIZE) {
				return o2;
			} /* if */ 
		} /* while */ 

	} /* if */ 

	return 0;
} /* ActiveMOG2::find_activeinstance */ 


void ActiveMOG2::remove_focusing_references(ActiveOBJECT *o)
{
	int i;
	List<ActiveOBJECT> l3;
	ActiveOBJECT *o2;
	List<VIEWPORT> lvp;
	VIEWPORT *v;

	lvp.Instance(viewports);
	lvp.Rewind();
	while(lvp.Iterate(v)) {

		if (v->current_interface!=0) {
			for(i=0;i<MAX_OBJECT_PLANES;i++) {
				l3.Instance(v->current_interface->objects[i]);
				l3.Rewind();
				while(l3.Iterate(o2)) {
					if (o2->object->control==CTRL_GAMEWINDOW) {
						while(o2->gw_focusing.MemberRefP(o)) o2->gw_focusing.DeleteElement(o);
					} /* if */ 
				} /* while */ 
			} /* for */ 
		} /* if */ 
	} /* while */ 

	{
		int i;

		i=player_objects.PositionRef(o);
		if (i!=-1) {
			player_objects.DeletePosition(i);
			player_names.DeletePosition(i);
		} /* if */ 
	}
} /* ActiveMOG2::remove_focusing_references */ 


void ActiveMOG2::draw(SDL_Surface *sfc)
{
	List<VIEWPORT> l;
	VIEWPORT *v;

	l.Instance(viewports);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->vp.w!=0 && v->vp.h!=0) {
			SDL_SetClipRect(sfc,&v->vp);
			if (v->zoom==1.0) {
				v->last_x=v->vp.x;
				v->last_y=v->vp.y;
				if (v->current_interface!=0) {
					v->current_interface->draw(sfc,v->vp);
				} /* if */ 

			} else {
				SDL_Rect r,r2;
				SDL_Surface *tmp=0,*tmp2=0;
				r.x=0;
				r.y=0;
				r.w=int(v->vp.w/v->zoom);
				r.h=int(v->vp.h/v->zoom);

				if (v->current_interface!=0) {
					if (r.w>v->current_interface->interf->dx) r.w=v->current_interface->interf->dx;
					if (r.h>v->current_interface->interf->dy) r.h=v->current_interface->interf->dy;

					tmp=SDL_CreateRGBSurface(surface_flags,r.w,r.h,32,0,0,0,0);
					SDL_FillRect(tmp,0,0);
					v->current_interface->draw(tmp,r);
				} /* if */ 

				if (tmp!=0) {
					if (v->zoom>=2.0) tmp2=zoomSurface(tmp, v->zoom, v->zoom, 0);
								 else tmp2=zoomSurface(tmp, v->zoom, v->zoom, 1);

					v->last_x=r2.x=v->vp.x+v->vp.w/2-tmp2->w/2;
					v->last_y=r2.y=v->vp.y+v->vp.h/2-tmp2->h/2;
					r2.w=tmp2->w;
					r2.h=tmp2->h;
					SDL_BlitSurface(tmp2,0,sfc,&r2);

					SDL_FreeSurface(tmp2);
					SDL_FreeSurface(tmp);
				} /* if */ 
			} /* if */ 
		} /* if */ 
	} /* while */ 

	SDL_SetClipRect(sfc,0);

} /* ActiveMOG2::draw */ 


bool ActiveMOG2::focusing_object(ActiveOBJECT *o)
{
	if (player_objects.MemberRefP(o)) return true;
	return false;
} /* ActiveMOG2::focusing_object */ 


ActiveOBJECT *ActiveMOG2::find_object(int ID)
{
	List<ActiveMAP> l;
	ActiveMAP *m;
	ActiveOBJECT *o;

	l.Instance(active_maps);
	l.Rewind();
	while(l.Iterate(m)) {
		o=m->find_object(ID);
		if (o!=0) return o;
	} /* while */ 

	return 0;
} /* ActiveMOG2::find_object */ 


bool ActiveMOG2::find_object(ActiveOBJECT *object)
{
	List<ActiveMAP> l;
	ActiveMAP *m;
	List<VIEWPORT> l2;
	VIEWPORT *v;
	List<ActiveOBJECT> lo;
	ActiveOBJECT *o;

	lo.Instance(player_objects);
	lo.Rewind();
	while(lo.Iterate(o)) {
		if (o->offspringp(object)) return true;
	} /* while */  

	l.Instance(active_maps);
	l.Rewind();
	while(l.Iterate(m)) {
		if (m->find_object(object)) return true;
	} /* while */ 

	l2.Instance(viewports);
	l2.Rewind();
	while(l2.Iterate(v)) {
		if (v->current_interface!=0 &&
			v->current_interface->find_object(object)) return true;
	} /* while */ 

	return false;
} /* ActiveMOG2::find_object */ 


ActiveROOM *ActiveMOG2::add_room(ROOM *room)
{
	/* Test if the map is actually in the list of active maps: */ 
	List<ActiveMAP> l;
	ActiveMAP *m;
	bool found=false;

	l.Instance(active_maps);
	l.Rewind();
	while(!found && l.Iterate(m)) {
		if (m->map==room->map) found=true;
	} /* while */ 

	if (!found) {
		m=new ActiveMAP(room->map,this);
		active_maps.Add(m);
		m->init_map(room->map);
	} /* if */ 

	return m->add_room(room);
} /* ActiveMOG2::add_room */ 

 

OBJECT *ActiveMOG2::find_object(char *name)
{
	if (active_game!=0) return active_game->find_object(name);
	return 0;
} /* ActiveMOG2::find_object */ 


int ActiveMOG2::get_variable_value(char *vname,int *ival,char **sval,ActiveOBJECT **oval)
{
	List<VARIABLE> l;
	VARIABLE *v;

	if (strcmp(vname,"game_cycle")==0) {
		*ival=get_current_cycle();
		return VALUE_INTEGER;
	} else {
		l.Instance(game_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			if (v->cmp(vname)) return v->get(ival,sval,oval);
		} /* while */ 
	} /* if */ 
	return VALUE_NONE;
} /* ActiveMOG2::get_variable_value */ 


void ActiveMOG2::set_variable_value(char *vname,int value)
{
	List<VARIABLE> l;
	VARIABLE *v;


	l.Instance(game_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 

} /* ActiveMOG2::set_variable_value */ 


void ActiveMOG2::set_variable_value(char *vname,char *value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(game_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveMOG2::set_variable_value */ 


void ActiveMOG2::set_variable_value(char *vname,ActiveOBJECT *value)
{
	List<VARIABLE> l;
	VARIABLE *v;


	l.Instance(game_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 

} /* ActiveMOG2::set_variable_value */ 


bool ActiveMOG2::hypothetic_cycle(KEYBOARDSTATE **LVK,int n_LVKs,int cycle_num)
{
	return cycle(LVK,n_LVKs);
} /* ActiveMOG2::hypothetic_cycle */ 


bool ActiveMOG2::hypothetic_test_and_fix(KEYBOARDSTATE **LVK,int n_LVKs,int cycle_num)
{
	return true;
} /* ActiveMOG2::hypothetic_test_and_fix */ 



void ActiveMOG2::reset(void)
{
	game_variables.Delete();
	game_variables.Copy(active_game->game_variables);

	delete game_action;
	if (active_game->game_action!=0) {
		game_action=new ActionSCRIPT(*active_game->game_action);
	} else {
		game_action=0;
	} /* if */ 

	while(!player_objects.EmptyP()) {
		player_objects.ExtractIni();
	} /* while */ 
	player_names.Delete();
	active_maps.Delete();

} /* ActiveMOG2::reset */ 


int ActiveMOG2::get_current_cycle(void) 
{
	if (app!=0) return app->get_current_cycle(); 
		   else return 0;
} /* ActiveMOG2::get_current_cycle */ 


void ActiveMOG2::add_sample(PLAYSAMPLE *s)
{
	if (app!=0) app->play_samples.Add(s);
		   else delete s;
} /* ACtiveMOG2::add_sample */ 