#include "stdio.h"
#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "List.h"

#include "collision.h"
#include "auxiliar.h"
#include "samples.h"
#include "animscript.h"
#include "valuescript.h"
#include "conditionscript.h"
#include "actionscript.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activeMAP.h"
#include "activemog2.h"
#include "ActiveINTERFACE.h"
#include "ActiveMOG2Application.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

ConditionSCRIPT::ConditionSCRIPT(void)
{
	int i;

	command=CONDITION_CMD_FALSE;
	timmer=0;
	for(i=0;i<MAX_CONDITION_PARAMETERS;i++) vpar[i]=0;

	dx=0;
	dy=0;
} /* ConditionSCRIPT::ConditionSCRIPT */ 


ConditionSCRIPT::ConditionSCRIPT(ConditionSCRIPT &s)
{
	int i;

	command=s.command;
	timmer=s.timmer;
	for(i=0;i<MAX_CONDITION_PARAMETERS;i++) {
		vpar[i]=0;
		if (s.vpar[i]!=0) vpar[i]=new ValueSCRIPT(*s.vpar[i]);
	} /* for */ 
	aparlist.Copy(s.aparlist);

	dx=0;
	dy=0;
} /* ConditionSCRIPT::ConditionSCRIPT */ 


ConditionSCRIPT::~ConditionSCRIPT(void)
{
	int i;
	for(i=0;i<MAX_CONDITION_PARAMETERS;i++) {
		delete vpar[i];
		vpar[i]=0;
	} /* for */ 
} /* ConditionSCRIPT::~ConditionSCRIPT */ 


char *ConditionSCRIPT::generate_string(void)
{
	char *str=0;

	switch(command) {

	case CONDITION_CMD_TRUE:
		str=new char[strlen("TRUE")+1];
		sprintf(str,"TRUE");
		break;
	case CONDITION_CMD_FALSE:
		str=new char[strlen("FALSE")+1];
		sprintf(str,"FALSE");
		break;

	case CONDITION_CMD_NOT:
		{
			char *s;
			int len=0;

			s=aparlist[0]->generate_string();
			len+=strlen(s);
			len+=strlen("NOT (  )")+1;

			str=new char[len];

			sprintf(str,"NOT ( %s )",s);
			delete []s;
		}
		break;
	case CONDITION_CMD_AND:
		{
			List<ConditionSCRIPT> l;
			ConditionSCRIPT *as;
			List<char> ls;
			char *s;
			int len=0,pos;

			l.Instance(aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				s=as->generate_string();
				ls.Add(s);
				len+=strlen(s);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length()*3;
			} /* if */ 
			len+=strlen("AND (  )")+1;

			str=new char[len];

			sprintf(str,"AND ( ");
			pos=strlen("AND ( ");

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(str+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(str+pos," , ");
					pos+=3;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(str+pos," )");
		}
		break;
	case CONDITION_CMD_OR:
		{
			List<ConditionSCRIPT> l;
			ConditionSCRIPT *as;
			List<char> ls;
			char *s;
			int len=0,pos;

			l.Instance(aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				s=as->generate_string();
				ls.Add(s);
				len+=strlen(s);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length()*3;
			} /* if */ 
			len+=strlen("OR (  )")+1;

			str=new char[len];

			sprintf(str,"OR ( ");
			pos=strlen("OR ( ");

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(str+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(str+pos," , ");
					pos+=3;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(str+pos," )");
		}
		break;

	case CONDITION_CMD_TIMMER:
		{
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("TIMMER (  )")+len+1];
			sprintf(str,"TIMMER ( %s )",s);

			delete []s;
		}
		break;
	case CONDITION_CMD_COLLISION:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("COLLISION (  ,  )")+len+1];
			sprintf(str,"COLLISION ( %s , %s )",s,s2);

			delete []s;
			delete []s2;
		}
		break;
	case CONDITION_CMD_COLLISION_WITH_OBJECT:
		{
			char *s,*s2,*s3,*s4;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			s4=aparlist[0]->generate_string();
			len+=strlen(s4);

			str=new char[strlen("COLLISION_WITH_OBJECT (  ,  ,  ,  )")+len+1];
			sprintf(str,"COLLISION_WITH_OBJECT ( %s , %s , %s , %s )",s,s2,s3,s4);

			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
		}
		break;
	case CONDITION_CMD_INSIDE_ROOM:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("INSIDE_ROOM (  ,  )")+len+1];
			sprintf(str,"INSIDE_ROOM ( %s , %s )",s,s2);

			delete []s;
			delete []s2;
		}
		break;

	case CONDITION_CMD_PARTIALLY_INSIDE_ROOM:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("PARTIALLY_INSIDE_ROOM (  ,  )")+len+1];
			sprintf(str,"PARTIALLY_INSIDE_ROOM ( %s , %s )",s,s2);

			delete []s;
			delete []s2;
		}
		break;

	case CONDITION_CMD_H_COLLISION:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("H_COLLISION (  ,  )")+len+1];
			sprintf(str,"H_COLLISION ( %s , %s )",s,s2);

			delete []s;
			delete []s2;
		}
		break;
	case CONDITION_CMD_H_COLLISION_WITH_OBJECT:
		{
			char *s,*s2,*s3,*s4;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			s4=aparlist[0]->generate_string();
			len+=strlen(s4);

			str=new char[strlen("H_COLLISION_WITH_OBJECT (  ,  ,  ,  )")+len+1];
			sprintf(str,"H_COLLISION_WITH_OBJECT ( %s , %s , %s , %s )",s,s2,s3,s4);

			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
		}
		break;
	case CONDITION_CMD_H_INSIDE_ROOM:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("H_INSIDE_ROOM (  ,  )")+len+1];
			sprintf(str,"H_INSIDE_ROOM ( %s , %s )",s,s2);

			delete []s;
			delete []s2;
		}
		break;

	case CONDITION_CMD_H_PARTIALLY_INSIDE_ROOM:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("H_PARTIALLY_INSIDE_ROOM (  ,  )")+len+1];
			sprintf(str,"H_PARTIALLY_INSIDE_ROOM ( %s , %s )",s,s2);

			delete []s;
			delete []s2;
		}
		break;


	case CONDITION_CMD_SENSE_OBJECT:
		{
			char *s,*s2,*s3,*s4,*s5,*s6;
			int len=0;

			s=vpar[0]->generate_string();
			s2=vpar[1]->generate_string();
			s3=vpar[2]->generate_string();
			s4=vpar[3]->generate_string();
			s5=vpar[4]->generate_string();
			s6=aparlist[0]->generate_string();
			len+=strlen(s)+strlen(s2)+strlen(s3)+strlen(s4)+strlen(s5)+strlen(s6);

			str=new char[strlen("SENSE_OBJECT (  ,  ,  ,  ,  ,  )")+len+1];
			sprintf(str,"SENSE_OBJECT ( %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6);

			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
		}
		break;

	case CONDITION_CMD_SENSE_SPECIFIC_OBJECT:
		{
			char *s,*s2,*s3,*s4,*s5,*s6;
			int len=0;

			s=vpar[0]->generate_string();
			s2=vpar[1]->generate_string();
			s3=vpar[2]->generate_string();
			s4=vpar[3]->generate_string();
			s5=vpar[4]->generate_string();
			s6=aparlist[0]->generate_string();
			len+=strlen(s)+strlen(s2)+strlen(s3)+strlen(s4)+strlen(s5)+strlen(s6);

			str=new char[strlen("SENSE_SPECIFIC_OBJECT (  ,  ,  ,  ,  ,  )")+len+1];
			sprintf(str,"SENSE_SPECIFIC_OBJECT ( %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6);

			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
		}
		break;

	case CONDITION_CMD_KEYPRESS:
		{
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("KEYPRESS (  )")+len+1];
			sprintf(str,"KEYPRESS ( %s )",s);

			delete []s;
		}
		break;
	case CONDITION_CMD_KEYPRESSED:
		{
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("KEYPRESSED (  )")+len+1];
			sprintf(str,"KEYPRESSED ( %s )",s);

			delete []s;
		}
		break;

	case CONDITION_CMD_VARIABLE_EQ:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("TEST  = ")+len+1];
			sprintf(str,"TEST %s = %s",s,s2);

			delete []s;
			delete []s2;
		}
		break;
	case CONDITION_CMD_VARIABLE_GR:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("TEST  > ")+len+1];
			sprintf(str,"TEST %s > %s",s,s2);

			delete []s;
			delete []s2;
		}
		break;
	case CONDITION_CMD_VARIABLE_GREQ:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("TEST  >= ")+len+1];
			sprintf(str,"TEST %s >= %s",s,s2);

			delete []s;
			delete []s2;
		}
		break;
	case CONDITION_CMD_VARIABLE_LW:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("TEST  < ")+len+1];
			sprintf(str,"TEST %s < %s",s,s2);

			delete []s;
			delete []s2;
		}
		break;
	case CONDITION_CMD_VARIABLE_LWEQ:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("TEST  <= ")+len+1];
			sprintf(str,"TEST %s <= %s",s,s2);

			delete []s;
			delete []s2;
		}
		break;
	case CONDITION_CMD_FIND_SUBSTRING:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("FIND_SUBSTRING (  ,  )")+len+1];
			sprintf(str,"FIND_SUBSTRING ( %s , %s )",s,s2);

			delete []s;
			delete []s2;
		}
		break;

	case CONDITION_CMD_INVINCIBLE:
		{
			str=new char[strlen("INVINCIBLE")+1];
			sprintf(str,"INVINCIBLE");
		}
		break;

	case CONDITION_CMD_INLADDER:
		{
			str=new char[strlen("INLADDER")+1];
			sprintf(str,"INLADDER");
		}
		break;

	case CONDITION_CMD_LOADGAME:
		{
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("LOADGAME (  )")+len+1];
			sprintf(str,"LOADGAME ( %s )",s);

			delete []s;
		}
		break;

	case CONDITION_CMD_SAVEGAME:
		{
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("SAVEGAME (  )")+len+1];
			sprintf(str,"SAVEGAME ( %s )",s);

			delete []s;
		}
		break;

	case CONDITION_CMD_VALID_OBJECT:
		{
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("VALID_OBJECT (  )")+len+1];
			sprintf(str,"VALID_OBJECT ( %s )",s);

			delete []s;
		}
		break;

	} /* switch */ 

	return str;
} /* ConditionSCRIPT::generate_string */ 


bool ConditionSCRIPT::load(FILE *fp,MOG2 *game)
{
	char tmp[80];

	if (1!=fscanf(fp,"%s",tmp)) return false;

	aparlist.Delete();

	if (strcmp(tmp,"TRUE")==0) {
		command=CONDITION_CMD_TRUE;

	} else if (strcmp(tmp,"FALSE")==0) {
		command=CONDITION_CMD_FALSE;

	} else if (strcmp(tmp,"NOT")==0) {
		command=CONDITION_CMD_NOT;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		aparlist.Add(new ConditionSCRIPT());
		if (!aparlist[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"AND")==0) {
		bool end=false;
		command=CONDITION_CMD_AND;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		while(!end) {
			ConditionSCRIPT *as;
			as=new ConditionSCRIPT();
			if (as->load(fp,game)) {
				aparlist.Add(as);
				if (1!=fscanf(fp,"%s",tmp)) return false;
				if (strcmp(tmp,")")==0) end=true;
			} else {
				end=true;
			} /* if */ 
		} /* while */ 

	} else if (strcmp(tmp,"OR")==0) {
		bool end=false;
		command=CONDITION_CMD_OR;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		while(!end) {
			ConditionSCRIPT *as;
			as=new ConditionSCRIPT();
			if (as->load(fp,game)) {
				aparlist.Add(as);
				if (1!=fscanf(fp,"%s",tmp)) return false;
				if (strcmp(tmp,")")==0) end=true;
			} else {
				end=true;
			} /* if */ 
		} /* while */ 

	} else if (strcmp(tmp,"TIMMER")==0) {
		command=CONDITION_CMD_TIMMER;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"COLLISION")==0) {
		command=CONDITION_CMD_COLLISION;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"COLLISION_WITH_OBJECT")==0) {
		command=CONDITION_CMD_COLLISION_WITH_OBJECT;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		aparlist.Add(new ConditionSCRIPT());
		if (!aparlist[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"INSIDE_ROOM")==0) {
		command=CONDITION_CMD_INSIDE_ROOM;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"PARTIALLY_INSIDE_ROOM")==0) {
		command=CONDITION_CMD_PARTIALLY_INSIDE_ROOM;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"H_COLLISION")==0) {
		command=CONDITION_CMD_H_COLLISION;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"H_COLLISION_WITH_OBJECT")==0) {
		command=CONDITION_CMD_H_COLLISION_WITH_OBJECT;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		aparlist.Add(new ConditionSCRIPT());
		if (!aparlist[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"H_INSIDE_ROOM")==0) {
		command=CONDITION_CMD_H_INSIDE_ROOM;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"H_PARTIALLY_INSIDE_ROOM")==0) {
		command=CONDITION_CMD_H_PARTIALLY_INSIDE_ROOM;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"SENSE_OBJECT")==0) {
		command=CONDITION_CMD_SENSE_OBJECT;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[4];
		vpar[4]=new ValueSCRIPT();
		if (!vpar[4]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		aparlist.Add(new ConditionSCRIPT());
		if (!aparlist[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;


	} else if (strcmp(tmp,"SENSE_SPECIFIC_OBJECT")==0) {
		command=CONDITION_CMD_SENSE_SPECIFIC_OBJECT;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[4];
		vpar[4]=new ValueSCRIPT();
		if (!vpar[4]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		aparlist.Add(new ConditionSCRIPT());
		if (!aparlist[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"KEYPRESS")==0) {
		command=CONDITION_CMD_KEYPRESS;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"KEYPRESSED")==0) {
		command=CONDITION_CMD_KEYPRESSED;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"TEST")==0) {
		command=CONDITION_CMD_TRUE;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (strcmp(tmp,"=")==0) command=CONDITION_CMD_VARIABLE_EQ;
		if (strcmp(tmp,">")==0) command=CONDITION_CMD_VARIABLE_GR;
		if (strcmp(tmp,">=")==0) command=CONDITION_CMD_VARIABLE_GREQ;
		if (strcmp(tmp,"<")==0) command=CONDITION_CMD_VARIABLE_LW;
		if (strcmp(tmp,"<=")==0) command=CONDITION_CMD_VARIABLE_LWEQ;
		if (command==CONDITION_CMD_TRUE) {
			delete vpar[0];
			vpar[0]=0;
			delete vpar[1];
			vpar[1]=0;
		} /* if */ 

	} else if (strcmp(tmp,"FIND_SUBSTRING")==0) {
		command=CONDITION_CMD_FIND_SUBSTRING;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"INVINCIBLE")==0) {
		command=CONDITION_CMD_INVINCIBLE;

	} else if (strcmp(tmp,"INLADDER")==0) {
		command=CONDITION_CMD_INLADDER;

	} else if (strcmp(tmp,"LOADGAME")==0) {
		command=CONDITION_CMD_LOADGAME;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"SAVEGAME")==0) {
		command=CONDITION_CMD_SAVEGAME;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"VALID_OBJECT")==0) {
		command=CONDITION_CMD_VALID_OBJECT;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	}  else {
		return false;
	} /* if */ 
	return true;
} /* ConditionSCRIPT::load */ 


bool ConditionSCRIPT::save(FILE *fp)
{
	char *str;

	str=generate_string();

	fprintf(fp,"%s\n",str);

	delete []str;
	return true;
} /* ConditionSCRIPT::save */ 




void ConditionSCRIPT::init(void)
{
	aparlist.Rewind();

	switch(command) {

	case CONDITION_CMD_NOT:
		aparlist[0]->init();
		break;
	case CONDITION_CMD_AND:
		{
			List<ConditionSCRIPT> l;
			ConditionSCRIPT *cs;

			l.Instance(aparlist);
			l.Rewind();
			while(l.Iterate(cs)) cs->init();
		}
		break;
	case CONDITION_CMD_OR:
		{
			List<ConditionSCRIPT> l;
			ConditionSCRIPT *cs;

			l.Instance(aparlist);
			l.Rewind();
			while(l.Iterate(cs)) cs->init();
		}
		break;

	case CONDITION_CMD_TIMMER:
		timmer=0;
		break;
	} /* switch */ 
} /* ConditionSCRIPT::init */ 


bool ConditionSCRIPT::cycle(ActiveOBJECT *obj,ActiveROOM *room,ActiveMAP *map,ActiveMOG2 *game,ActiveINTERFACE *interf,ActiveMOG2Application *app,ActiveOBJECT **foreign_object,int location,KEYBOARDSTATE **LVK,int n_LVKs)
{
	switch(command) {

	case CONDITION_CMD_TRUE:
		return true;
		break;
	case CONDITION_CMD_FALSE:
		return false;
		break;

	case CONDITION_CMD_NOT:
		if (aparlist[0]->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) return false;
		return true;
		break;
	case CONDITION_CMD_AND:
		{
			List<ConditionSCRIPT> l;
			ConditionSCRIPT *cs;

			l.Instance(aparlist);
			l.Rewind();
			while(l.Iterate(cs)) 
				if (!cs->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) return false;

			return true;
		}
		break;
	case CONDITION_CMD_OR:
		{
			List<ConditionSCRIPT> l;
			ConditionSCRIPT *cs;

			l.Instance(aparlist);
			l.Rewind();
			while(l.Iterate(cs)) 
				if (cs->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) return true;

			return false;
		}
		break;

	case CONDITION_CMD_TIMMER:
		int itmp;

		if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
			if (timmer>=itmp) return true;
			timmer++;
			return false;		
		} /* if */ 
		break;
	case CONDITION_CMD_COLLISION:
		{
			int itmp,itmp2;

			if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				int tmpx,tmpy;
				int ax,ay;
				int tmp_res=0,tmp_res2=0;
				tmpx=(obj->precision_x+itmp)/100;
				tmpy=(obj->precision_y+itmp2)/100;
				
//				ax=obj->old_x+obj->room->room->x*TILESIZE;
//				ay=obj->old_y+obj->room->room->y*TILESIZE;
				ax=obj->x+obj->room->room->x*TILESIZE;
				ay=obj->y+obj->room->room->y*TILESIZE;
				
				tmp_res=map->object_collision(obj,ax+tmpx,ay+tmpy,false);
				if ((tmp_res&CONS_SOLID)!=0) return true;
				if (tmp_res==0) return false;
				if (tmpy<=0) return false;
				tmp_res2=(map->object_collision(obj,ax,ay,false))&CONS_SOLID_SURFACE;
				if (tmp_res2==0) return true;
				return false;
			} /* if */ 
		}
		break;
	case CONDITION_CMD_COLLISION_WITH_OBJECT:
		{
			char *stmp;
			int itmp,itmp2;

			if (vpar[0]->cycle(0,&stmp,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
				vpar[1]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				int tmpx,tmpy;
				int ax,ay;
				int cons=0;
				tmpx=(obj->precision_x+itmp)/100;
				tmpy=(obj->precision_y+itmp2)/100;
				
//				ax=obj->old_x+obj->room->room->x*TILESIZE;
//				ay=obj->old_y+obj->room->room->y*TILESIZE;
				ax=obj->x+obj->room->room->x*TILESIZE;
				ay=obj->y+obj->room->room->y*TILESIZE;

				if (substr(stmp,"none")) cons|=CONS_NONE;
				if (substr(stmp,"solid")) cons|=CONS_SOLID;
				if (substr(stmp,"solid_sfc")) cons|=CONS_SOLID_SURFACE;
				if (substr(stmp,"weapon")) cons|=CONS_WEAPON;
				if (substr(stmp,"ladder")) cons|=CONS_LADDER;
				if (substr(stmp,"character")) cons|=CONS_CHARACTER;
				if (substr(stmp,"item")) cons|=CONS_ITEM;
				*foreign_object=0;

				if (cons!=0) {
					*foreign_object=map->object_collision2(obj,ax+tmpx,ay+tmpy,cons,aparlist[0],foreign_object,false);
				} /* if */ 
				if (*foreign_object!=0) return true;
							  	   else return false;
			} /* if */ 
		}
		break;
	case CONDITION_CMD_INSIDE_ROOM:
		{
			int itmp,itmp2;

			if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				int x1,y1,x2,y2;
				int tmpx,tmpy;

				tmpx=(obj->precision_x+itmp)/100;
				tmpy=(obj->precision_y+itmp2)/100;
				
				obj->get_bbox(&x1,&y1,&x2,&y2);
				x1+=tmpx;
				y1+=tmpy;
				x2+=tmpx;
				y2+=tmpy;
				if ((!room->room->horizontal_circularity &&
					 (x1<0 ||
					  x2>=room->room->dx*TILESIZE)) ||
					(!room->room->vertical_circularity &&
					 (y1<0 ||
					  y2>=room->room->dy*TILESIZE))) {
					return false;
				} /* if */ 

				return true;
			} /* if */ 
		}
		break;
	case CONDITION_CMD_PARTIALLY_INSIDE_ROOM:
		{
			int itmp,itmp2;

			if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				int x1,y1,x2,y2;
				int tmpx,tmpy;

				tmpx=(obj->precision_x+itmp)/100;
				tmpy=(obj->precision_y+itmp2)/100;
				
				obj->get_bbox(&x1,&y1,&x2,&y2);
				x1+=tmpx;
				y1+=tmpy;
				x2+=tmpx;
				y2+=tmpy;
				if ((!room->room->horizontal_circularity &&
					 (x2<0 ||
					  x1>=room->room->dx*TILESIZE)) ||
					(!room->room->vertical_circularity &&
					 (y2<0 ||
					  y1>=room->room->dy*TILESIZE))) {
					return false;
				} /* if */ 

				return true;
			} /* if */ 
		}
		break;

	case CONDITION_CMD_H_COLLISION:
		{
			int itmp,itmp2;

			if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				int tmpx,tmpy;
				int ax,ay;
				int tmp_res=0,tmp_res2=0;
				tmpx=(obj->precision_x+itmp)/100;
				tmpy=(obj->precision_y+itmp2)/100;
				
//				ax=obj->old_x+obj->room->room->x*TILESIZE;
//				ay=obj->old_y+obj->room->room->y*TILESIZE;
				ax=obj->x+obj->room->room->x*TILESIZE;
				ay=obj->y+obj->room->room->y*TILESIZE;

				tmp_res=map->object_collision(obj,ax+tmpx,ay+tmpy,false);
				if ((tmp_res&CONS_SOLID)!=0) return true;
				if (tmp_res==0) return false;
				if (tmpy<=0) return false;
				tmp_res2=(map->object_collision(obj,ax,ay,false))&CONS_SOLID_SURFACE;
				if (tmp_res2==0) return true;
				return false;
			} /* if */ 
		}
		break;
	case CONDITION_CMD_H_COLLISION_WITH_OBJECT:
		{
			char *obj_cons;
			int itmp,itmp2;

			if (vpar[0]->cycle(0,&obj_cons,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
				vpar[1]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				int tmpx,tmpy;
				int ax,ay;
				int cons;
				tmpx=(obj->precision_x+itmp)/100;
				tmpy=(obj->precision_y+itmp2)/100;
				
//				ax=obj->old_x+obj->room->room->x*TILESIZE;
//				ay=obj->old_y+obj->room->room->y*TILESIZE;
				ax=obj->x+obj->room->room->x*TILESIZE;
				ay=obj->y+obj->room->room->y*TILESIZE;

				cons=0;
				if (substr(obj_cons,"none")) cons|=CONS_NONE;
				if (substr(obj_cons,"solid")) cons|=CONS_SOLID;
				if (substr(obj_cons,"solid_sfc")) cons|=CONS_SOLID_SURFACE;
				if (substr(obj_cons,"weapon")) cons|=CONS_WEAPON;
				if (substr(obj_cons,"ladder")) cons|=CONS_LADDER;
				if (substr(obj_cons,"character")) cons|=CONS_CHARACTER;
				if (substr(obj_cons,"item")) cons|=CONS_ITEM;
				*foreign_object=0;

				if (cons!=0) {
					*foreign_object=map->object_collision2(obj,ax+tmpx,ay+tmpy,cons,aparlist[0],foreign_object,true);
				} /* if */ 
				if (*foreign_object!=0) return true;
							  	   else return false;
			} /* if */ 
		}
		break;
	case CONDITION_CMD_H_INSIDE_ROOM:
		{
			int itmp,itmp2;

			if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				int x1,y1,x2,y2;
				int tmpx,tmpy;

				tmpx=(obj->precision_x+itmp)/100;
				tmpy=(obj->precision_y+itmp2)/100;
				
				obj->get_family_bbox(&x1,&y1,&x2,&y2);
				x1+=tmpx;
				y1+=tmpy;
				x2+=tmpx;
				y2+=tmpy;
				if ((!room->room->horizontal_circularity &&
					 (x1<0 ||
					  x2>=room->room->dx*TILESIZE)) ||
					(!room->room->vertical_circularity &&
					 (y1<0 ||
					  y2>=room->room->dy*TILESIZE))) {
					return false;
				} /* if */ 

				return true;
			} /* if */ 
		}
		break;
	case CONDITION_CMD_H_PARTIALLY_INSIDE_ROOM:
		{
			int itmp,itmp2;

			if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				int x1,y1,x2,y2;
				int tmpx,tmpy;

				tmpx=(obj->precision_x+itmp)/100;
				tmpy=(obj->precision_y+itmp2)/100;
				
				obj->get_family_bbox(&x1,&y1,&x2,&y2);
				x1+=tmpx;
				y1+=tmpy;
				x2+=tmpx;
				y2+=tmpy;
				if ((!room->room->horizontal_circularity &&
					 (x2<0 ||
					  x1>=room->room->dx*TILESIZE)) ||
					(!room->room->vertical_circularity &&
					 (y2<0 ||
					  y1>=room->room->dy*TILESIZE))) {
					return false;
				} /* if */ 

				return true;
			} /* if */ 
		}
		break;
	case CONDITION_CMD_SENSE_OBJECT:
		{
			char *obj_cons;
			int x,y,dx,dy;
			int ox,oy,ox2,oy2;
			int i;
			int cons=0;

			if (vpar[0]->cycle(0,&obj_cons,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
				vpar[1]->cycle(&x,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&y,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&dx,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[4]->cycle(&dy,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				List<ActiveOBJECT> l;
				ActiveOBJECT *o;
				SDL_Surface *tmp_sfc=0;
				MOG2CollisionData *tmp_collision=0;
				if (location==LOCATION_OBJECT) {
					x+=obj->x;
					y+=obj->y;
				} /* if */ 

				cons=0;
				if (substr(obj_cons,"none")) cons|=CONS_NONE;
				if (substr(obj_cons,"solid")) cons|=CONS_SOLID;
				if (substr(obj_cons,"solid_sfc")) cons|=CONS_SOLID_SURFACE;
				if (substr(obj_cons,"weapon")) cons|=CONS_WEAPON;
				if (substr(obj_cons,"ladder")) cons|=CONS_LADDER;
				if (substr(obj_cons,"character")) cons|=CONS_CHARACTER;
				if (substr(obj_cons,"item")) cons|=CONS_ITEM;
				*foreign_object=0;

				for(i=0;i<MAX_OBJECT_PLANES;i++) {
					if (room==0 ||
						(room->room->plane_h_speed[i]==100 && room->room->plane_v_speed[i]==100)) {
						if (interf!=0) l.Instance(interf->objects[i]);
								  else l.Instance(room->objects[i]);
						l.Rewind();
						while(l.Iterate(o)) {
							if (o!=obj && (o->object->constitution&cons)!=0) {
								o->get_bbox(&ox,&oy,&ox2,&oy2);
								if (((ox2>=x && ox<x+dx) ||
									 (o->room!=0 && o->room->room->horizontal_circularity &&
									  ((ox2+o->room->room->get_dx()*TILESIZE>=x && ox+o->room->room->get_dx()*TILESIZE<x+dx) || 
									   (ox2-o->room->room->get_dx()*TILESIZE>=x && ox-o->room->room->get_dx()*TILESIZE<x+dx)))) && 
									((oy2>=y && oy<y+dy) ||
									 (o->room!=0 && o->room->room->vertical_circularity &&
									  ((oy2+o->room->room->get_dy()*TILESIZE>=y && oy+o->room->room->get_dy()*TILESIZE<y+dy) || 
									   (oy2-o->room->room->get_dy()*TILESIZE>=y && oy-o->room->room->get_dy()*TILESIZE<y+dy))))) {
									*foreign_object=o;
									if (aparlist[0]->cycle(obj,room,map,game,interf,app,foreign_object,location,0,0)) {
									    if (tmp_collision!=0) MOG2_destroy_cmap(tmp_collision);
										if (tmp_sfc!=0) SDL_FreeSurface(tmp_sfc);
										return true;
									} /* else */ 
									*foreign_object=0;
								} /* if */ 
							} /* if */ 
						} /* while */ 
					} /* if */ 
				} /* for */ 

				if (tmp_collision!=0) MOG2_destroy_cmap(tmp_collision);
				if (tmp_sfc!=0) SDL_FreeSurface(tmp_sfc);
			} /* if */ 

			return false;
		}
		break;
	case CONDITION_CMD_SENSE_SPECIFIC_OBJECT:
		{
			char *obj_name;
			int x,y,dx,dy;
			int ox,oy,ox2,oy2;
			int i;

			if (vpar[0]->cycle(0,&obj_name,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
				vpar[1]->cycle(&x,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&y,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&dx,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[4]->cycle(&dy,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				List<ActiveOBJECT> l;
				ActiveOBJECT *o;
				SDL_Surface *tmp_sfc=0;
				MOG2CollisionData *tmp_collision=0;
				if (location==LOCATION_OBJECT) {
					x+=obj->x;
					y+=obj->y;
				} /* if */ 

				for(i=0;i<MAX_OBJECT_PLANES;i++) {
					if (room==0 ||
						(room->room->plane_h_speed[i]==100 && room->room->plane_v_speed[i]==100)) {
						if (interf!=0) l.Instance(interf->objects[i]);
								  else l.Instance(room->objects[i]);
						l.Rewind();
						while(l.Iterate(o)) {
							if (o!=obj && strcmp(o->object->name,obj_name)==0) {
								o->get_bbox(&ox,&oy,&ox2,&oy2);
								if (((ox2>=x && ox<x+dx) ||
									 (o->room!=0 && o->room->room->horizontal_circularity &&
									  ((ox2+o->room->room->get_dx()*TILESIZE>=x && ox+o->room->room->get_dx()*TILESIZE<x+dx) || 
									   (ox2-o->room->room->get_dx()*TILESIZE>=x && ox-o->room->room->get_dx()*TILESIZE<x+dx)))) && 
									((oy2>=y && oy<y+dy) ||
									 (o->room!=0 && o->room->room->vertical_circularity &&
									  ((oy2+o->room->room->get_dy()*TILESIZE>=y && oy+o->room->room->get_dy()*TILESIZE<y+dy) || 
									   (oy2-o->room->room->get_dy()*TILESIZE>=y && oy-o->room->room->get_dy()*TILESIZE<y+dy))))) {
									*foreign_object=o;
									if (aparlist[0]->cycle(obj,room,map,game,interf,app,foreign_object,location,0,0)) {
									    if (tmp_collision!=0) MOG2_destroy_cmap(tmp_collision);
										if (tmp_sfc!=0) SDL_FreeSurface(tmp_sfc);
										return true;
									} /* if */ 
									*foreign_object=0;
								} /* if */ 
							} /* if */ 
						} /* while */ 
					} /* if */ 
				} /* for */ 
		    
				if (tmp_collision!=0) MOG2_destroy_cmap(tmp_collision);
				if (tmp_sfc!=0) SDL_FreeSurface(tmp_sfc);
			} /* if */ 

			return false;
		}
		break;

	case CONDITION_CMD_KEYPRESS:
		{
			int itmp;

			if (LVK!=0 && n_LVKs!=0 &&
				vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				int key=-1;
				if (itmp>=0) {
					key=itmp;
				} else {
					switch(itmp) {
					case -1:key=obj->left_key;
						break;
					case -2:key=obj->right_key;
						break;
					case -3:key=obj->up_key;
						break;
					case -4:key=obj->down_key;
						break;
					case -5:key=obj->fire1_key;
						break;
					case -6:key=obj->fire2_key;
						break;
					case -7:key=obj->fire3_key;
						break;
					case -8:key=obj->fire4_key;
						break;
					case -9:
							key=-1;
						break;
					} /* switch */ 
				} /* if */ 

				if (obj==0 || obj->LVK_ID==-1) {
					int i,j;
					for(i=0;i<n_LVKs;i++) {
						if (key==-1) {
							for(j=0;j<LVK[i]->k_size;j++) {
								if (LVK[i]->keyboard[j] && !LVK[i]->old_keyboard[j]) return true;
							} /* for */ 
						} else {
							if (key<LVK[i]->k_size &&
								LVK[i]->keyboard[key] && !LVK[i]->old_keyboard[key]) 
								return true;
						} /* if */ 
					} /* for */ 
				} else {
					if (key==-1) {
						int i;
						for(i=0;i<LVK[obj->LVK_ID]->k_size;i++) {
							if (LVK[obj->LVK_ID]->keyboard[i] && !LVK[obj->LVK_ID]->old_keyboard[i]) return true;
						} /* for */ 
					} else {
						if (key<LVK[obj->LVK_ID]->k_size &&
							LVK[obj->LVK_ID]->keyboard[key] && !LVK[obj->LVK_ID]->old_keyboard[key]) return true;
					} /* if */ 
				} /* if */ 

				return false;
			} /* if */ 
		}
		break;
	case CONDITION_CMD_KEYPRESSED:
		{
			int itmp;

			if (LVK!=0 && n_LVKs!=0 &&
				vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				int key=-1;
				if (itmp>=0) {
					key=itmp;
				} else {
					switch(itmp) {
					case -1:key=obj->left_key;
						break;
					case -2:key=obj->right_key;
						break;
					case -3:key=obj->up_key;
						break;
					case -4:key=obj->down_key;
						break;
					case -5:key=obj->fire1_key;
						break;
					case -6:key=obj->fire2_key;
						break;
					case -7:key=obj->fire3_key;
						break;
					case -8:key=obj->fire4_key;
						break;
					case -9:
							key=-1;
						break;
					} /* switch */ 
				} /* if */ 

				if (obj==0 || obj->LVK_ID==-1) {
					int i,j;
					for(i=0;i<n_LVKs;i++) {
						if (key==-1) {
							for(j=0;j<LVK[i]->k_size;j++) {
								if (LVK[i]->keyboard[j]) return true;
							} /* for */ 
						} else {
							if (key<LVK[i]->k_size && LVK[i]->keyboard[key]) 
								return true;
						} /* if */ 
					} /* for */ 

				} else {
					if (key==-1) {
						int i;
						for(i=0;i<LVK[obj->LVK_ID]->k_size;i++) {
							if (LVK[obj->LVK_ID]->keyboard[i]) return true;
						} /* for */ 
					} else {
						if (key<LVK[obj->LVK_ID]->k_size && LVK[obj->LVK_ID]->keyboard[key]) return true;
					} /* if */ 
				} /* if */ 

				return false;
			} /* if */ 
 
		}
		break;

	case CONDITION_CMD_VARIABLE_EQ:
		{
			int v1,v2;
			ActiveOBJECT *otmp,*otmp2;
			char *stmp,*stmp2;
			int itmp,itmp2;

			v1=vpar[0]->cycle(&itmp,&stmp,&otmp,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
			v2=vpar[1]->cycle(&itmp2,&stmp2,&otmp2,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);

			if (v1!=v2) return false;
			if (v1==VALUE_INTEGER) {
				return (itmp==itmp2 ? true:false);
			} /* if */ 
			if (v1==VALUE_STRING) {
				if (stmp!=0 && stmp2!=0) return (strcmp(stmp,stmp2)==0 ? true:false);
									else return false;
			} /* if */ 
			if (v1==VALUE_OBJECT) {
				if (app!=0) {
					if (!app->find_object(otmp)) otmp=0;
					if (!app->find_object(otmp2)) otmp2=0;
				} else {
					if (game!=0) {
						if (!game->find_object(otmp)) otmp=0;
						if (!game->find_object(otmp2)) otmp2=0;
					} /* if */ 
				} /* if */ 
				if (otmp==otmp2) return true;
				if (otmp!=0 && otmp2!=0) {
					if (otmp->other_instances.MemberRefP(otmp2)) return true;
				} /* if */ 
			} /* if */ 

			return false;
		}
		break;
	case CONDITION_CMD_VARIABLE_GR:
		{
			int itmp,itmp2;

			if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {

				return itmp>itmp2;
			} /* if */ 

			return false;
		}
		break;
	case CONDITION_CMD_VARIABLE_GREQ:
		{
			int itmp,itmp2;

			if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {

				return itmp>=itmp2;
			} /* if */ 

			return false;
		}
		break;
	case CONDITION_CMD_VARIABLE_LW:
		{
			int itmp,itmp2;

			if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {

				return itmp<itmp2;
			} /* if */ 

			return false;
		}
		break;
	case CONDITION_CMD_VARIABLE_LWEQ:
		{
			int itmp,itmp2;

			if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {

				return itmp<=itmp2;
			} /* if */ 

			return false;
		}
		break;
	case CONDITION_CMD_FIND_SUBSTRING:
		{
			char *s1,*s2;
			if (vpar[0]->cycle(0,&s1,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
				vpar[1]->cycle(0,&s2,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
				int i=0,j=0,where=0;
				bool found=false;

				for(i=0;s2[i]!=0 && !found;i++) {
					for(j=0;s1[j]!=0 && s2[i+j]==s1[j];j++);
					if (s1[j]==0) {
						where=i;
						found=true;					
					} /* for */ 
				} /* for */ 

				return found;
			} /* if */ 
		}
		break;

	case CONDITION_CMD_INVINCIBLE:
		if (obj->invincible_script!=0) return true;
		return false;
		break;

	case CONDITION_CMD_INLADDER:
		if (obj->in_ladder!=0) return true;
		return false;
		break;

	case CONDITION_CMD_VALID_OBJECT:
		{
			ActiveOBJECT *o1;
			if (vpar[0]->cycle(0,0,&o1,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_OBJECT) {
				return app->find_object(o1);
			} /* if */ 
		}
		break;

	case CONDITION_CMD_LOADGAME:
		return false;
		break;

	case CONDITION_CMD_SAVEGAME:
		return false;
		break;
		
	} /* switch */ 
	return false;
} /* ConditionSCRIPT::cycle */ 


void ConditionSCRIPT::copy_state(ConditionSCRIPT *s)
{
	int i;
	timmer=s->timmer;

	command=s->command;
	for(i=0;i<MAX_CONDITION_PARAMETERS;i++) {
		if (vpar[i]==0) {
			if (s->vpar[i]!=0) {
				vpar[i]=new ValueSCRIPT();
				vpar[i]->copy_state(s->vpar[i]);
			} /* if */ 
		} else {
			if (s->vpar[i]==0) {
				delete vpar[i];
				vpar[i]=0;
			} else {
				vpar[i]->copy_state(s->vpar[i]);
			} /* if */ 
		} /* if */ 
	} /* if */ 

	if (aparlist.Length()!=s->aparlist.Length()) {
		aparlist.Delete();
		aparlist.Copy(s->aparlist);
	} else {
		List<ConditionSCRIPT> l,l2;
		ConditionSCRIPT *ap,*ap2;

		l.Instance(aparlist);
		l2.Instance(s->aparlist);
		l.Rewind();
		l2.Rewind();

		while(l.Iterate(ap) &&
			  l2.Iterate(ap2)) {
			ap->copy_state(ap2);
		} /* while */ 
	} /* if */ 
	aparlist.Synchronize(&(s->aparlist));

	dx=0;
	dy=0;
} /* ConditionSCRIPT::copy_state */ 


void ConditionSCRIPT::simplify(void)
{
	ConditionSCRIPT *tmp,*tmp2;
	List<ConditionSCRIPT> l,l2,ltmp,todelete;

	l.Instance(aparlist);
	l.Rewind();
	while(l.Iterate(tmp)) {
		if (tmp->command==CONDITION_CMD_DELETE) {
			if (command==CONDITION_CMD_AND ||
				command==CONDITION_CMD_OR) {
				todelete.Add(tmp);
			} else {
				tmp->command=CONDITION_CMD_FALSE;
			} /* if */ 
		} else {
			tmp->simplify();
		} /* if */ 
	} /* while */ 

	while(!todelete.EmptyP()) {
		tmp=todelete.ExtractIni();
		aparlist.DeleteElement(tmp);
		delete tmp;
	} /* while */ 

	switch(command) {
	case CONDITION_CMD_DELETE:
		command=CONDITION_CMD_FALSE;
		break;
	case CONDITION_CMD_NOT:
		if (aparlist[0]->command==CONDITION_CMD_NOT) {
			ConditionSCRIPT *tmp;

			tmp=new ConditionSCRIPT(*(aparlist[0]->aparlist[0]));
			this->copy_state(tmp);
			delete tmp;
		} /* if */ 

		break;
	case CONDITION_CMD_AND:
	case CONDITION_CMD_OR:
		{			
			ltmp.Delete();

			l.Instance(aparlist);
			l.Rewind();
			while(l.Iterate(tmp)) {
				if (tmp->command==command) {
					l2.Instance(tmp->aparlist);
					l2.Rewind();
					while(l2.Iterate(tmp2)) {
						ltmp.Add(new ConditionSCRIPT(*tmp2));
					} /* while */ 
				} else {
					ltmp.Add(new ConditionSCRIPT(*tmp));
				} /* if */ 
			} /* while */ 

			aparlist.Delete();
			aparlist.Copy(ltmp);
			ltmp.Delete();

			if (aparlist.Length()==1) {
				tmp=aparlist.ExtractIni();

				copy_state(tmp);
				delete tmp;
			} /* if */ 
		}
		break;

	default:
		break;
	} /* switch */ 

} /* ConditionSCRIPT::simplify */ 


bool ConditionSCRIPT::uses_sample(SAMPLE *s)
{
	int i;
	ConditionSCRIPT *tmp;
	List<ConditionSCRIPT> l;

	l.Instance(aparlist);
	l.Rewind();
	while(l.Iterate(tmp)) if (tmp->uses_sample(s)) return true;

	for(i=0;i<MAX_CONDITION_PARAMETERS;i++) if (vpar[i]!=0) if (vpar[i]->uses_sample(s)) return true;

	return false;
} /* ConditionSCRIPT::uses_sample */ 
