#include "assert.h"
#include "stdlib.h"
#include "string.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"

#include "actionscript.h"

#include "interfaces.h"

#include "auxiliar.h"
#include "MOG2_auxiliar.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif


INTERFACE::INTERFACE(void)
{
	game=0;
	name=0;
	display_mode=0;
	dx=640;
	dy=480;
	used_object_planes=2;
	interface_action=new ActionSCRIPT();
} /* INTERFACE::INTERFACE */ 


INTERFACE::INTERFACE(char *n,MOG2 *g)
{
	game=g;
	name=new char[strlen(n)+1];
	strcpy(name,n);
	display_mode=0;
	dx=640;
	dy=480;
	used_object_planes=2;
	interface_action=new ActionSCRIPT();
} /* INTERFACE::INTERFACE */ 


INTERFACE::INTERFACE(char *n,INTERFACE *i)
{
	int c;

	game=i->game;
	name=new char[strlen(n)+1];
	strcpy(name,n);

	interface_variables.Copy(i->interface_variables);
	used_object_planes=i->used_object_planes;
	dx=i->dx;
	dy=i->dy;
	interface_action=new ActionSCRIPT(*i->interface_action);
	display_mode=i->display_mode;

	for(c=0;c<MAX_OBJECT_PLANES;c++) {
		tiles[c].Copy(i->tiles[c]);
		objects[c].Copy(i->objects[c]);
	} /* for */ 
} /* INTERFACE::INTERFACE */ 


/* This constructor automatically generates an interface with a GAMEWINDOW */ 
INTERFACE::INTERFACE(int w,int h,OBJECT **o_p)
{
	OBJECT *o;
	PLACED_OBJECT *po;

	game=0;
	name=0;
	display_mode=0;
	dx=w;
	dy=h;
	used_object_planes=1;
	interface_action=new ActionSCRIPT();

	o=new OBJECT();
	o->control=CTRL_GAMEWINDOW;
	o->gw_dx=w;
	o->gw_dy=h;
	*o_p=o;

	po=new PLACED_OBJECT(0,0,o,0,0,this);
	objects[0].Add(po);

} /* INTERFACE::INTERFACE */ 



INTERFACE::~INTERFACE(void)
{
	delete []name;
	name=0;
	delete interface_action;
} /* INTERFACE::~INTERFACE */ 


bool INTERFACE::save(FILE *fp,MOG2 *game)
{
	char *tmp;
	int i;
	List<VARIABLE> l;
	VARIABLE *v;
	List<PLACED_TILE> l1;
	PLACED_TILE *t;
	List<PLACED_OBJECT> l2;
	PLACED_OBJECT *o;

	encode_string(name,&tmp);
	fprintf(fp,"NAME %s\n",tmp);
	delete []tmp;

	fprintf(fp,"VARIABLES %i\n",interface_variables.Length());
	l.Instance(interface_variables);
	l.Rewind();
	while(l.Iterate(v)) v->save(fp);

	fprintf(fp,"SIZE %i %i\n",dx,dy);
	fprintf(fp,"%i\n",display_mode);

	fprintf(fp,"%i\n",used_object_planes);
	for(i=0;i<used_object_planes;i++) {
	    fprintf(fp,"%i\n",tiles[i].Length());
	    l1.Instance(tiles[i]);
	    l1.Rewind();
	    while(l1.Iterate(t)) t->save(fp,game);

	    fprintf(fp,"%i\n",objects[i].Length());
	    l2.Instance(objects[i]);
	    l2.Rewind();
	    while(l2.Iterate(o)) o->save(fp,game);
	} /* for */ 

	interface_action->save(fp);

	return true;
} /* INTERFACE::save */ 


bool INTERFACE::load(FILE *fp,MOG2 *g)
{
	char tmp[80];
	int i,j,n;

	game=g;
	if (1!=fscanf(fp,"%s ",tmp)) return false;
	if (name!=0) delete name;
	if (!load_string(fp,&name)) return false;

	if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;

	interface_variables.Delete();
	for(i=0;i<n;i++) {
		VARIABLE *v;

		v=new VARIABLE();
		v->load(fp);
		interface_variables.Add(v);
	} /* for */ 

	if (3!=fscanf(fp,"%s %i %i",tmp,&dx,&dy)) return false;
	if (1!=fscanf(fp,"%i",&display_mode)) return false;

	if (1!=fscanf(fp,"%i",&used_object_planes)) return false;

	if (used_object_planes>MAX_OBJECT_PLANES) return false;

	for(j=0;j<used_object_planes;j++) {
	    if (1!=fscanf(fp,"%i",&n)) return false;
	    tiles[j].Delete();
	    for(i=0;i<n;i++) {
			PLACED_TILE *t;

			t=new PLACED_TILE();
			t->load(fp,game);
			tiles[j].Add(t);
	    } /* for */ 

	    if (1!=fscanf(fp,"%i",&n)) return false;
	    objects[j].Delete();
	    for(i=0;i<n;i++) {
		PLACED_OBJECT *o;

		o=new PLACED_OBJECT();
		o->load(fp,game,0,this);
		objects[j].Add(o);
	    } /* for */ 
	} /* for */ 	

	if (interface_action!=0) delete interface_action;
	interface_action=new ActionSCRIPT();
	if (!interface_action->load(fp,game)) return false;


	return true;
} /* INTERFACE::load */ 


bool INTERFACE::cmp(char *n)
{
	if (strcmp(name,n)==0) return true;

	return false;
} /* INTERFACE::cmp */ 


bool INTERFACE::cmp(INTERFACE *i)
{
	if (strcmp(name,i->name)==0) return true;

	return false;
} /* INTERFACE::cmp */ 


void INTERFACE::draw(SDL_Surface *sfc,int sx,int sy,int dx,int dy,int stx,int sty,float scale,bool *dplane)
{
	int i;

	for(i=0;i<used_object_planes;i++) {
		if (dplane==0 || dplane[i]) {
			List<PLACED_TILE> l1;
			List<PLACED_OBJECT> l2;
			PLACED_TILE *t;
			PLACED_OBJECT *o;

			l1.Instance(tiles[i]);
			l1.Rewind();
			if (scale==1.0) while(l1.Iterate(t)) t->draw(sx,sy,sfc);
					   else while(l1.Iterate(t)) t->draw_scaled(sx,sy,sfc,scale);

			l2.Instance(objects[i]);
			l2.Rewind();
			if (scale==1.0) while(l2.Iterate(o)) o->draw(sx,sy,sfc);
					   else while(l2.Iterate(o)) o->draw_scaled(sx,sy,sfc,scale);
		} /* if */ 
	} /* for */ 
} /* INTERFACE::draw */ 



