#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "variables.h"
#include "mog2.h"

#include "actionscript.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

MAP::MAP(MOG2 *g)
{
	name=0;
	game=g;
	map_action=new ActionSCRIPT();
} /* MAP::MAP */ 


MAP::MAP(char *n,MOG2 *g)
{
	game=g;
	name=new char[strlen(n)+1];
	strcpy(name,n);
	map_action=new ActionSCRIPT();
} /* MAP::MAP */ 


MAP::~MAP(void)
{
	game=0;
	delete []name;
	delete map_action;
} /* MAP::~MAP */ 


bool MAP::save(FILE *fp,MOG2 *game)
{
	List<VARIABLE> l;
	VARIABLE *v;
	List<ROOM> l2;
	ROOM *r;

	fprintf(fp,"NAME %s\n",name);

	fprintf(fp,"VARIABLES %i\n",map_variables.Length());
	l.Instance(map_variables);
	l.Rewind();
	while(l.Iterate(v)) v->save(fp);

	fprintf(fp,"ROOMS %i\n",rooms.Length());
	l2.Instance(rooms);
	l2.Rewind();
	while(l2.Iterate(r)) r->save(fp,game);

	map_action->save(fp);

	return true;
} /* MAP::save */ 


bool MAP::load(FILE *fp,MOG2 *game)
{
	char tmp[80];
	int i,n;

	if (1!=fscanf(fp,"%s ",tmp)) return false;
	fgets(tmp,80,fp);
	while((tmp[strlen(tmp)-1]==13) ||
		  (tmp[strlen(tmp)-1]==10)) tmp[strlen(tmp)-1]=0;

	if (name!=0) delete name;
	name=new char[strlen(tmp)+1];
	strcpy(name,tmp);

	if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;

	map_variables.Delete();
	for(i=0;i<n;i++) {
		VARIABLE *v;

		v=new VARIABLE();
		v->load(fp);
		map_variables.Add(v);
	} /* for */ 

	if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;

	rooms.Delete();
	for(i=0;i<n;i++) {
		ROOM *r;

		r=new ROOM(this);
		r->load(fp,game);
		rooms.Add(r);
	} /* for */ 

	if (map_action!=0) delete map_action;
	map_action=new ActionSCRIPT();
	map_action->load(fp,game);

	return true;
} /* MAP::load */ 


bool MAP::cmp(char *n)
{
	if (strcmp(name,n)==0) return true;

	return false;
} /* MAP::MAP */ 


void MAP::CMC(int *x1p,int *y1p,int *x2p,int *y2p)
{
	int x1=0,y1=0;
	int x2=0,y2=0;
	ROOM *r;
	List<ROOM> l;
	bool first=true;

	l.Instance(rooms);
	l.Rewind();
	while(l.Iterate(r)) {
		if (r->get_x()!=NOPOSITION && r->get_y()!=NOPOSITION) {
			if (first) {
				x1=r->get_x();
				y1=r->get_y();
				x2=r->get_x()+r->get_dx();
				y2=r->get_y()+r->get_dy();
				first=false;
			} /* if */ 
			if (r->get_x()<x1) x1=r->get_x();
			if (r->get_y()<y1) y1=r->get_y();
			if (r->get_x()+r->get_dx()>x2) x2=r->get_x()+r->get_dx();
			if (r->get_y()+r->get_dy()>y2) y2=r->get_y()+r->get_dy();
		} /* if */ 
	} /* while */ 

	*x1p=x1;
	*y1p=y1;
	*x2p=x2;
	*y2p=y2;
} /* if */ 


void MAP::BiggestRoom(int *dxp,int *dyp)
{
	int dx=0,dy=0;
	ROOM *r;
	List<ROOM> l;
	bool first=true;

	l.Instance(rooms);
	l.Rewind();
	while(l.Iterate(r)) {
		if (first) {
			dx=r->get_dx();
			dy=r->get_dy();
			first=false;
		} /* if */ 
		if (r->get_dx()>dx) dx=r->get_dx();
		if (r->get_dy()>dy) dy=r->get_dy();
	} /* while */ 

	*dxp=dx;
	*dyp=dy;
} /* MAP::BiggestRoom */ 


void MAP::BiggestUnplacedRoom(int *dxp,int *dyp)
{
	int dx=32,dy=24;
	ROOM *r;
	List<ROOM> l;
	bool first=true;

	l.Instance(rooms);
	l.Rewind();
	while(l.Iterate(r)) {
		if (r->get_x()==NOPOSITION || r->get_y()==NOPOSITION) {
			if (first) {
				dx=r->get_dx();
				dy=r->get_dy();
				first=false;
			} /* if */ 
			if (r->get_dx()>dx) dx=r->get_dx();
			if (r->get_dy()>dy) dy=r->get_dy();
		} /* if */ 
	} /* while */ 

	*dxp=dx;
	*dyp=dy;
} /* MAP::BiggestUnplacedRoom */ 



void MAP::draw(SDL_Surface *sfc,int x,int y,int dx,int dy,int stx,int sty,float scale)
{
	ROOM *r;
	List<ROOM> l;
	int x1,y1,x2,y2;

	CMC(&x1,&y1,&x2,&y2);

	l.Instance(rooms);
	l.Rewind();
	while(l.Iterate(r)) {
		if (r->get_x()!=NOPOSITION && r->get_y()!=NOPOSITION) {
			SDL_Rect rect;

			rect.x=int(x+(r->get_x()-x1)*TILESIZE*scale);
			rect.y=int(y+(r->get_y()-y1)*TILESIZE*scale);
			rect.w=int(r->get_dx()*TILESIZE*scale);
			rect.h=int(r->get_dy()*TILESIZE*scale);
			SDL_FillRect(sfc,&rect,0);

			r->draw(sfc,int(x+(r->get_x()-x1)*TILESIZE*scale),
			  		    int(y+(r->get_y()-y1)*TILESIZE*scale),dx,dy,stx,sty,scale,0);
		} /* if */ 
	} /* while */ 
} /* MAP::draw */ 


void MAP::draw_frame(SDL_Surface *sfc,int x,int y,int dx,int dy,int stx,int sty,float scale)
{
	ROOM *r;
	List<ROOM> l;
	int x1,y1,x2,y2;

	CMC(&x1,&y1,&x2,&y2);

	l.Instance(rooms);
	l.Rewind();
	while(l.Iterate(r)) {
		if (r->get_x()!=NOPOSITION && r->get_y()!=NOPOSITION) r->draw_frame(sfc,int(x+(r->get_x()-x1)*TILESIZE*scale),
																				int(y+(r->get_y()-y1)*TILESIZE*scale),dx,dy,stx,sty,scale);
	} /* while */ 
} /* MAP::draw */ 


bool MAP::fit_roomp(int x,int y,int dx,int dy)
{
	ROOM *r;
	List<ROOM> l;

	l.Instance(rooms);
	l.Rewind();
	while(l.Iterate(r)) {
		if (r->get_x()!=NOPOSITION && r->get_y()!=NOPOSITION) {
			if (r->get_x()+r->get_dx()>x &&
				r->get_x()<x+dx &&
				r->get_y()+r->get_dy()>y &&
				r->get_y()<y+dy) {
				return false;
			} /* if */ 
		} /* if */ 
	} /* while */ 
	return true;
} /* MAP::fit_roomp */ 


ROOM *MAP::get_room(int x,int y)
{
	ROOM *r;
	List<ROOM> l;

	l.Instance(rooms);
	l.Rewind();
	while(l.Iterate(r)) {
		if (r->get_x()!=NOPOSITION && r->get_y()!=NOPOSITION) {
			if (r->get_x()+r->get_dx()>=x &&
				r->get_x()<=x &&
				r->get_y()+r->get_dy()>=y &&
				r->get_y()<=y) {
				return r;
			} /* if */ 
		} /* if */ 
	} /* while */ 

	return 0;
} /* MAP::get_room */ 


PLACED_OBJECT *MAP::search_ladder(int x,int y,int dx,int dy)
{
	List<ROOM> rl;
	ROOM *r;
	PLACED_OBJECT *o;

	rl.Instance(rooms);
	rl.Rewind();
	while(rl.Iterate(r)) {
		if (x+dx>r->get_x()*TILESIZE &&
			x<(r->get_x()+r->get_dx())*TILESIZE &&
			y+dy>r->get_y()*TILESIZE &&
			y<(r->get_y()+r->get_dy())*TILESIZE) {

			o=r->search_ladder(x-(r->get_x()*TILESIZE),y-(r->get_y()*TILESIZE),dx,dy);

			if (o!=0) return o;
		} /* if */ 
	} /* while */ 
	
	return 0;
} /* ActiveMOG2::search_ladder */ 