#include "SDL.h"
#include "SDL_mixer.h"
#include "SDL_sound.h"
#include "SDL_ttf.h"

#include "music.h"
#include "stdio.h"
#include "string.h"
#include "stdlib.h"

#include "List.h"

#include "auxiliar.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "samples.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activeMAP.h"
#include "activemog2.h"
#include "ActiveMoG2Application.h"

extern bool SOUND_ENABLED;

bool playing_music=true;
Sound_Sample *music_sound=0;

void myMusicPlayer(void *udata, Uint8 *stream, int len);


void init_music(ActiveMOG2Application *app)
{
	if (music_sound!=0) Sound_FreeSample(music_sound);
	music_sound=0;
	if (app->music_playing!=0) delete []app->music_playing;
	app->music_playing=0;
} /* init_music */ 


/* a check to see if file is readable and greater than zero */
int file_check(char *fname)
{
	FILE *fp;

	if ((fp=fopen(fname,"r"))!=NULL) {
		if (fseek(fp,0L, SEEK_END)==0 && ftell(fp)>0) {
  			fclose(fp);
			return true;
		} /* if */ 
		/* either the file could not be read (==-1) or size was zero (==0) */ 
		fprintf(stderr,"ERROR in file_check(): the file %s is corrupted.\n", fname);
		fclose(fp);
		exit(1);
	} /* if */ 
	return false;
} /* file_check */ 


Sound_Sample *Sound_create_stream(char *file)
{
	int n_ext=6;
	char *ext[6]={".WAV",".OGG",".MP3",".wav",".ogg",".mp3"};
	char name[256];
	int i;

	Sound_AudioInfo inf;

	inf.format=AUDIO_S16;
	inf.channels=2;
	inf.rate=44100;

	for(i=0;i<n_ext;i++) {
		strcpy(name,"sound/");
		strcat(name,file);
		strcat(name,ext[i]);
		if (file_check(name)) return Sound_NewSampleFromFile(name,&inf,2048);
	} /* for */ 
	
	fprintf(stderr,"ERROR in Sound_create_stream(): Could not load sound file: %s.(wav|ogg|mp3)\n", file);
	return 0;
} /* Sound_create_stream */ 


void myMusicPlayer(void *udata, Uint8 *stream, int len)
{
	int i,act=0;
	Sint16 *ptr2;
	ActiveMOG2Application *game=(ActiveMOG2Application *)udata;

	if (SOUND_ENABLED && stream!=0) {
		ptr2=(Sint16 *)stream;
		if (playing_music && game!=0 && game->music_sequence!=0) {

			if (music_sound==0) {
				if (game->music_sequence[game->music_sequence_ip]==0) game->music_sequence_ip=0;
				switch(game->music_sequence[game->music_sequence_ip++]) {
				case 'S':game->music_part=0;
					break;
				case 'G':game->music_part=int(game->music_sequence[game->music_sequence_ip++]-'0');
					break;
				case 'E':
					game->music_part=-1;
					break;
				} /* switch */ 

				if (game->music_part>=game->music_parts.Length()) game->music_part=0;

				if (game->music_part>=0 && game->music_part<game->music_parts.Length()) {
					music_sound=Sound_create_stream(game->music_parts[game->music_part]);
					if (music_sound!=0) {
						if (game->music_playing!=0) delete []game->music_playing;
						game->music_playing=new char[strlen(game->music_parts[game->music_part])+1];
						strcpy(game->music_playing,game->music_parts[game->music_part]);
					} /* if */ 
				} else {
/*
					game->music_priority=0;
					delete []game->music_playing;
					game->music_playing=0;
					game->music_parts.Delete();
					delete []game->music_sequence;
					game->music_sequence=0;
					game->music_sequence_ip=0;
					game->music_part=0;
					if (music_sound!=0) Sound_FreeSample(music_sound);
					music_sound=0;
*/
				} /* if */ 
			} /* if */ 

			while(act<len) {
				if (music_sound!=0) {
					/* Play a music file: */ 

					if ((music_sound->flags&SOUND_SAMPLEFLAG_EOF)) {

						/* End of file: */ 
						if (music_sound!=0) Sound_FreeSample(music_sound);
						music_sound=0;
						if (game->music_sequence!=0) {
							switch(game->music_sequence[game->music_sequence_ip++]) {
							case 'S':game->music_part++;
								break;
							case 'G':game->music_part=int(game->music_sequence[game->music_sequence_ip++]-'0');
								break;
							case 'E':
								game->music_part=-1;
								break;
							} /* switch */ 
						} /* if */ 

						if (game->music_part>=game->music_parts.Length()) game->music_part=0;

						if (game->music_part>=0) {
							music_sound=Sound_create_stream(game->music_parts[game->music_part]);
							if (music_sound!=0) {
								if (game->music_playing!=0) delete []game->music_playing;
								game->music_playing=new char[strlen(game->music_parts[game->music_part])+1];
								strcpy(game->music_playing,game->music_parts[game->music_part]);
							} /* if */ 
						} else {
							game->music_priority=0;
							delete []game->music_playing;
							game->music_playing=0;
							game->music_parts.Delete();
							delete []game->music_sequence;
							game->music_sequence=0;
							game->music_sequence_ip=0;
							game->music_part=0;
						} /* if */ 

					} else {
						/* In the middle of the file: */ 
						int decoded=0;
						Sint16 *ptr;

						Sound_SetBufferSize(music_sound, len-act);

						decoded=Sound_Decode(music_sound);
						ptr=(Sint16 *)music_sound->buffer;
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
						Uint8 *ptr3,tmp;
						if (music_volume==100) {
						    for(i=0;i<decoded;i+=2,ptr++,ptr2++) *ptr2=*ptr;
						} else {
						    for(i=0;i<decoded;i+=2,ptr++,ptr2++) {
							    ptr3=(Uint8 *)ptr;
							    tmp=ptr3[0];
							    ptr3[0]=ptr3[1];
							    ptr3[1]=tmp;
							    *ptr2=Sint16((Sint32(*ptr)*Sint32(game->music_volume))/100);
							    ptr3=(Uint8 *)ptr2;
							    tmp=ptr3[0];
							    ptr3[0]=ptr3[1];
							    ptr3[1]=tmp;
						    } // for
						} /* if */ 
#else
						if (game->music_volume==100) {
						    for(i=0;i<decoded;i+=2,ptr++,ptr2++) *ptr2=*ptr;
						} else {
						    for(i=0;i<decoded;i+=2,ptr++,ptr2++) {
							    *ptr2=Sint16((Sint32(*ptr)*Sint32(game->music_volume))/100);
						    } // for
						} // if
#endif
						act+=decoded;
					} /* if */ 
				} else {
					/* No music file loaded: */ 
					for(i=act;i<len;i++) stream[i]=0;
					act=len;
				} /* if */ 
			} /* while */ 
		} else {
			if (game->music_sequence==0) {
				if (music_sound!=0) Sound_FreeSample(music_sound);
				music_sound=0;
			} /* if */ 

			/* No music to play: */ 
			for(i=0;i<len;i++) stream[i]=0;
		} /* if */ 
	} else {
		fprintf(stderr,"ERROR in myMusicPlayer(): null music stream!!\n");
	} /* if */ 
} /* myMusicPlayer */ 

