#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "BWindow.h"

#include "auxiliar.h"

#include "BListWnd.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

BListWnd::BListWnd(int x,int y,int dx,int dy,List<char> *items,TTF_Font *font) : BWindow(x,y,dx,dy,font)
{
	items_p=items;
	selected_name=0;
	selected=-1;

	right_clicked=left_clicked=-1;

} /* BFileListWnd::BFileListWnd */ 


BListWnd::~BListWnd()
{
	items_p=0;
	selected=0;
} /* BListWnd::~BListWnd */ 


bool BListWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	left_clicked=-1;
	right_clicked=-1;

	if ((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		(mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		List<char> l;
		char *f;
		int y=2;
		int i=0;

		l.Instance(*items_p);
		l.Rewind();
		while(l.Iterate(f)) {
			if (local_mouse_y>=y && 
				local_mouse_y<=y+TTF_FontHeight(font) &&
				local_mouse_x>=2 &&
				local_mouse_x<=get_text_width(f,font)+10) {	
				if ((mouse_button&SDL_BUTTON(1) &&
					!(old_mouse_button&SDL_BUTTON(1)))) {
					selected_name=f;
					selected=i;
					set_dirty();
					left_clicked=i;
				} /* if */ 
				if ((mouse_button&SDL_BUTTON(3) &&
					!(old_mouse_button&SDL_BUTTON(3)))) {
					right_clicked=i;
				} /* if */ 
			} /* if */ 
			i++;
			y+=2+TTF_FontHeight(font);
		} /* while */ 
	} /* if */ 

	return true;
} /* BListWnd::cycle */ 


void BListWnd::get_clicked_status(int *l_clicked,int *r_clicked)
{
	if (l_clicked!=0) *l_clicked=left_clicked;
	if (r_clicked!=0) *r_clicked=right_clicked;
} /* BListWnd::get_clicked_status */ 


void BListWnd::contens_size(int *dx,int *dy)
{
	int max_y;

	*dx=old_dx;
	*dy=old_dy;

	max_y=2+items_p->Length()*(TTF_FontHeight(font)+2);

	if (max_y>*dy) *dy=max_y;
} /* BListWnd::contens_size */ 


void BListWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	int y;
	char *tmp;
	List<char> l;

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	y=2;
	l.Instance(*items_p);
	l.Rewind();
	while(l.Iterate(tmp)) {
		if (tmp==selected_name) {
			SDL_Rect r;
			r.x=1-stx;
			r.y=(y-1)-sty;
			r.w=dx-2;
			r.h=TTF_FontHeight(font)+2;
			SDL_FillRect(sfc,&r,SDL_MapRGBA(sfc->format,255,0,0,0));
		} /* if */ 
		print_left(tmp,font,255,255,255,sfc,2-stx,y-sty);
		y+=2+TTF_FontHeight(font);
	} /* while */ 

} /* BListWnd::draw_contens */ 


int BListWnd::get_selected(void)
{
	return selected;
} /* BListWnd::get_selected */ 


char  *BListWnd::get_selected_name(void)
{
	return selected_name;
} /* BListWnd::get_selected_name */ 


void BListWnd::set_selected(int s)
{
	if (s>=0 && s<items_p->Length()) {
		selected=s;
		selected_name=items_p->operator [](selected);
	} else {
		if (s==-1) {
			selected_name=0;
			selected=-1;
		} /* if */ 
	} /* if */ 
} /* BListWnd::set_selected */ 


void BListWnd::set_selected_name(char *name)
{
	List<char> l;
	char *c;
	int i=0;

	l.Instance(*items_p);
	l.Rewind();
	while(l.Iterate(c)) {
		if (strcmp(c,name)==0) {
			selected_name=c;
			selected=i;
		} /* if */ 
		i++;
	} /* while */ 
} /* BListWnd::set_selected_name */ 




