#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"
#include "MOG2_auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "interfaces.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BChoiceDlg.h"
#include "BListWnd.h"
#include "BMultiListWnd.h"

#include "GVariableWnd.h"
#include "AVariableWnd.h"
#include "GMapWnd.h"
#include "GTopWnd.h"

#include "MTopWnd.h"
#include "MVariableWnd.h"
#include "MRoomsWnd.h"
#include "MMapWnd.h"

#include "RTopWnd.h"
#include "RVariableWnd.h"
#include "RObjectWnd.h"
#include "REditWnd.h"
#include "RToolsWnd.h"

#include "OTopWnd.h"
#include "OBitmapWnd.h"
#include "OSampleWnd.h"
#include "OObjectWnd.h"
#include "OObjectBmpWnd.h"
#include "OPropertiesWnd.h"

#include "GrabTopWnd.h"
#include "GrabGrabberWnd.h"
#include "GrabToolsWnd.h"

#include "ITopWnd.h"
#include "IObjectWnd.h"
#include "IVariablesWnd.h"
#include "IToolsWnd.h"
#include "IEditWnd.h"

#include "Editor.h"

#include "animscript.h"
#include "valuescript.h"
#include "conditionscript.h"
#include "actionscript.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int buffer_fps,buffer_fps_d;
extern int SCREEN_X;
extern int SCREEN_Y;


EDITOR::EDITOR(TTF_Font *f)
{
	view=GAME_VIEW;
	game=new MOG2();
	map=0;
	room=0;
	font=f;

	{
		FILE *fp;
		fp=fopen("default.mg2","r");

		if (fp!=0) {
			MOG2 *newgame;
			newgame=new MOG2();
			if (newgame->load(fp)) {
				delete game;
				game=newgame;
			} /* if */ 
			fclose(fp);
		} /* if */ 
	}

	show_miniatures=true;

	/* GAME_VIEW: */ 
	gv_separator_x=int(0.65*SCREEN_X);
	gv_separator_y=int(0.5*SCREEN_Y);
	GVW=new GVariableWnd(0,0,0,0,this,font);
	AVW=new AVariableWnd(0,0,0,0,this,font);
	GMW=new GMapWnd(0,0,0,0,this,font);
	GTW=new GTopWnd(0,0,0,0,this,font);

	/* MAP_VIEW: */ 
	mv_separator_x=int(0.7*SCREEN_X);
	mv_separator_y=int(0.833*SCREEN_Y);
	mv_selected=0;
	MTW=new MTopWnd(0,0,0,0,this,font);
	MVW=new MVariableWnd(0,0,0,0,this,font);
	MRW=new MRoomsWnd(0,0,0,0,this,font);
	MMW=new MMapWnd(0,0,0,0,this,font);

	/* ROOM_VIEW: */ 
	rv_separator_x=int(0.725*SCREEN_X);
	rv_separator1_y=int(0.3*SCREEN_Y);
	rv_separator2_y=int(0.833*SCREEN_Y);
	rv_selected_obj=0;
	rv_selected_tile=0;
	rv_displaying_objects=false;
	rv_editing=0;
	rv_editmode=2;
	rv_grid=0;
	rv_dragging_object=0;
	RTW=new RTopWnd(0,0,0,0,this,font);
	RVW=new RVariableWnd(0,0,0,0,this,font);
	ROW=new RObjectWnd(0,0,0,0,this,font);
	REW=new REditWnd(0,0,0,0,this,font);
	RToolW=new RToolsWnd(0,0,0,0,this,font);

	/* OBJECT_VIEW: */ 
	ov_separator1_x=int(0.3*SCREEN_X);
	ov_separator2_x=int(0.48*SCREEN_X);
	ov_separator_y=int(0.27*SCREEN_Y);
	ov_last_view=GAME_VIEW;
	ov_selected=0;
	ov_selected_ob=-1;
	ov_bitmap_category=0;
	ov_regrabbing=0;
	ov_bitmaps_button=new BButton(4,20,80,12,"BITMAPS",0);
	ov_samples_button=new BButton(108,20,80,12,"SAMPLES",1);
	ov_showing_bitmaps=true;
	OTW=new OTopWnd(0,0,0,0,this,font);
	OBW=new OBitmapWnd(0,0,0,0,this,font);
	OSW=new OSampleWnd(0,0,0,0,this,font);
	OOW=new OObjectWnd(0,0,0,0,this,font);
	OOBW=new OObjectBmpWnd(0,0,0,0,this,font);
	OPW=new OPropertiesWnd(0,0,0,0,this,font);

	/* GRABBER_VIEW: */ 
	GrabTW=new GrabTopWnd(0,0,0,0,this,font);
	GrabGW=new GrabGrabberWnd(0,0,0,0,this,font);
	GrabToolW=new GrabToolsWnd(0,0,0,0,this,font);
	grabv_bitmap=0;
	grabv_grid=0;
	grabv_grid_x=0;
	grabv_grid_y=0;
	grabv_state=0;

	/* INTERFACE VIEW: */ 
	iv_separator_x=int(0.725*SCREEN_X);
	iv_separator1_y=int(0.3*SCREEN_Y);
	iv_separator2_y=int(0.75*SCREEN_Y);
	iv_selected_obj=0;
	iv_selected_tile=0;
	iv_displaying_objects=false;	
	iv_grid=0;
	interf=0;
	iv_editing=0;
	iv_editmode=2;
	iv_dragging_object=0;
	ITW=new ITopWnd(0,0,0,0,this,font);
	IVW=new IVariableWnd(0,0,0,0,this,font);
	IOW=new IObjectWnd(0,0,0,0,this,font);
	IEW=new IEditWnd(0,0,0,0,this,font);
	IToolW=new IToolsWnd(0,0,0,0,this,font);

	/* SCRIPT EDITORS: */ 
	se_clipboard_content=0;
	se_clipboard_anim=0;
	se_clipboard_action=0;
	se_clipboard_condition=0;
	se_clipboard_value=0;

} /* EDITOR::EDITOR */ 


EDITOR::~EDITOR()
{
	delete game;

	delete GVW;
	delete AVW;
	delete GMW;
	delete GTW;

	delete MTW;
	delete MVW;
	delete MRW;
	delete MMW;

	delete RTW;
	delete RVW;
	delete ROW;
	delete REW;
	delete RToolW;

	delete OTW;
	delete OBW;
	delete OSW;
	delete OOW;
	delete OOBW;
	delete OPW;
	delete ov_bitmaps_button;
	delete ov_samples_button;

	delete GrabTW;
	delete GrabGW;
	delete GrabToolW;

	delete ITW;
	delete IVW;
	delete IOW;
	delete IEW;
	delete IToolW;

	if (se_clipboard_anim!=0) delete se_clipboard_anim;
	if (se_clipboard_action!=0) delete se_clipboard_action;
	if (se_clipboard_condition!=0) delete se_clipboard_condition;
	if (se_clipboard_value!=0) delete se_clipboard_value;
	se_clipboard_content=0;
	se_clipboard_anim=0;
	se_clipboard_action=0;
	se_clipboard_condition=0;
	se_clipboard_value=0;

} /* EDITOR */ 


bool EDITOR::cycle(void)
{
	switch(view) {
	case GAME_VIEW:
		GTW->check_mouse();
		GVW->cycle(gv_separator_x+4,20);
		AVW->cycle(gv_separator_x+4,gv_separator_y+4);
		GMW->cycle(0,20);
		if (!GTW->cycle(0,0)) return false;
		break;

	case MAP_VIEW:
		MTW->check_mouse();
		MTW->cycle(0,0);
		MVW->cycle(mv_separator_x+4,20);
		MRW->cycle(0,mv_separator_y+4);
		MMW->cycle(0,20);
		break;

	case ROOM_VIEW:
		RTW->check_mouse();
		RTW->cycle(0,0);
		RVW->cycle(rv_separator_x+4,rv_separator2_y+4);
		ROW->cycle(rv_separator_x+4,rv_separator1_y+4);
		REW->cycle(0,20);
		RToolW->cycle(rv_separator_x+4,20);
		break;

	case OBJECT_VIEW:
		{
			int mouse_x,mouse_y;
			int mouse_button;

			mouse_button=SDL_GetMouseState(&mouse_x, &mouse_y);

			OTW->check_mouse();
			OTW->cycle(0,0);
			if ((mouse_button&SDL_BUTTON(1))!=0) {
				if (mouse_x>=ov_bitmaps_button->m_x && mouse_x<ov_bitmaps_button->m_x+ov_bitmaps_button->m_dx &&
					mouse_y>=ov_bitmaps_button->m_y && mouse_y<ov_bitmaps_button->m_y+ov_bitmaps_button->m_dy) {
					ov_showing_bitmaps=true;
				} /* if */ 
				if (mouse_x>=ov_samples_button->m_x && mouse_x<ov_samples_button->m_x+ov_samples_button->m_dx &&
					mouse_y>=ov_samples_button->m_y && mouse_y<ov_samples_button->m_y+ov_samples_button->m_dy) {
					ov_showing_bitmaps=false;
				} /* if */ 
			} /* if */ 

			if (ov_showing_bitmaps) OBW->cycle(0,40);
							   else OSW->cycle(0,40);
			OOW->cycle(ov_separator1_x+4,20);
			OOBW->cycle(ov_separator2_x+4,20);
			OPW->cycle(ov_separator2_x+4,ov_separator_y+4);
		}
		break;

	case GRABBER_VIEW:
		GrabTW->check_mouse();
		GrabTW->cycle(0,0);
		GrabGW->cycle(0,40);
		GrabToolW->cycle(0,20);
		break;

	case INTERFACE_VIEW:
		ITW->check_mouse();
		ITW->cycle(0,0);
		IVW->cycle(iv_separator_x+4,iv_separator2_y+4);
		IOW->cycle(iv_separator_x+4,iv_separator1_y+4);
		IEW->cycle(0,20);
		IToolW->cycle(iv_separator_x+4,20);
		break;

	} /* switch */ 
	return true;
} /* cycle */ 


bool EDITOR::event(SDL_Event *event)
{
	switch(view) {
	case GAME_VIEW: 
		GTW->event(event);
		GVW->event(event);
		AVW->event(event);
		GMW->event(event);
		break;

	case MAP_VIEW:
		MTW->event(event);
		MVW->event(event);
		MRW->event(event);
		MMW->event(event);
		break;

	case ROOM_VIEW:
		RTW->event(event);
		RVW->event(event);
		ROW->event(event);
		REW->event(event);
		RToolW->event(event);
		break;

	case OBJECT_VIEW:
		OTW->event(event);
		if (ov_showing_bitmaps) OBW->event(event);
						   else OBW->event(event);
		OOW->event(event);
		OOBW->event(event);
		OPW->event(event);
		break;

	case GRABBER_VIEW:
		GrabTW->event(event);
		GrabTW->event(event);
		GrabGW->event(event);
		GrabToolW->event(event);
		break;

	case INTERFACE_VIEW:
		ITW->event(event);
		IVW->event(event);
		IOW->event(event);
		IEW->event(event);
		IToolW->event(event);
		break;

	} /* switch */ 
	return true;
} /* EDITOR::event */ 


void EDITOR::redraw(SDL_Surface *sfc)
{
	SDL_FillRect(sfc,0,0);

	switch(view) {
	case GAME_VIEW:
		{
			draw_line(sfc,gv_separator_x,20,gv_separator_x,sfc->h-4,0x00ffffff);
			draw_line(sfc,gv_separator_x+4,gv_separator_y,sfc->w-4,gv_separator_y,0x00ffffff);

			GVW->draw(sfc,gv_separator_x+4,20,sfc->w-(gv_separator_x+4),gv_separator_y-20);
			AVW->draw(sfc,gv_separator_x+4,gv_separator_y+4,sfc->w-(gv_separator_x+4),sfc->h-(gv_separator_y+4));
			GMW->draw(sfc,0,20,gv_separator_x-4,sfc->h-20);
			GTW->draw(sfc,0,0,sfc->w,16);
		}
		break;

	case MAP_VIEW:
		{
			draw_line(sfc,mv_separator_x,20,mv_separator_x,sfc->h-4,0x00ffffff);
			draw_line(sfc,4,mv_separator_y,mv_separator_x-4,mv_separator_y,0x00ffffff);

			MTW->draw(sfc,0,0,sfc->w,16);
			MVW->draw(sfc,mv_separator_x+4,20,sfc->w-(mv_separator_x+4),sfc->h-20);
			MRW->draw(sfc,0,mv_separator_y+4,mv_separator_x-4,sfc->h-(mv_separator_y+4));
			MMW->draw(sfc,0,20,mv_separator_x-4,mv_separator_y-24);
		}
		break;

	case ROOM_VIEW:
		{
			RTW->draw(sfc,0,0,sfc->w,16);
			RVW->draw(sfc,rv_separator_x+4,rv_separator2_y+4,sfc->w-(rv_separator_x+4),sfc->h-(rv_separator2_y+4));
			ROW->draw(sfc,rv_separator_x+4,rv_separator1_y+4,sfc->w-(rv_separator_x+4),(rv_separator2_y-rv_separator1_y)-8);
			REW->draw(sfc,0,20,rv_separator_x-4,sfc->h-24);
			RToolW->draw(sfc,rv_separator_x+4,20,sfc->w-(rv_separator_x+4),rv_separator1_y-24);

			draw_line(sfc,rv_separator_x,20,rv_separator_x,sfc->h-4,0x00ffffff);
			draw_line(sfc,rv_separator_x+4,rv_separator1_y,sfc->w-4,rv_separator1_y,0x00ffffff);
			draw_line(sfc,rv_separator_x+4,rv_separator2_y,sfc->w-4,rv_separator2_y,0x00ffffff);
		}
		break;

	case OBJECT_VIEW:
		{
			OTW->draw(sfc,0,0,sfc->w,16);
			if (ov_showing_bitmaps) OBW->draw(sfc,0,40,ov_separator1_x-4,sfc->h-40);
							   else OSW->draw(sfc,0,40,ov_separator1_x-4,sfc->h-40);
			OOW->draw(sfc,ov_separator1_x+4,20,(ov_separator2_x-ov_separator1_x)-8,sfc->h-20);
			OOBW->draw(sfc,ov_separator2_x+4,20,sfc->w-(ov_separator2_x+4),ov_separator_y-24);
			OPW->draw(sfc,ov_separator2_x+4,ov_separator_y+4,sfc->w-(ov_separator2_x+4),sfc->h-(ov_separator_y+4));

			draw_button2(sfc,ov_bitmaps_button,font,ov_showing_bitmaps);
			draw_button2(sfc,ov_samples_button,font,(ov_showing_bitmaps ? false:true));

			draw_line(sfc,ov_separator1_x,20,ov_separator1_x,sfc->h-4,0x00ffffff);
			draw_line(sfc,ov_separator2_x,20,ov_separator2_x,sfc->h-4,0x00ffffff);
			draw_line(sfc,ov_separator2_x+4,ov_separator_y,sfc->w-4,ov_separator_y,0x00ffffff);
		}
		break;

	case GRABBER_VIEW:
		{
			GrabTW->draw(sfc,0,0,sfc->w,16);
			GrabGW->draw(sfc,0,40,sfc->w,sfc->h-40);
			GrabToolW->draw(sfc,0,20,sfc->w,30);
		}
		break;

	case INTERFACE_VIEW:
		{
			ITW->draw(sfc,0,0,sfc->w,16);
			IVW->draw(sfc,iv_separator_x+4,iv_separator2_y+4,sfc->w-(iv_separator_x+4),sfc->h-(iv_separator2_y+4));
			IOW->draw(sfc,iv_separator_x+4,iv_separator1_y+4,sfc->w-(iv_separator_x+4),(iv_separator2_y-rv_separator1_y)-8);
			IEW->draw(sfc,0,20,iv_separator_x-4,sfc->h-24);
			IToolW->draw(sfc,iv_separator_x+4,20,sfc->w-(iv_separator_x+4),iv_separator1_y-24);

			draw_line(sfc,iv_separator_x,20,iv_separator_x,sfc->h-4,0x00ffffff);
			draw_line(sfc,iv_separator_x+4,iv_separator1_y,sfc->w-4,iv_separator1_y,0x00ffffff);
			draw_line(sfc,iv_separator_x+4,iv_separator2_y,sfc->w-4,iv_separator2_y,0x00ffffff);
		}
		break;

	} /* switch */ 

	SDL_SetClipRect(sfc,0);

	/* Show mouse: */ 
	draw_mouse_pointer(sfc);

	/* show FPS: */ 
	{
		char tmp[80];
		sprintf(tmp,"FPS: %i/%i (%i)",buffer_fps,buffer_fps_d,game->fonts.Length());
		print_left(tmp,font,255,255,255,sfc,SCREEN_X-100,SCREEN_Y-10);


	} 
} /* EDITOR::redraw */ 


void EDITOR::delete_tile(TILE *t)
{
	int i;
	List<OBJECT> l1;
	OBJECT *o;
	List<MAP> l2;
	MAP *m;
	List<ROOM> l3;
	ROOM *r;
	List<PLACED_TILE> l4;
	PLACED_TILE *pt;

	int j;
	bool found=false;

	if (game==0) return;

	l1.Instance(game->objects);
	l1.Rewind();
	while(!found && l1.Iterate(o)) {
		for(j=0;j<o->ntiles;j++) {
			if (o->tiles[j]==t) found=true;
		} /* for */ 
	} /* while */ 

	l2.Instance(game->maps);
	l2.Rewind();
	while(!found && l2.Iterate(m)) {
		l3.Instance(m->rooms);
		l3.Rewind();
		while(!found && l3.Iterate(r)) {
		    for(i=0;i<MAX_OBJECT_PLANES;i++) {
			l4.Instance(r->tiles[i]);
			l4.Rewind();
			while(!found && l4.Iterate(pt)) if (pt->tile==t) found=true;
		    } /* for */ 
		} /* while */ 
	} /* while */ 

	if (found) {
		/* There is an object or room that contains this bitmap: */ 
		int res;
		BChoiceDlg *dlg=new BChoiceDlg(300,200,200,50,"This TILE is still being used.","Accept",&res,font);
		dlg->Execute();
		delete dlg;
	} else {
		int i;

		for(i=0;i<TILECATEGORIES;i++) game->tiles[i].DeleteElement(t);
		delete t;
	} /* if */ 

} /* EDITOR::delete_tile */ 


void EDITOR::delete_object(OBJECT *o)
{
	int i;
	List<MAP> l2;
	MAP *m;
	List<ROOM> l3;
	ROOM *r;
	List<PLACED_OBJECT> l4;
	PLACED_OBJECT *po;

	bool found=false;

	if (game==0) return;

	l2.Instance(game->maps);
	l2.Rewind();
	while(!found && l2.Iterate(m)) {
		l3.Instance(m->rooms);
		l3.Rewind();
		while(!found && l3.Iterate(r)) {
		    for(i=0;i<MAX_OBJECT_PLANES;i++) {
			l4.Instance(r->objects[i]);
			l4.Rewind();
			while(!found && l4.Iterate(po)) if (po->object==o) found=true;
		    } /* for */ 
		} /* while */ 
	} /* while */ 

	if (found) {
		/* There is room that contains this object: */ 
		int res;
		BChoiceDlg *dlg=new BChoiceDlg(300,200,200,50,"This OBJECT is still being used.","Accept",&res,font);
		dlg->Execute();
		delete dlg;
	} else {
		game->objects.DeleteElement(o);
		delete o;
	} /* if */ 
} /* EDITOR::delete_object */ 


void EDITOR::set_all_dirty(void)
{
	GVW->set_dirty();
	AVW->set_dirty();
	GMW->set_dirty();
	GTW->set_dirty();

	MTW->set_dirty();
	MVW->set_dirty();
	MRW->set_dirty();
	MMW->set_dirty();

	RTW->set_dirty();
	RVW->set_dirty();
	ROW->set_dirty();
	REW->set_dirty();
	RToolW->set_dirty();
	RToolW->lw->set_dirty();

	OTW->set_dirty();
	OBW->set_dirty();
	OSW->set_dirty();
	OOW->set_dirty();
	OOBW->set_dirty();
	OPW->set_dirty();

	GrabTW->set_dirty();
	GrabGW->set_dirty();
	GrabToolW->set_dirty();

	ITW->set_dirty();
	IVW->set_dirty();
	IOW->set_dirty();
	IEW->set_dirty();
	IToolW->set_dirty();
	IToolW->ilw->set_dirty();
	IToolW->plw->set_dirty();

} /* EDITOR::delete_object */ 
