#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"

#include "GVariableWnd.h"
#include "GMapWnd.h"

#include "OTopWnd.h"
#include "OBitmapWnd.h"
#include "interfaces.h"

#include "editor.h"
#include "ActiveMoG2Application.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int SCREEN_X;

const int min_map_size=64;

GMapWnd::GMapWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	BButton *b;

	editor=e;

	b=new BButton(150,0,100,12,"NEW MAP",1);
	buttons.Add(b);

	b=new BButton(260,0,120,12,"FRAMES/MINIATURES",2);
	buttons.Add(b);

	b=new BButton(390,0,100,12,"PREVIEW",3);
	buttons.Add(b);

} /* GMapWnd::GMapWnd */ 


GMapWnd::~GMapWnd()
{
} /* GMapWnd::~GMapWnd */ 


bool GMapWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (mouse_button&SDL_BUTTON(3) &&
		!(old_mouse_button&SDL_BUTTON(3)) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		List<MAP> l,todelete;
		MAP *m;
		int y=TTF_FontHeight(font)+7;

		l.Instance(editor->game->maps);
		l.Rewind();
		while(l.Iterate(m)) {
			int x1,y1,x2,y2,m_dx,m_dy;

			m->CMC(&x1,&y1,&x2,&y2);
			m_dx=x2-x1;
			m_dy=y2-y1;

			if (m_dx<min_map_size) m_dx=min_map_size;
			if (m_dy<min_map_size) m_dy=min_map_size;

			if (local_mouse_y>=y && 
				local_mouse_y<=y+TTF_FontHeight(font) &&
				local_mouse_x>=m_dx+12 &&
				local_mouse_x<=m_dx+12+get_text_width(m->name,font)) {

				BStringEdDlg *dlg;
				BChoiceDlg *dlg2;
				char tmp[128];
				int res=0;
				int y,x;

				new_window_coordinates(200,60,&x,&y);

				dlg2=new BChoiceDlg(x,y,200,50,"Choose action:","Edit name","Delete",&res,font);
				dlg2->Execute();
				delete dlg2;

				if (res==1) {
					strcpy(tmp,m->name);
					new_window_coordinates(int(SCREEN_X*0.8),60,&x,&y);
					dlg=new BStringEdDlg(x,y,int(SCREEN_X*0.8),60,"Enter map name:",tmp,128," ",font);
					dlg->Execute();
					delete dlg;

					delete m->name;
					m->name=new char[strlen(tmp)+1];
					strcpy(m->name,tmp);
					dirty=true;
				} /* if */ 
				if (res==2) {
					todelete.Add(m);
					dirty=true;
				} /* if */ 
			} /* if */ 

			if (local_mouse_y>=y && 
				local_mouse_y<=y+m_dy &&
				local_mouse_x>=0 &&
				local_mouse_x<=m_dx) {

				BChoiceDlg *dlg;
				int res=0;
				int y,x;

				x=mouse_x-110;
				y=mouse_y;
				if (y>(old_y+old_dy-60)) y=old_y+old_dy-60;
				if (x>(old_x+old_dx-220)) x=old_x+old_dx-220;
				if (x<0) x=0;

				dlg=new BChoiceDlg(x,y,200,50,"Choose action:","Edit map","Delete",&res,font);
				dlg->Execute();
				delete dlg;

				if (res==1) {
					/* Go to the MAP VIEW */ 
					editor->map=m;
					editor->view=MAP_VIEW;
					editor->set_all_dirty();
				} /* if */ 
				if (res==2) {
					todelete.Add(m);
					dirty=true;
				} /* if */ 
			} /* if */ 

			y+=m_dy+12;
		} /* while */ 	

		while(!todelete.EmptyP()) {
			MAP *m;
			m=todelete.ExtractIni();
			editor->game->maps.DeleteElement(m);
			delete m;
		} /* while */ 

	} /* if */ 

	return true;
} /* GMapWnd::cycle */ 


void GMapWnd::contens_size(int *dx,int *dy)
{
	int max_x=0;
	int max_y=0;
	int y;
	MAP *m;
	List<MAP> l;

	
	BWindow::contens_size(dx,dy);
	
	l.Instance(editor->game->maps);
	l.Rewind();
	y=TTF_FontHeight(font)+7;;
	while(l.Iterate(m)) {
		int x1,y1,x2,y2,m_dx,m_dy;

		m->CMC(&x1,&y1,&x2,&y2);
		m_dx=x2-x1;
		m_dy=y2-y1;

		if (m_dx<min_map_size) m_dx=min_map_size;
		if (m_dy<min_map_size) m_dy=min_map_size;
		y+=m_dy+12;

		if (max_x<m_dx+12+get_text_width(m->name,font)) 
			max_x=m_dx+12+get_text_width(m->name,font);
		if (max_x<m_dx+12+get_text_width("Rooms: 000",font)) 
			max_x=m_dx+12+get_text_width("Rooms: 000",font);
		if (max_x<m_dx+12+get_text_width("Variables: 000",font)) 
			max_x=m_dx+12+get_text_width("Variables: 000",font);
	} /* while */ 
	if (y>max_y) max_y=y;

	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* GMapWnd::contens_size */ 


void GMapWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	int y;
	MAP *m;
	List<MAP> l;
	char tmp[80];

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	print_left("GAME MAPS:",font,255,255,255,sfc,3-stx,3-sty);

	l.Instance(editor->game->maps);
	l.Rewind();
	y=TTF_FontHeight(font)+7;;
	while(l.Iterate(m)) {
		/* Draw MAP info: */ 
		int x1,y1,x2,y2,m_dx,m_dy;

		m->CMC(&x1,&y1,&x2,&y2);
		m_dx=x2-x1;
		m_dy=y2-y1;

		if (m_dx<min_map_size) m_dx=min_map_size;
		if (m_dy<min_map_size) m_dy=min_map_size;

		draw_rectangle(sfc,3-stx,y-sty,m_dx+4,m_dy+4,0x00ffffffL);
		if (editor->show_miniatures) m->draw(sfc,5-stx,y+2-sty,m_dx,m_dy,0,0,0.0625f);
								else m->draw_frame(sfc,5-stx,y+2-sty,m_dx,m_dy,0,0,0.0625f);

		print_left(m->name,font,255,255,255,sfc,m_dx+12-stx,y-sty);

		sprintf(tmp,"Rooms: %i",m->rooms.Length());
		print_left(tmp,font,255,255,255,sfc,m_dx+12-stx,y+TTF_FontHeight(font)+2-sty);

		sprintf(tmp,"Variables: %i",m->map_variables.Length());
		print_left(tmp,font,255,255,255,sfc,m_dx+12-stx,y+(TTF_FontHeight(font)+2)*2-sty);

		y+=m_dy+12;
	} /* while */ 

} /* GMapWnd::draw_contens */ 


bool GMapWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		/* New Map!!: */ 
		{
			MAP *m;
			List<MAP> l;
			int i=0;
			char tmp[80];
			bool added=false,found;

			do{
				if (i==0) {
					sprintf(tmp,"new_map");
				} else {
					sprintf(tmp,"new_map_%i",i);
				} /* if */ 

				l.Instance(editor->game->maps);
				l.Rewind();
				found=false;
				while(!found && l.Iterate(m)) {
					if (m->cmp(tmp)) found=true;
				} /* while */ 

				if (!found) {
					m=new MAP(tmp,editor->game);
					editor->game->maps.Add(m);
					added=true;
				} /* if */ 

				i++;
			}while(!added);

			set_dirty();
		}
		break;
	case 2:
		{
			/* FRAMES/MINIATURES: */ 
			if (editor->show_miniatures) editor->show_miniatures=false;
									else editor->show_miniatures=true;

			set_dirty();
		}
		break;

	case 3:
		/* COMPLETE GAME PREVIEW!!!!!! */ 
		{
			ActiveMOG2Application *app=new ActiveMOG2Application(editor->game);
			app->Execute();
			delete app;
		}
		break;

	default:
		break;
	} /* switch */ 

	return true;
} /* GMapWnd::button_pressed */ 



bool GMapWnd::event(SDL_Event *event)
{
	if (active_window!=this) return true;

	if (event->type==SDL_KEYDOWN) {
		/* Keyboard event */ 
		if (event->key.keysym.sym==SDLK_UP) {
			old_sty-=TILESIZE;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_DOWN) {
			old_sty+=TILESIZE;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_LEFT) {
			old_stx-=TILESIZE;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_RIGHT) {
			old_stx+=TILESIZE;
		} /* if */ 

		if (event->key.keysym.sym==SDLK_PAGEUP) {
			old_sty-=TILESIZE*16;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_PAGEDOWN) {
			old_sty+=TILESIZE*16;
		} /* if */ 

		set_dirty();
    } /* if */ 

	return true;
} /* GMapWnd::Event */ 