#ifdef _WIN32
#include "windows.h"
#include "direct.h"
#endif

#include "string.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"

#include "BWindow.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "AnimScriptWnd.h"
#include "LinksWnd.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif


LinksWnd::LinksWnd(int x,int y,int dx,int dy,OBJECT *o,int t,int sf,TTF_Font *font) : BActiveWindow(x,y,dx,dy,font)
{
	BButton *b;
	int but_dx;
	int i;

	but_dx=get_text_width("RESET",font)+10;
	b=new BButton(20,20,but_dx,12,"RESET",1);
	buttons.Add(b);

	but_dx=get_text_width("+",font)+10;
	b=new BButton(10,60,but_dx,12,"+",4);
	buttons.Add(b);
	but_dx=get_text_width("-",font)+10;
	b=new BButton(30,60,but_dx,12,"-",5);
	buttons.Add(b);

	but_dx=get_text_width("<",font)+10;
	b=new BButton(10,80,but_dx,12,"<",7);
	buttons.Add(b);
	but_dx=get_text_width(">",font)+10;
	b=new BButton(30,80,but_dx,12,">",6);
	buttons.Add(b);


	but_dx=get_text_width("ACCEPT",font)+10;
	b=new BButton(20,120,but_dx,12,"ACCEPT",2);
	buttons.Add(b);

	but_dx=get_text_width("CANCEL",font)+10;
	b=new BButton(20,140,but_dx,12,"CANCEL",3);
	buttons.Add(b);

	object=o;
	tile=t;
	scaling_factor=sf;

	n_links=object->nlinks[tile];
	for(i=0;i<n_links;i++) {
		link_x[i]=object->link_x[tile][i];
		link_y[i]=object->link_y[tile][i];
	} /* for */ 

	if (n_links>0) act_link=0;
			  else act_link=-1;

} /* LinksWnd::LinksWnd */ 


LinksWnd::~LinksWnd()
{

} /* LinksWnd::~LinksWnd */ 


void LinksWnd::contens_size(int *dx,int *dy)
{
	int max_x,max_y;
	
	max_x=100+(object->tiles[tile]->orig->w*scaling_factor);
	max_y=20+(object->tiles[tile]->orig->h*scaling_factor);

	if (max_x>old_dx) *dx=max_x;
				 else *dx=old_dx;
	if (max_y>old_dy) *dy=max_y;
				 else *dy=old_dy;

} /* LinksWnd::contens_size */ 



bool LinksWnd::cycle(int stx,int sty)
{
	if (!BActiveWindow::cycle(stx,sty)) return false;

	if (local_mouse_x>=old_stx &&
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy &&
		local_mouse_x>=90 && 
		local_mouse_y>=10 &&
		local_mouse_x<90+(object->tiles[tile]->orig->w*scaling_factor) &&
		local_mouse_y<10+(object->tiles[tile]->orig->h*scaling_factor) &&
		act_link>=0) {
		if (mouse_button&SDL_BUTTON(1) &&
			!(old_mouse_button&SDL_BUTTON(1))) {
			link_x[act_link]=(local_mouse_x-90)/scaling_factor;
			link_y[act_link]=(local_mouse_y-10)/scaling_factor;
			set_dirty();
		} /* if */ 
	} /* if */ 

	return true;
} /* LinksWnd::cycle */ 



void LinksWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	int i;
	SDL_Rect r;

	SDL_FillRect(sfc,0,0x0000007f);

	r.x=1-stx;
	r.y=1-sty;
	r.w=dx-2;
	r.h=dy-2;
	SDL_FillRect(sfc,&r,0x000000ff);

	BActiveWindow::draw_contens(sfc,stx,sty,dx,dy);

	/* Show numbers */ 
	{
		char tmp[80];

		sprintf(tmp,"%i",n_links);
		print_left(tmp,font,255,255,255,sfc,stx+60,sty+64);

		if (act_link>=0) {
			sprintf(tmp,"%i",act_link);
			print_left(tmp,font,255,255,255,sfc,stx+60,sty+84);
		} else {
			print_left("--",font,255,255,255,sfc,stx+60,sty+84);
		} /* if */ 
	}

	r.x=88-stx;
	r.y=8-sty;
	r.w=(object->tiles[tile]->orig->w*scaling_factor)+4;
	r.h=(object->tiles[tile]->orig->h*scaling_factor)+4;
	SDL_FillRect(sfc,&r,0);

	r.x=88-stx;
	r.y=8-sty;
	r.w=(object->tiles[tile]->orig->w*scaling_factor)+4;
	r.h=(object->tiles[tile]->orig->h*scaling_factor)+4;

	if (scaling_factor!=1) object->tiles[tile]->draw_transform(r.x+2,r.y+2,sfc,float(scaling_factor),0);
					  else object->tiles[tile]->draw(r.x+2,r.y+2,sfc);

	for(i=0;i<n_links;i++) {
		if (i!=act_link) {
			draw_line(sfc,r.x,r.y+2+(link_y[i]*scaling_factor+(scaling_factor-1)/2),
						  r.x+r.w,r.y+2+(link_y[i]*scaling_factor+(scaling_factor-1)/2),0x00ffffff);
			draw_line(sfc,r.x+2+(link_x[i]*scaling_factor+(scaling_factor-1)/2),r.y,
						  r.x+2+(link_x[i]*scaling_factor+(scaling_factor-1)/2),r.y+r.h,0x00ffffff);
		} /* if */ 
	} /* if */ 

	if (act_link>=0) {
		draw_line(sfc,r.x,r.y+2+(link_y[act_link]*scaling_factor+(scaling_factor-1)/2),
					  r.x+r.w,r.y+2+(link_y[act_link]*scaling_factor+(scaling_factor-1)/2),0x00ff0000);
		draw_line(sfc,r.x+2+(link_x[act_link]*scaling_factor+(scaling_factor-1)/2),r.y,
					  r.x+2+(link_x[act_link]*scaling_factor+(scaling_factor-1)/2),r.y+r.h,0x00ff0000);
	} /* if */ 



} /* LinksWnd::draw_contens */ 


bool LinksWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		link_x[act_link]=0;
		link_y[act_link]=0;
		set_dirty();
		return true;
		break;

	case 2:
		{
			int i;

			object->nlinks[tile]=n_links;
			for(i=0;i<n_links;i++) {
				object->link_x[tile][i]=link_x[i];
				object->link_y[tile][i]=link_y[i];
			} /* for */ 
		} 
		return false;
		break;

	case 3:
		return false;
		break;

	case 4:
		if (n_links<MAX_LINKS) {
			link_x[n_links]=0;
			link_y[n_links]=0;
			n_links++;
		} /* if */ 
		if (act_link<0) act_link=0;
		set_dirty();
		return true;
		break;
	case 5:
		if (n_links>=0) n_links--;
		if (act_link>=n_links) act_link=n_links-1;
		set_dirty();
		return true;
		break;

	case 6:
		if (act_link<n_links-1) act_link++;
		set_dirty();
		return true;
		break;
	case 7:
		if (act_link>0) act_link--;
		set_dirty();
		return true;
		break;

	} /* switch */ 

	return false;
} /* LinksWnd::button_pressed */ 

