#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_image.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"

#include "BWindow.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BFileDlg.h"
#include "LinksWnd.h"
#include "OPartsWnd.h"

#include "OObjectBmpWnd.h"
#include "interfaces.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int SCREEN_X,SCREEN_Y;

OObjectBmpWnd::OObjectBmpWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font)
{
	BButton *b;

	editor=e;

	b=new BButton(4,20,40,12,"ADD",1);
	buttons.Add(b);

	b=new BButton(4,40,40,12,"SUB",2);
	buttons.Add(b);

	b=new BButton(4,60,40,12,"PARTS",3);
	buttons.Add(b);


} /* OObjectBmpWnd::OObjectBmpWnd */ 


OObjectBmpWnd::~OObjectBmpWnd()
{
} /* OObjectBmpWnd::~OObjectBmpWnd */ 


bool OObjectBmpWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (editor->ov_selected!=0) {

		if (mouse_button&SDL_BUTTON(1) &&
			!(old_mouse_button&SDL_BUTTON(1)) &&
			local_mouse_x>=old_stx && 
			local_mouse_y>=old_sty &&
			local_mouse_x<old_endx &&
			local_mouse_y<old_endy) {
			int i;
			int x=50;
			char tmp[80];

			for(i=0;i<editor->ov_selected->ntiles;i++) {
				int bdx=4,bdy=4,tdx;
				sprintf(tmp,"BITMAP %i",i);
				tdx=get_text_width(tmp,font);

				if (local_mouse_x>=x &&
					local_mouse_x<x+tdx &&
					local_mouse_y>=16 &&
					local_mouse_y<=16+TTF_FontHeight(font)) {
					editor->ov_selected_ob=i;
					set_dirty();
				} /* if */ 

				if (editor->ov_selected->tiles[i]!=0) {
					bdx=editor->ov_selected->tiles[i]->r.w+4;
					bdy=editor->ov_selected->tiles[i]->r.h+4;
				} /* if */ 

				if (local_mouse_x>=x &&
					local_mouse_x<x+bdx &&
					local_mouse_y>=22+TTF_FontHeight(font) &&
					local_mouse_y<22+TTF_FontHeight(font)+bdy) {
					editor->ov_selected_ob=i;
					set_dirty();
				} /* if */ 

				if (tdx>bdx) bdx=tdx;
				x+=bdx+10;				
			} /* for */ 
		} /* if */ 

		if (mouse_button&SDL_BUTTON(3) &&
			!(old_mouse_button&SDL_BUTTON(3)) &&
			local_mouse_x>=old_stx && 
			local_mouse_y>=old_sty &&
			local_mouse_x<old_endx &&
			local_mouse_y<old_endy) {
			int i;
			int x=50;
			char tmp[80];
			int selected=-1;

			for(i=0;i<editor->ov_selected->ntiles;i++) {
				int bdx=4,bdy=4,tdx;
				sprintf(tmp,"BITMAP %i",i);
				tdx=get_text_width(tmp,font);

				if (local_mouse_x>=x &&
					local_mouse_x<x+tdx &&
					local_mouse_y>=16 &&
					local_mouse_y<=16+TTF_FontHeight(font)) {
					selected=i;
				} /* if */ 

				if (editor->ov_selected->tiles[i]!=0) {
					bdx=editor->ov_selected->tiles[i]->r.w+4;
					bdy=editor->ov_selected->tiles[i]->r.h+4;
				} /* if */ 

				if (local_mouse_x>=x &&
					local_mouse_x<x+bdx &&
					local_mouse_y>=22+TTF_FontHeight(font) &&
					local_mouse_y<22+TTF_FontHeight(font)+bdy) {
					selected=i;
				} /* if */ 

				if (tdx>bdx) bdx=tdx;
				x+=bdx+10;				
			} /* for */ 

			if (selected!=-1) {
				/* Contextual menu: */ 
				BVChoiceDlg *dlg;
				int res=0;

				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose action:","Unassign\nAssign\nSet LINKS\n",&res,font);
				dlg->Execute();
				delete dlg;

				if (res==1) {
					editor->ov_selected->tiles[selected]=0;
					set_dirty();
				} /* if */ 

				if (res==2) {
					editor->ov_selected_ob=selected;
					set_dirty();
				} /* if */ 

				if (res==3) {
					if (editor->ov_selected->tiles[selected]!=0) {
						/* Set LINKS: */ 
						LinksWnd *dlg2;
						int y,x,dx,dy,sf=4;
						dx=(editor->ov_selected->tiles[selected]->r.w*sf)+100;
						dy=(editor->ov_selected->tiles[selected]->r.h*sf)+20;
						while(sf>1 && (dx>640 || dx>480)) {
							sf--;
							dx=(editor->ov_selected->tiles[selected]->r.w*sf)+100;
							dy=(editor->ov_selected->tiles[selected]->r.h*sf)+20;
						} /* while */ 

						if (dx>SCREEN_X) dx=SCREEN_X;
						if (dy>SCREEN_Y) dy=SCREEN_Y;
						if (dy<160) dy=160;

						x=mouse_x-dx/2;
						y=mouse_y;
						if (y>SCREEN_Y-dy) y=SCREEN_Y-dy;
						if (x>SCREEN_X-dx) x=SCREEN_X-dx;
						if (x<0) x=0;
						if (y<0) y=0;

						dlg2=new LinksWnd(x,y,dx,dy,editor->ov_selected,selected,sf,font);
						dlg2->Execute();
						delete dlg2;
					} else {
						BChoiceDlg *dlg2=new BChoiceDlg(240,40,200,50,"Assign a bitmap first","Accept",&res,editor->font);
						dlg2->Execute();
						delete dlg2;
					} /* if */ 
					set_dirty();
				} /* if */ 
			} /* if */ 
		} /* if */ 
	} /* if */ 

	return true;
} /* OObjectBmpWnd::cycle */ 


void OObjectBmpWnd::contens_size(int *dx,int *dy)
{
	int max_x=0;
	int max_y=0;
	
	BWindow::contens_size(dx,dy);

	if (editor->ov_selected!=0) {
		int i;
		int x=50;
		char tmp[80];

		max_x=4+get_text_width("OBJECT BITMAPS:",font);

		for(i=0;i<editor->ov_selected->ntiles;i++) {
			int bdx=0,bdy=0,tdx;
			sprintf(tmp,"BITMAP %i",i);
			tdx=get_text_width(tmp,font);

			if (editor->ov_selected->tiles[i]!=0) {
				bdx=editor->ov_selected->tiles[i]->r.w;
				bdy=editor->ov_selected->tiles[i]->r.h;
			} /* if */ 

			if (24+TTF_FontHeight(font)+bdy>max_y) max_y=24+TTF_FontHeight(font)+bdy;

			bdx+=4;
			if (tdx>bdx) bdx=tdx;
			x+=bdx+10;
		} /* for */ 
		if (x>max_x) max_x=x;

		if (max_x>*dx) *dx=max_x;
		if (max_y>*dy) *dy=max_y;
	} /* if */ 
} /* OObjectBmpWnd::contens_size */ 


void OObjectBmpWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_FillRect(sfc,0,0);


	if (editor->ov_selected!=0) {
		int i;
		int x=50;
		char tmp[80];

		BWindow::draw_contens(sfc,stx,sty,dx,dy);
	
		print_left("OBJECT BITMAPS:",font,255,255,255,sfc,4-stx,4-sty);

		for(i=0;i<editor->ov_selected->ntiles;i++) {
			int bdx=0,bdy=0;
			
			sprintf(tmp,"BITMAP %i",i);
			print_left(tmp,font,255,255,255,sfc,x-stx,16-sty);

			if (editor->ov_selected->tiles[i]!=0) {
				bdx=editor->ov_selected->tiles[i]->r.w;
				bdy=editor->ov_selected->tiles[i]->r.h;
	
				editor->ov_selected->tiles[i]->draw(x+2-stx,22+TTF_FontHeight(font)-sty,sfc);

				/* Draw the Hot Spot (i.e. LINK 0): */ 
				putpixel(sfc,(x+2-stx)-4,(22+TTF_FontHeight(font)-sty)+editor->ov_selected->link_y[i][0],0x00ffffff);
				putpixel(sfc,(x+2-stx)+editor->ov_selected->link_x[i][0],(22+TTF_FontHeight(font)-sty)-4,0x00ffffff);

			} /* if */ 
			if (editor->ov_selected_ob==i) draw_rectangle(sfc,x-stx,20+TTF_FontHeight(font)-sty,bdx+3,bdy+3,0x00ff0000);
									  else draw_rectangle(sfc,x-stx,20+TTF_FontHeight(font)-sty,bdx+3,bdy+3,0x00ffffff);

			bdx+=4;
			if (get_text_width(tmp,font)>bdx) bdx=get_text_width(tmp,font);
			x+=bdx+10;
		} /* for */ 

	} /* if */ 

} /* OObjectBmpWnd::draw_contens */ 


bool OObjectBmpWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		/* ADD BITMAP: */ 
		if (editor->ov_selected!=0) {
			TILE **nt;
//			int *n_hx,*n_hy;
			int **l_x,**l_y;
			int *nl;
			int i;


			nt=new TILE *[editor->ov_selected->ntiles+1];
//			n_hx=new int [editor->ov_selected->ntiles+1];
//			n_hy=new int [editor->ov_selected->ntiles+1];
			nl=new int[editor->ov_selected->ntiles+1];
			l_x=new int *[editor->ov_selected->ntiles+1];
			l_y=new int *[editor->ov_selected->ntiles+1];
			
			for(i=0;i<editor->ov_selected->ntiles;i++) {
				nt[i]=editor->ov_selected->tiles[i];
//				n_hx[i]=editor->ov_selected->hot_x[i];
//				n_hy[i]=editor->ov_selected->hot_y[i];
				nl[i]=editor->ov_selected->nlinks[i];
				l_x[i]=editor->ov_selected->link_x[i];
				l_y[i]=editor->ov_selected->link_y[i];
			} /* for */ 
			nt[editor->ov_selected->ntiles]=0;
//			n_hx[editor->ov_selected->ntiles]=0;
//			n_hy[editor->ov_selected->ntiles]=0;
			nl[editor->ov_selected->ntiles]=1;
			l_x[editor->ov_selected->ntiles]=new int[MAX_LINKS];
			l_x[editor->ov_selected->ntiles][0]=0;
			l_y[editor->ov_selected->ntiles]=new int[MAX_LINKS];
			l_y[editor->ov_selected->ntiles][0]=0;

			if (editor->ov_selected->tiles!=0) {
				delete []editor->ov_selected->tiles;
//				delete editor->ov_selected->hot_x;
//				delete editor->ov_selected->hot_y;
				delete []editor->ov_selected->nlinks;
				delete []editor->ov_selected->link_x;
				delete []editor->ov_selected->link_y;
			} /* f */ 
			editor->ov_selected->tiles=nt;
//			editor->ov_selected->hot_x=n_hx;
//			editor->ov_selected->hot_y=n_hy;
			editor->ov_selected->nlinks=nl;
			editor->ov_selected->link_x=l_x;
			editor->ov_selected->link_y=l_y;
			editor->ov_selected->ntiles++;
		} /* if */ 
		editor->ov_selected_ob=-1;
		set_dirty();
		break;

	case 2:
		/* SUB BITMAP: */ 
		if (editor->ov_selected!=0) {
			if (editor->ov_selected->ntiles>0) {
				TILE **nt;
//				int *n_hx,*n_hy;
				int **l_x,**l_y;
				int *nl;
				int i;

				nt=new TILE *[editor->ov_selected->ntiles-1];
//				n_hx=new int [editor->ov_selected->ntiles-1];
//				n_hy=new int [editor->ov_selected->ntiles-1];
				nl=new int[editor->ov_selected->ntiles-1];
				l_x=new int *[editor->ov_selected->ntiles-1];
				l_y=new int *[editor->ov_selected->ntiles-1];
			
				for(i=0;i<editor->ov_selected->ntiles-1;i++) {
					nt[i]=editor->ov_selected->tiles[i];
//					n_hx[i]=editor->ov_selected->hot_x[i];
//					n_hy[i]=editor->ov_selected->hot_y[i];
					nl[i]=editor->ov_selected->nlinks[i];
					l_x[i]=editor->ov_selected->link_x[i];
					l_y[i]=editor->ov_selected->link_y[i];
				} /* for */ 
//				delete editor->ov_selected->tiles[editor->ov_selected->ntiles-1];
				delete editor->ov_selected->link_x[editor->ov_selected->ntiles-1];
				delete editor->ov_selected->link_y[editor->ov_selected->ntiles-1];
				if (editor->ov_selected->tiles!=0) {
					delete editor->ov_selected->tiles;
//					delete editor->ov_selected->hot_x;
//					delete editor->ov_selected->hot_y;
					delete editor->ov_selected->nlinks;
					delete editor->ov_selected->link_x;
					delete editor->ov_selected->link_y;
				} /* f */ 
				editor->ov_selected->tiles=nt;
//				editor->ov_selected->hot_x=n_hx;
//				editor->ov_selected->hot_y=n_hy;
				editor->ov_selected->nlinks=nl;
				editor->ov_selected->link_x=l_x;
				editor->ov_selected->link_y=l_y;
				editor->ov_selected->ntiles--;
			} /* if */ 
		} /* if */ 
		editor->ov_selected_ob=-1;
		set_dirty();
		break;

	case 3:
		/* PARTS: */ 
		if (editor->ov_selected->ntiles>0) {
			OPartsWnd *dlg;
			int y,x,dx,dy;
			dx=512;
			dy=384;

			x=mouse_x-dx/2;
			y=mouse_y;
			if (y>SCREEN_Y-dy) y=SCREEN_Y-dy;
			if (x>SCREEN_X-dx) x=SCREEN_X-dx;
			if (x<0) x=0;
			if (y<0) y=0;

			dlg=new OPartsWnd(x,y,dx,dy,editor->ov_selected,4,font);
			dlg->Execute();
			delete dlg;
			editor->set_all_dirty();
		} else {
			BChoiceDlg *dlg=new BChoiceDlg(240,40,320,50,"Please add some bitmaps to the object first","Accept",0,editor->font);
			dlg->Execute();
			delete dlg;
		} /* if */ 

		break;

	default:
		break;
	} /* switch */ 

	return true;
} /* OObjectBmpWnd::button_pressed */ 
