#ifdef _WIN32
#include "windows.h"
#include "direct.h"
#endif

#include "string.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"

#include "BWindow.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "AnimScriptWnd.h"
#include "TileViewWnd.h"
#include "TileSelectionWnd.h"
#include "BIntegerEdDlg.h"
#include "OPartsWnd.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int SCREEN_X,SCREEN_Y;

OPartsWnd::OPartsWnd(int x,int y,int dx,int dy,OBJECT *o,int sf,TTF_Font *font) : BActiveWindow(x,y,dx,dy,font)
{
	int i;
	BButton *b;
	int but_dx;

	but_dx=get_text_width("ACCEPT",font)+10;
	b=new BButton(20,10,but_dx,12,"ACCEPT",1);
	buttons.Add(b);

	but_dx=get_text_width("CANCEL",font)+10;
	b=new BButton(20,30,but_dx,12,"CANCEL",2);
	buttons.Add(b);

	but_dx=get_text_width("VIEW",font)+10;
	b=new BButton(20,50,but_dx,12,"VIEW",3);
	buttons.Add(b);

	but_dx=get_text_width("TILE",font)+10;
	b=new BButton(20,70,but_dx,12,"TILE",4);
	buttons.Add(b);

	but_dx=get_text_width("+",font)+10;
	b=new BButton(10,100,but_dx,12,"+",5);
	buttons.Add(b);

	but_dx=get_text_width("-",font)+10;
	b=new BButton(30,100,but_dx,12,"-",6);
	buttons.Add(b);

	/* Next/prev part: */ 
	but_dx=get_text_width("<",font)+10;
	b=new BButton(10,120,but_dx,12,"<",7);
	buttons.Add(b);

	but_dx=get_text_width(">",font)+10;
	b=new BButton(30,120,but_dx,12,">",8);
	buttons.Add(b);

	/* Link to: */ 
	but_dx=get_text_width("<",font)+10;
	b=new BButton(10,160,but_dx,12,"<",9);
	buttons.Add(b);

	but_dx=get_text_width(">",font)+10;
	b=new BButton(30,160,but_dx,12,">",10);
	buttons.Add(b);

	/* Links to join: */ 
	but_dx=get_text_width("<",font)+10;
	b=new BButton(10,180,but_dx,12,"<",11);
	buttons.Add(b);

	but_dx=get_text_width(">",font)+10;
	b=new BButton(30,180,but_dx,12,">",12);
	buttons.Add(b);

	but_dx=get_text_width("<",font)+10;
	b=new BButton(10,200,but_dx,12,"<",13);
	buttons.Add(b);

	but_dx=get_text_width(">",font)+10;
	b=new BButton(30,200,but_dx,12,">",14);
	buttons.Add(b);

	object=o;
	scaling_factor=sf;

	/* Transfer the current configuration from the object: */ 
	nparts=object->nparts;
	for(i=0;i<nparts;i++) {
		tile[i]=object->default_tile[i];
		link_part[i]=object->link_part[i];
		link_a[i]=object->link_a[i];
		link_b[i]=object->link_b[i];
		part_precedence[i]=object->part_precedence[i];
	} /* for */ 

	show_tiles=false;
	act_part=0;

	centerx=0;
	centery=0;
	width=0;
	height=0;
} /* OPartsWnd::OPartsWnd */ 


OPartsWnd::~OPartsWnd()
{

} /* OPartsWnd::~OPartsWnd */ 


void OPartsWnd::contens_size(int *dx,int *dy)
{
	int max_x=96+width,max_y=8+height;

	BWindow::contens_size(dx,dy);
	
	if (max_x>old_dx) *dx=max_x;
				 else *dx=old_dx;
	if (max_y>old_dy) *dy=max_y;
				 else *dy=old_dy;

} /* OPartsWnd::contens_size */ 



bool OPartsWnd::cycle(int stx,int sty)
{
	int i,minx=0,miny=0,maxx=0,maxy=0;

	if (!BActiveWindow::cycle(stx,sty)) return false;

	for(i=0;i<nparts;i++) {
		int cx=0,cy=0;
                int t=0;
		int tdx,tdy,hx,hy;

		if (i!=0) {
//			cx=hot_x_place[i]*scaling_factor;
//			cy=hot_y_place[i]*scaling_factor;
		} /* if */ 

        if (tile[i]>=0) t=tile[i];
		tdx=(object->tiles[t]->orig->w)*scaling_factor;
		tdy=(object->tiles[t]->orig->h)*scaling_factor;
		hx=object->link_x[t][0]*scaling_factor;
		hy=object->link_y[t][0]*scaling_factor;

		if (cx-hx<minx) minx=cx-hx;
		if (cy-hy<miny) miny=cy-hy;
		if ((cx-hx)+tdx>maxx) maxx=(cx-hx)+tdx;
		if ((cy-hy)+tdy>maxy) maxy=(cy-hy)+tdy;	
	} /* for */ 

	width=(maxx-minx)+32*scaling_factor;
	height=(maxy-miny)+32*scaling_factor;

	if (centerx!=96-(minx-16*scaling_factor)) {
		centerx=96-(minx-16*scaling_factor);
		set_dirty();
	} /* if */ 

	if (centery!=8-(miny-16)) {
		centery=8-(miny-16);
		set_dirty();
	} /* if */ 

	if (mouse_button&SDL_BUTTON(1) &&
		!(old_mouse_button&SDL_BUTTON(1))) {

		if (local_mouse_x>=10 &&
			local_mouse_x<60+get_text_width("999999",font) &&
			local_mouse_y>=224 &&
			local_mouse_y<224+TTF_FontHeight(font)) {
			BIntegerEdDlg *dlg;
			int y,x;

			x=mouse_x-110;
			y=mouse_y;
			if (y>(old_y+old_dy-60)) y=old_y+old_dy-60;
			if (x>(old_x+old_dx-140)) x=old_x+old_dx-140;
			if (x<0) x=0;

			dlg=new BIntegerEdDlg(x,y,140,60,"Enter precedence:",&part_precedence[act_part],6,font);
			dlg->Execute();
			delete dlg;

			set_dirty();
		} /* if */ 

		if (local_mouse_x>=old_stx &&
			local_mouse_y>=old_sty &&
			local_mouse_x<old_endx &&
			local_mouse_y<old_endy &&
			local_mouse_x>=96 && 
			local_mouse_x<96+width &&
			local_mouse_y>=8 &&
			local_mouse_y<8+height &&
			act_part!=0) {

//			hot_x_place[act_part]=((local_mouse_x-centerx)+scaling_factor/2)/scaling_factor;
//			hot_y_place[act_part]=((local_mouse_y-centery)+scaling_factor/2)/scaling_factor;
			set_dirty();
		} /* if */ 
	} /* if */ 

	return true;
} /* OPartsWnd::cycle */ 



void OPartsWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_Rect r;
	int i;
	int order[MAX_PARTS];

	SDL_FillRect(sfc,0,0x0000007f);

	r.x=1-stx;
	r.y=1-sty;
	r.w=dx-2;
	r.h=dy-2;
	SDL_FillRect(sfc,&r,0x000000ff);

	BActiveWindow::draw_contens(sfc,stx,sty,dx,dy);

	/* Show numbers */ 
	{
		char tmp[80];

		sprintf(tmp,"%i",nparts);
		print_left(tmp,font,255,255,255,sfc,60-stx,104-sty);

		sprintf(tmp,"%i",act_part);
		print_left(tmp,font,255,255,255,sfc,60-stx,124-sty);

		print_left("PART LA LB",font,255,255,255,sfc,60-stx,144-sty);

		sprintf(tmp,"%i",link_part[act_part]);
		print_left(tmp,font,255,255,255,sfc,60-stx,164-sty);
		sprintf(tmp,"%i",link_a[act_part]);
		print_left(tmp,font,255,255,255,sfc,60-stx,184-sty);
		sprintf(tmp,"%i",link_b[act_part]);
		print_left(tmp,font,255,255,255,sfc,60-stx,204-sty);

		print_left("PREC:",font,255,255,255,sfc,10-stx,224-sty);
		sprintf(tmp,"%i",part_precedence[act_part]);
		print_left(tmp,font,255,255,255,sfc,60-stx,224-sty);

	}


	r.x=96-stx;
	r.y=8-sty;
	r.w=width;
	r.h=height;
	SDL_FillRect(sfc,&r,0);

	if (nparts>1) {
		int tmp;
		bool change=false;
		/* Sort for precedences: */ 
		for(i=0;i<nparts;i++) order[i]=i;

		do{
			change=false;
			for(i=0;i<nparts-1;i++) {
				if (part_precedence[order[i]]>part_precedence[order[i+1]]) {
					tmp=order[i];
					order[i]=order[i+1];
					order[i+1]=tmp;
					change=true;
				} /* if */ 
			} /* for */ 
		}while(change);
	} else {
		order[0]=0;
	} /* if */ 

	/* Draw bitmaps: */ 
	for(i=0;i<nparts;i++) {
		int cx=centerx,cy=centery;
		int colour=0xffffffff;
		int factor=0;
		int c=0;
		int j=order[i];

		if (j==act_part) {
			colour=0xff<<(ROFFSET*8);

			factor=(SDL_GetTicks()/2)%512;
			if (factor>=256) factor=512-factor;
			factor=(factor*100)/256;
		} /* if */ 

		while(j!=0 && c<nparts) {
			if (object->nlinks[tile[link_part[j]]]>link_a[j]) {
				cx+=object->link_x[tile[link_part[j]]][link_a[j]]*scaling_factor;
				cy+=object->link_y[tile[link_part[j]]][link_a[j]]*scaling_factor;
			} /* if */ 
			if (object->nlinks[tile[j]]>link_b[j]) {
				cx-=object->link_x[tile[j]][link_b[j]]*scaling_factor;
				cy-=object->link_y[tile[j]][link_b[j]]*scaling_factor;
			} /* if */ 

			j=link_part[j];
			c++;
		} /* while */ 

		j=order[i];

		if (show_tiles) {
            int t=0;
			int tdx,tdy,hx,hy;

            if (tile[j]>=0) t=tile[j];
			tdx=object->tiles[t]->orig->w*scaling_factor;
			tdy=object->tiles[t]->orig->h*scaling_factor;
			hx=object->link_x[0][0]*scaling_factor;
			hy=object->link_y[0][0]*scaling_factor;
			
//			draw_rectangle(sfc,cx-(hx+stx),cy-(hy+sty),tdx,tdy,colour);
            if (tile[j]>=0) object->tiles[tile[j]]->draw_shaded_transform(cx-(hx+stx),cy-(hy+sty),sfc,float(scaling_factor),0,factor,255,255,255,-1);

			draw_line(sfc,(cx-stx)-1,(cy-sty)-1,(cx-stx)+1,(cy-sty)+1,colour);
			draw_line(sfc,(cx-stx)-1,(cy-sty)+1,(cx-stx)+1,(cy-sty)-1,colour);
		} else {
            int t=0;
			int tdx,tdy,hx,hy;

            if (tile[j]>=0) t=tile[j];
			tdx=object->tiles[t]->orig->w*scaling_factor;
			tdy=object->tiles[t]->orig->h*scaling_factor;
			hx=object->link_x[0][0]*scaling_factor;
			hy=object->link_y[0][0]*scaling_factor;
			
			draw_rectangle(sfc,cx-(hx+stx),cy-(hy+sty),tdx,tdy,colour);

			draw_line(sfc,(cx-stx)-1,(cy-sty)-1,(cx-stx)+1,(cy-sty)+1,colour);
			draw_line(sfc,(cx-stx)-1,(cy-sty)+1,(cx-stx)+1,(cy-sty)-1,colour);
		} /* if */ 
	} /* for */ 

	set_dirty();

} /* OPartsWnd::draw_contens */ 


bool OPartsWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		{
			int i;

			/* ACCEPT: */ 
			delete []object->link_part;
			delete []object->link_a;
			delete []object->link_b;
			object->link_part=new int[nparts];
			object->link_a=new int[nparts];
			object->link_b=new int[nparts];
			for(i=0;i<nparts;i++) {
				object->default_tile[i]=tile[i];
				object->link_part[i]=link_part[i];
				object->link_a[i]=link_a[i];
				object->link_b[i]=link_b[i];
				object->part_precedence[i]=part_precedence[i];
			} /* for */ 
			object->nparts=nparts;
			return false;
		}
		break;

	case 2:
		/* CANCEL: */ 
		return false;
		break;

	case 3:
		/* VIEW: */ 
		if (show_tiles) show_tiles=false;
				   else show_tiles=true;

		set_dirty();
		return true;
		break;

	case 4:
		/* TILE: */ 
		{
			BTileSelectionWnd *dlg;
			int y,x,res;

			x=mouse_x-70;
			y=mouse_y;
			if (y>(SCREEN_Y-200)) y=SCREEN_Y-200;
			if (x>(SCREEN_X-140)) x=SCREEN_Y-140;
			if (x<0) x=0;

			dlg=new BTileSelectionWnd(x,y,140,200,object,&res,font);
			dlg->Execute();
			delete dlg;

			if (res!=-2) tile[act_part]=res;

			set_dirty();
			return true;
		}
		break;

	case 5:
		/* ADD: */ 
		if (nparts<MAX_PARTS) {
			tile[nparts]=0;
			link_part[nparts]=0;
			link_a[nparts]=0;
			link_b[nparts]=0;
			nparts++;
		} /* if */ 

		set_dirty();
		return true;
		break;

	case 6:
		/* SUB: */ 
		if (nparts>1) nparts--;

		set_dirty();
		return true;
		break;

	case 7:
		/* PREV: */ 
		act_part--;
		if (act_part<0) act_part=0;

		set_dirty();
		return true;
		break;

	case 8:
		/* NEXT: */ 
		act_part++;
		if (act_part>=nparts) act_part=nparts-1;

		set_dirty();
		return true;
		break;

	case 9:
		if (link_part[act_part]>0) link_part[act_part]--;
		set_dirty();
		return true;
		break;

	case 10:
		if (link_part[act_part]<nparts-1) link_part[act_part]++;
		set_dirty();
		return true;
		break;

	case 11:
		if (link_a[act_part]>0) link_a[act_part]--;
		set_dirty();
		return true;
		break;

	case 12:
		if (link_a[act_part]<MAX_LINKS-1) link_a[act_part]++;
		set_dirty();
		return true;
		break;

	case 13:
		if (link_b[act_part]>0) link_b[act_part]--;
		set_dirty();
		return true;
		break;

	case 14:
		if (link_b[act_part]<MAX_LINKS-1) link_b[act_part]++;
		set_dirty();
		return true;
		break;


	} /* switch */ 

	return false;
} /* OPartsWnd::button_pressed */ 

