#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "samples.h"
#include "interfaces.h"

#include "BWindow.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BFileDlg.h"

#include "actionscript.h"

#include "OSampleWnd.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern bool SOUND_ENABLED;
extern int surface_flags;

OSampleWnd::OSampleWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font)
{
	BButton *b;

	editor=e;

	b=new BButton(4,0,100,12,"NEW SAMPLE",1);
	buttons.Add(b);

} /* OSampleWnd::OSampleWnd */ 


OSampleWnd::~OSampleWnd()
{
} /* OSampleWnd::~OSampleWnd */ 


bool OSampleWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (((mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3))) ||
		 (mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		int y;
		SAMPLE *s;
		List<SAMPLE> l,todelete;
		char tmp[256];
		bool clicked;
	
		y=25+TTF_FontHeight(font);
		l.Instance(editor->game->samples);
		while(l.Iterate(s)) {
			clicked=false;

			sprintf(tmp,"SAMPLE: %s",s->name);
			if (local_mouse_y>=y && local_mouse_y<y+TTF_FontHeight(font) &&
				local_mouse_x>=0 && local_mouse_x<get_text_width(tmp,font)) {
				clicked=true;
			} /* if */ 
			y+=TTF_FontHeight(font);

			sprintf(tmp,"FILE: %s",s->fname);
			if (local_mouse_y>=y && local_mouse_y<y+TTF_FontHeight(font) &&
				local_mouse_x>=0 && local_mouse_x<get_text_width(tmp,font)) {
				clicked=true;
			} /* if */ 

			if (clicked) {
				BVChoiceDlg *dlg;
				int res=0;

				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose action:","Listen\nRename\nDelete\n",&res,font);
				dlg->Execute();
				delete dlg;

				if (res==1 && SOUND_ENABLED) {
					Mix_PlayChannel(-1,s->sample,0);
				} /* if */ 

				if (res==2) {
					BStringEdDlg *dlg2;
					int x,y;

					strcpy(tmp,s->name);
					new_window_coordinates(320,60,&x,&y);
					dlg2=new BStringEdDlg(x,y,320,60,"Enter sample name:",tmp,40," ",font);
					dlg2->Execute();
					delete dlg2;

					s->change_name(tmp);
					set_dirty();
				} /* if */ 

				if (res==3) {
					/* Test whether the SAMPLE is used in any script: */ 
					List<OBJECT> l;
					OBJECT *o;
					int i;
					OBJECT *found=0;

					l.Instance(editor->game->objects);
					l.Rewind();
					while(found==0 && l.Iterate(o)) {
						for(i=0;found==0 && i<o->n_states;i++) 
							if (o->g_state_action[i]->uses_sample(s)) found=o;
						for(i=0;found==0 && i<o->n_s_states;i++) 
							if (o->g_s_state_action[i]->uses_sample(s)) found=o;
						if (o->constitution==CONS_CHARACTER) {
							for(i=0;found==0 && i<CPOS_NSTATES;i++) 
								if (o->state_sample_command[i]!=0 &&
									o->state_sample[i]==s) found=o;
						} /* if */ 
					} /* while */ 

					if (found!=0) {
						char tmp[128];
						int res;
						BChoiceDlg *dlg2;

						sprintf(tmp,"Sample is still in use in object: %s",found->name);
						dlg2=new BChoiceDlg(40,40,400,50,tmp,"Accept",&res,editor->font);
						dlg2->Execute();
						delete dlg2;
					} else {
						todelete.Add(s);
					} /* if */ 

					set_dirty();
				} /* if */ 

			} /* if */ 

			y+=TTF_FontHeight(font)*2;
		} /* while */ 		

		while(!todelete.EmptyP()) {
			s=todelete.ExtractIni();
			editor->game->samples.DeleteElement(s);
		} /* while */ 
	} /* if */ 

	return true;
} /* OSampleWnd::cycle */ 


void OSampleWnd::contens_size(int *dx,int *dy)
{
	int max_x=0;
	int max_y=0;
	SAMPLE *s;
	List<SAMPLE> l;
	char tmp[256];
	
	BWindow::contens_size(dx,dy);
	
	max_x=get_text_width("AVAILABLE SAMPLES:",font);

	max_y=25+TTF_FontHeight(font);
	l.Instance(editor->game->samples);
	while(l.Iterate(s)) {
		sprintf(tmp,"SAMPLE: %s",s->name);
		if (get_text_width(tmp,font)>max_x) max_x=get_text_width(tmp,font);
		sprintf(tmp,"FILE: %s",s->fname);
		if (get_text_width(tmp,font)>max_x) max_x=get_text_width(tmp,font);
		max_y+=TTF_FontHeight(font)*3;
	} /* while */ 

	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* OSampleWnd::contens_size */ 


void OSampleWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	int y;
	SAMPLE *s;
	List<SAMPLE> l;
	char tmp[256];

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	print_left("AVAILABLE SAMPLES:",font,255,255,255,sfc,0-stx,17-sty);


	y=25+TTF_FontHeight(font);
	l.Instance(editor->game->samples);
	while(l.Iterate(s)) {
		sprintf(tmp,"SAMPLE: %s",s->name);
		print_left(tmp,font,255,255,255,sfc,0-stx,y-sty);
		y+=TTF_FontHeight(font);
		sprintf(tmp,"FILE: %s",s->fname);
		print_left(tmp,font,255,255,255,sfc,0-stx,y-sty);
		y+=TTF_FontHeight(font)*2;
	} /* while */ 
	

} /* OSampleWnd::draw_contens */ 


bool OSampleWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		/* NEW SAMPLE: */ 
		{
			char file[256]="*.*";
			BFileDlg *dlg=new BFileDlg(30,30,200,200,"Choose a sound file","Select",file,editor->font);

			dlg->Execute();
			delete dlg;

			if (file[0]!=0) {
				/* Load SAMPLE file: */ 
				SAMPLE *s;
				List<SAMPLE> l;
				int i=0;
				char tmp[80];
				bool added=false,found;

				do{
					if (i==0) {
						sprintf(tmp,"new_sample");
					} else {
						sprintf(tmp,"new_sample_%i",i);
					} /* if */ 

					l.Instance(editor->game->samples);
					l.Rewind();
					found=false;
					while(!found && l.Iterate(s)) {
						if (s->cmp(tmp)) found=true;
					} /* while */ 

					if (!found) {
						added=true;
						s=new SAMPLE(tmp,file);
						if (s->sample!=0)  {
							editor->game->samples.Add(s);
						} else {
							int res;
							BChoiceDlg *dlg2=new BChoiceDlg(40,40,200,50,"Not a valid SOUND file","Accept",&res,editor->font);
							dlg2->Execute();
							delete dlg2;
							delete s;
						} /* if */ 
					} /* if */ 

					i++;
				}while(!added);

			} else {
				int res;
				BChoiceDlg *dlg2=new BChoiceDlg(40,40,200,50,"Cannot open file","Accept",&res,editor->font);
				dlg2->Execute();
				delete dlg2;
			} /* if */ 
			set_dirty();
		}
		break;


	default:
		break;
	} /* switch */ 

	return true;
} /* OSampleWnd::button_pressed */ 
