#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_net.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"
#include "MOG2_auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activeMAP.h"
#include "activemog2.h"
#include "mog2multiplayer.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BListWnd.h"
#include "GamePreviewDlg.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BFileDlg.h"
#include "POVariableWnd.h"

#include "GameWnd.h"
#include "GameDebugWnd.h"
#include "PlayerConfigWnd.h"
#include "interfaces.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif


BPlayerConfigDlg::BPlayerConfigDlg(int x,int y,int dx,int dy,NWMOG2PlayerInfo *cfg,List<OBJECT> *ch,List<INTERFACE> *il,TTF_Font *font) : BActiveWindow(x,y,dx,dy,font)
{
	BButton *b;

	b=new BButton(20,dy-20,80,12,"ACCEPT",1);
	buttons.Add(b);
	b=new BButton(120,dy-20,80,12,"CANCEL",2);
	buttons.Add(b);

	b=new BButton(320,10,80,12,"NONE",11);
	buttons.Add(b);


	original_pc=cfg;
	pc=*cfg;
	characters=ch;
	interfaces=il;

	{
		List<OBJECT> l;
		OBJECT *o;
		char *name;

		l.Instance(*characters);
		l.Rewind();
		while(l.Iterate(o)) {
			name=new char[strlen(o->name)+1];
			strcpy(name,o->name);
			character_names.Add(name);
		} /* while */ 
	}
	{
		List<INTERFACE> l;
		INTERFACE *i;
		char *name;

		l.Instance(*interfaces);
		l.Rewind();
		while(l.Iterate(i)) {
			name=new char[strlen(i->name)+1];
			strcpy(name,i->name);
			interface_names.Add(name);
		} /* while */ 
	}
	plw=new BListWnd(10,30,120,dy-60,&character_names,font);
	plw->set_selected_name(pc.avatar->name);

	ilw=new BListWnd(320,30,120,dy-60,&interface_names,font);
	if (pc.start_interface!=0) ilw->set_selected_name(pc.start_interface->name);

	b=new BButton(150,10,60,12,"UP",3);
	buttons.Add(b);
	b=new BButton(150,30,60,12,"DOWN",4);
	buttons.Add(b);
	b=new BButton(150,50,60,12,"LEFT",5);
	buttons.Add(b);
	b=new BButton(150,70,60,12,"RIGHT",6);
	buttons.Add(b);
	b=new BButton(150,90,60,12,"FIRE1",7);
	buttons.Add(b);
	b=new BButton(150,110,60,12,"FIRE2",8);
	buttons.Add(b);
	b=new BButton(150,130,60,12,"FIRE3",9);
	buttons.Add(b);
	b=new BButton(150,150,60,12,"FIRE4",10);
	buttons.Add(b);

	current_key=-1;
} /* BGamePreviewDlg::BGamePreviewDlg */ 


BPlayerConfigDlg::~BPlayerConfigDlg()
{
	delete plw;
	delete ilw;
} /* BPlayerConfigDlg::~BPlayerConfigDlg */ 


bool BPlayerConfigDlg::cycle(int stx,int sty)
{
	bool retval;

	retval=BWindow::cycle(stx,sty);

	plw->cycle(stx+10,sty+30);
	if (plw->is_dirty()) set_dirty();

	ilw->cycle(stx+320,sty+30);
	if (ilw->is_dirty()) set_dirty();

	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {

		if (local_mouse_x>=10 && local_mouse_x<10+get_text_width(pc.name,font) &&
			local_mouse_y>=10 && local_mouse_y<10+TTF_FontHeight(font)) {
			BStringEdDlg *dlg;
			int x,y;

			new_window_coordinates(320,60,&x,&y);

			dlg=new BStringEdDlg(x,y,320,60,"Enter player name:",pc.name,40,"",font);
			dlg->Execute();
			delete dlg;

			set_dirty();
		} /* if */ 

	} /* if */ 

	return retval;
} /* BPlayerConfigDlg::cycle */ 



void BPlayerConfigDlg::contens_size(int *dx,int *dy)
{
	BActiveWindow::contens_size(dx,dy);

	if (old_dx>*dx) *dx=old_dx;
	if (old_dy>*dy) *dy=old_dy;
} /* BPlayerConfigDlg::contens_size */ 


void BPlayerConfigDlg::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_Rect r;

	SDL_FillRect(sfc,0,0x0000007f);

	r.x=1-stx;
	r.y=1-sty;
	r.w=dx-2;
	r.h=dy-2;
	SDL_FillRect(sfc,&r,0x000000ff);

	BActiveWindow::draw_contens(sfc,stx,sty,dx,dy);
	
	print_left(pc.name,font,255,255,255,sfc,10-stx,10-sty);

	plw->draw(sfc,10-stx,30-sty,120,dy-60);
	ilw->draw(sfc,320-stx,30-sty,120,dy-60);

	print_left(input_key_name(pc.up_key),font,255,255,255,sfc,220-stx,12-sty);
	print_left(input_key_name(pc.down_key),font,255,255,255,sfc,220-stx,32-sty);
	print_left(input_key_name(pc.left_key),font,255,255,255,sfc,220-stx,52-sty);
	print_left(input_key_name(pc.right_key),font,255,255,255,sfc,220-stx,72-sty);
	print_left(input_key_name(pc.fire1_key),font,255,255,255,sfc,220-stx,92-sty);
	print_left(input_key_name(pc.fire2_key),font,255,255,255,sfc,220-stx,112-sty);
	print_left(input_key_name(pc.fire3_key),font,255,255,255,sfc,220-stx,132-sty);
	print_left(input_key_name(pc.fire4_key),font,255,255,255,sfc,220-stx,152-sty);

} /* BPlayerConfigDlg::draw_contens */ 


bool BPlayerConfigDlg::event(SDL_Event *event)
{
	if (event->type==SDL_KEYDOWN) {
		/* Keyboard event */ 
		switch(current_key) {
		case 0: pc.up_key=event->key.keysym.sym;
				break;
		case 1: pc.down_key=event->key.keysym.sym;
				break;
		case 2: pc.left_key=event->key.keysym.sym;
				break;
		case 3: pc.right_key=event->key.keysym.sym;
				break;
		case 4: pc.fire1_key=event->key.keysym.sym;
				break;
		case 5: pc.fire2_key=event->key.keysym.sym;
				break;
		case 6: pc.fire3_key=event->key.keysym.sym;
				break;
		case 7: pc.fire4_key=event->key.keysym.sym;
				break;
		} /* switch */ 
		set_dirty();
		current_key=-1;
    } /* if */ 

	return true;
} /* BPlayerConfigDlg::Event */ 


bool BPlayerConfigDlg::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		{
			pc.avatar=characters->operator [](plw->get_selected());
			if (ilw->get_selected()!=-1) pc.start_interface=interfaces->operator [](ilw->get_selected());
									else pc.start_interface=0;
			*original_pc=pc;
			return false;
		}
		break;

	case 2:
		return false;
		break;

	case 3:
		/* UP: */ 
		current_key=0;
		break;
	case 4:
		/* DOWN: */ 
		current_key=1;
		break;
	case 5:
		/* LEFT: */ 
		current_key=2;
		break;
	case 6:
		/* RIGHT: */ 
		current_key=3;
		break;
	case 7:
		/* FIRE1: */ 
		current_key=4;
		break;
	case 8:
		/* FIRE2: */ 
		current_key=5;
		break;
	case 9:
		/* FIRE3: */ 
		current_key=6;
		break;
	case 10:
		/* FIRE4: */ 
		current_key=7;
		break;

	case 11:
		/* No interface: */ 
		ilw->set_selected(-1);
		ilw->set_dirty();
		set_dirty();
		break;

	} /* switch */ 

	return true;
} /* BPlayerConfigDlg::button_pressed */ 
