#include "assert.h"

#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "interfaces.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BMultiListWnd.h"

#include "PObjectVarsWnd.h"

#include "RToolsWnd.h"
#include "REditWnd.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int SCREEN_X,SCREEN_Y;


REditWnd::REditWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	editor=e;

	edit_state=0;

} /* REditWnd::REditWnd */ 


REditWnd::~REditWnd()
{
} /* REditWnd::~REditWnd */ 


bool REditWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (editor->room!=0 &&
		local_mouse_x>old_stx &&
		local_mouse_y>old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {

		if (!(mouse_button&SDL_BUTTON(1)) &&
			!(old_mouse_button&SDL_BUTTON(1))) {
			if (init_x!=-1) set_dirty();
			init_x=-1;
			init_y=-1;
			dx=0;
			dy=0;
			end_x=0;
			end_y=0;
			cursor_text[0]=0;
		} /* if */ 

		if (editor->rv_editmode==1) {
			/* ERASE: */  
			List<PLACED_TILE> l1,todelete_t;
			PLACED_TILE *t,*found_t;
			List<PLACED_OBJECT> l2,todelete_o;
			PLACED_OBJECT *o,*found_o;
			bool found=false;

			init_x=(local_mouse_x-2);
			init_y=(local_mouse_y-2);

			end_x=init_x;
			end_y=init_y;

			l2.Instance(editor->room->objects[editor->rv_editing]);
			l1.Instance(editor->room->tiles[editor->rv_editing]);

			found_o=0;
			l2.Rewind();
			while(l2.Iterate(o)) {
				if (o->object->ntiles>0 &&
					o->object->control!=CTRL_TEXT &&
					o->object->control!=CTRL_GAMEWINDOW &&
					init_x>=o->x && init_x<=o->x+o->object->tiles[0]->r.w &&
					init_y>=o->y && init_y<=o->y+o->object->tiles[0]->r.h) {
					found=true;
					found_o=o;
				} /* if */ 

				if (o->object->ntiles==0 && o->object->control==CTRL_TEXT) {
					int x1,y1,x2,y2;
					o->get_text_bbox(&x1,&y1,&x2,&y2);

					if (init_x>=x1 && init_x<=x2 &&
						init_y>=y1 && init_y<=y2) {
						found=true;
						found_o=o;
					} /* if */ 
				} /* if */ 

				if (o->object->ntiles==0 && o->object->control==CTRL_GAMEWINDOW) {
					int x1,y1,x2,y2;
					o->get_gw_bbox(&x1,&y1,&x2,&y2);

					if (init_x>=o->x && init_x<=o->x+(x2-x1) &&
						init_y>=o->y && init_y<=o->y+(y2-y1)) {
						found=true;
						found_o=o;
					} /* if */ 
				} /* if */ 
			} /* while */ 

			if (found_o!=0) {
				init_x=found_o->x;
				init_y=found_o->y;
				end_x=init_x;
				end_y=init_y;
				if (found_o->object->control==CTRL_TEXT) {
					int x1,y1,x2,y2;
					found_o->get_text_bbox(&x1,&y1,&x2,&y2);
					init_x=x1;
					init_y=y1;
					end_x=init_x;
					end_y=init_y;
					dx=x2-x1;
					dy=y2-y1;
				} else {
					if (o->object->control==CTRL_GAMEWINDOW) {
						int x1,y1,x2,y2;
						o->get_gw_bbox(&x1,&y1,&x2,&y2);
						dx=x2-x1;
						dy=y2-y1;
					} else {
						dx=found_o->object->tiles[0]->r.w;
						dy=found_o->object->tiles[0]->r.h;
					} /* if */ 
				} /* if */ 
				set_dirty();

				if (mouse_button&SDL_BUTTON(1) &&
					!(old_mouse_button&SDL_BUTTON(1))) {
					todelete_o.Add(found_o);
				} /* if */ 
			} else {
				found_t=0;
				l1.Rewind();
				while(l1.Iterate(t)) {
					assert(t->tile!=0);
					if (init_x>=t->x && init_x<=t->x+t->tile->r.w &&
						init_y>=t->y && init_y<=t->y+t->tile->r.h) {
						found=true;
						found_t=t;
					} /* if */ 
				} /* while */ 
				if (found_t!=0) {
					init_x=found_t->x;
					init_y=found_t->y;
					end_x=init_x;
					end_y=init_y;
					dx=found_t->tile->r.w;
					dy=found_t->tile->r.h;
					set_dirty();

					if (mouse_button&SDL_BUTTON(1) &&
						!(old_mouse_button&SDL_BUTTON(1))) {
						todelete_t.Add(found_t);
					} /* if */ 
				} /* if */ 
			} /* if */ 

			if (!found) {
				init_x=-1;
				init_y=-1;
			} else {
				sprintf(cursor_text,"%i,%i",int(init_x),int(init_y));
			} /* if */ 

			while(!todelete_t.EmptyP()) {
				t=todelete_t.ExtractIni();
				editor->room->tiles[editor->rv_editing].DeleteElement(t);
				delete t;
			} /* while */ 
			while(!todelete_o.EmptyP()) {
				o=todelete_o.ExtractIni();
				editor->room->objects[editor->rv_editing].DeleteElement(o);
				delete o;
			} /* while */ 
		} /* if */ 


		if (editor->rv_editmode==2) {
			/* REGION: */ 
			int values[6]={1,2,4,8,16,32};
			int grid;
			long int mx,my;

			/* Determine the width of the tile/object currently placing: */ 
			if (editor->rv_displaying_objects) {
				/* OBJECTS: */ 
				if (editor->rv_selected_obj!=0 &&
					editor->rv_selected_obj->ntiles>0) {
					dx=editor->rv_selected_obj->tiles[0]->r.w;
					dy=editor->rv_selected_obj->tiles[0]->r.h;
				} else {
					if (editor->rv_selected_obj!=0 &&
						editor->rv_selected_obj->control==CTRL_TEXT) {
						int x1,y1,x2,y2;
						editor->rv_selected_obj->get_text_bbox(0,0,&x1,&y1,&x2,&y2,editor->game);
						dx=x2-x1;
						dy=y2-y1;
					} else {
						dx=0;
						dy=0;
					} /* if */ 
				} /* if */ 
			} else {
				/* BITMAPS: */ 
				if (editor->rv_selected_tile!=0) {
					dx=editor->rv_selected_tile->r.w;
					dy=editor->rv_selected_tile->r.h;
				} else {
					dx=0;
					dy=0;
				} /* if */ 
			} /* if */ 	
			
			if (dx!=0 && dy!=0) {				

				grid=values[editor->rv_grid];
				mx=(local_mouse_x-(2-grid/2)-dx/2)/grid;
				my=(local_mouse_y-(2-grid/2)-dy/2)/grid;
				mx*=grid;
				my*=grid;

				if (mouse_button&SDL_BUTTON(1) &&
					!(old_mouse_button&SDL_BUTTON(1)) &&
					edit_state==0) {
					init_x=mx;
					init_y=my;
					edit_state=1;
				} /* if */ 

				if (mouse_button&SDL_BUTTON(1) && edit_state==1) {
					mx=(mx-init_x)/dx;
					my=(my-init_y)/dy;
					mx=init_x+mx*dx;
					my=init_y+my*dy;
					end_x=mx;
					end_y=my;
				} else {
					if (!old_mouse_button&SDL_BUTTON(1)) {
						end_x=init_x=mx;
						end_y=init_y=my;
						sprintf(cursor_text,"%i,%i",int(init_x),int(init_y));
					} /* if */ 
				} /* if */ 

				if (local_mouse_x>=2 &&
					local_mouse_y>=2 &&
					init_x<(editor->room->dx*TILESIZE) &&
					init_y<(editor->room->dy*TILESIZE) &&
					end_x<(editor->room->dx*TILESIZE) &&
					end_y<(editor->room->dy*TILESIZE) &&
					edit_state==1) {
					set_dirty();
					sprintf(cursor_text,"%i,%i [%ix%i]",int(init_x),int(init_y),int((end_x-init_x)/dx+1),int((end_y-init_y)/dy+1));

					if (!(mouse_button&SDL_BUTTON(1)) &&
						old_mouse_button&SDL_BUTTON(1)) {
					
						int x1,y1,x2,y2;
						if (editor->rv_displaying_objects &&
							editor->rv_selected_obj!=0 && 
							editor->rv_selected_obj->control==CTRL_TEXT) {
							editor->rv_selected_obj->get_text_bbox(0,0,&x1,&y1,&x2,&y2,editor->game);
							init_x-=x1;
							init_y-=y1;
							end_x-=x1;
							end_y-=y1;
						} /* if */ 

						x1=init_x;
						y1=init_y;
						x2=end_x;
						y2=end_y;
						if (end_x<init_x) {
							int t=x1;
							x1=x2;
							x2=t;
						} /* if */ 
						if (end_y<init_y) {
							int t=y1;
							y1=y2;
							y2=t;
						} /* if */ 

						init_x=mx;
						init_y=my;
						/* RIGHT POSITION FOR PLACING: */ 
						if (editor->rv_displaying_objects) {
							int i,j;
							for(i=x1;i<=x2;i+=dx) {
								for(j=y1;j<=y2;j+=dy) {
									PLACED_OBJECT *o;

									o=new PLACED_OBJECT(i,j,editor->rv_selected_obj,editor->game,editor->room,0);
									editor->room->objects[editor->rv_editing].Add(o);
								} /* for */ 
							} /* for */ 
						} else {
							int i,j;
							for(i=x1;i<=x2;i+=dx) {
								for(j=y1;j<=y2;j+=dy) {
									PLACED_TILE *t;

									t=new PLACED_TILE(i,j,editor->rv_selected_tile);
									assert(t->tile!=0);
									editor->room->tiles[editor->rv_editing].Add(t);
								} /* for */ 
							} /* for */ 
						} /* if */ 
					} /* if */ 
				} /* if */ 

			} /* if */ 

		} /* if */ 

		if (editor->rv_editmode==3) {
			/* MODIFY */ 
			List<PLACED_OBJECT> l2;
			PLACED_OBJECT *o,*found_o;
			bool found=false;

			init_x=(local_mouse_x-2);
			init_y=(local_mouse_y-2);

			end_x=init_x;
			end_y=init_y;

			l2.Instance(editor->room->objects[editor->rv_editing]);

			found_o=0;
			l2.Rewind();
			while(l2.Iterate(o)) {
				if (o->object->ntiles>0 &&
					o->object->control!=CTRL_TEXT &&
					o->object->control!=CTRL_GAMEWINDOW &&
					init_x>=o->x && init_x<=o->x+o->object->tiles[0]->r.w &&
					init_y>=o->y && init_y<=o->y+o->object->tiles[0]->r.h) {
					found=true;
					found_o=o;
				} /* if */ 

				if (o->object->ntiles==0 && o->object->control==CTRL_TEXT) {
					int x1,y1,x2,y2;
					o->get_text_bbox(&x1,&y1,&x2,&y2);

					if (init_x>=x1 && init_x<=x2 &&
						init_y>=y1 && init_y<=y2) {
						found=true;
						found_o=o;
					} /* if */ 
				} /* if */ 

				if (o->object->ntiles==0 && o->object->control==CTRL_GAMEWINDOW) {
					int x1,y1,x2,y2;
					o->get_gw_bbox(&x1,&y1,&x2,&y2);

					if (init_x>=o->x && init_x<=o->x+(x2-x1) &&
						init_y>=o->y && init_y<=o->y+(y2-y1)) {
						found=true;
						found_o=o;
					} /* if */ 
				} /* if */ 

			} /* while */ 

			if (found_o!=0) {
				init_x=found_o->x;
				init_y=found_o->y;
				end_x=init_x;
				end_y=init_y;
				if (found_o->object->control==CTRL_TEXT) {
					int x1,y1,x2,y2;
					found_o->get_text_bbox(&x1,&y1,&x2,&y2);
					init_x=x1;
					init_y=y1;
					end_x=init_x;
					end_y=init_y;
					dx=x2-x1;
					dy=y2-y1;
				} else {
					if (found_o->object->control==CTRL_GAMEWINDOW) {
						int x1,y1,x2,y2;
						found_o->get_gw_bbox(&x1,&y1,&x2,&y2);
						dx=x2-x1;
						dy=y2-y1;
					} else {
						dx=found_o->object->tiles[0]->r.w;
						dy=found_o->object->tiles[0]->r.h;
					} /* if */ 
				} /* if */ 
				set_dirty();

				if (mouse_button&SDL_BUTTON(1) &&
					!(old_mouse_button&SDL_BUTTON(1))) {
					/* SHOW VARIABLE WINDOW: */ 
					BPObjectVarsWnd *wnd;
					int x,y;

					x=mouse_x-150;
					y=mouse_y;
					if (y>(SCREEN_Y-300)) y=SCREEN_Y-300;
					if (x>(SCREEN_X-200)) x=SCREEN_X-200;
					if (x<0) x=0;

					wnd=new BPObjectVarsWnd(x,y,200,300,found_o,font);
					wnd->Execute();
					delete wnd;
				} /* if */ 
			} /* if */ 

			if (!found) {
				init_x=-1;
				init_y=-1;
			} else {
				sprintf(cursor_text,"%i,%i",int(init_x),int(init_y));
			} /* if */ 

		} /* if */ 

		if (editor->rv_editmode==4) {
			/* ABSOLUTE COORDINATES: */ 
			int values[6]={1,2,4,8,16,32};
			int grid;
			int mx,my;

			grid=values[editor->rv_grid];
			mx=(local_mouse_x-(2-grid/2)-dx/2)/grid;
			my=(local_mouse_y-(2-grid/2)-dy/2)/grid;
			mx*=grid;
			my*=grid;			

			end_x=init_x=mx;
			end_y=init_y=my;
			dx=16;
			dy=16;
			sprintf(cursor_text,"%i,%i",int(init_x+editor->room->x*TILESIZE),int(init_y+editor->room->y*TILESIZE));
		} /* if */ 


		if (editor->rv_editmode==5) {
			/* DRAG: */ 
			List<PLACED_OBJECT> l2;
			PLACED_OBJECT *o;
			bool found=false;
			int values[6]={1,2,4,8,16,32};
			int grid;
			int mx,my;

			if (!old_mouse_button&SDL_BUTTON(1)) {
				init_x=(local_mouse_x-2);
				init_y=(local_mouse_y-2);

				end_x=init_x;
				end_y=init_y;

				/* Mouse button not pressed: */ 
				l2.Instance(editor->room->objects[editor->rv_editing]);

				editor->rv_dragging_object=0;
				l2.Rewind();
				while(l2.Iterate(o)) {
					if (o->object->ntiles>0 &&
						o->object->control!=CTRL_TEXT &&
						o->object->control!=CTRL_GAMEWINDOW &&
						init_x>=o->x && init_x<=o->x+o->object->tiles[0]->r.w &&
						init_y>=o->y && init_y<=o->y+o->object->tiles[0]->r.h) {
						found=true;
						editor->rv_dragging_object=o;
					} /* if */ 

					if (o->object->ntiles==0 && o->object->control==CTRL_TEXT) {
						int x1,y1,x2,y2;
						o->get_text_bbox(&x1,&y1,&x2,&y2);

						if (init_x>=x1 && init_x<=x2 &&
							init_y>=y1 && init_y<=y2) {
							found=true;
							editor->rv_dragging_object=o;
						} /* if */ 
					} /* if */ 

					if (o->object->ntiles==0 && o->object->control==CTRL_GAMEWINDOW) {
						int x1,y1,x2,y2;
						o->get_gw_bbox(&x1,&y1,&x2,&y2);

						if (init_x>=o->x && init_x<=o->x+(x2-x1) &&
							init_y>=o->y && init_y<=o->y+(y2-y1)) {
							found=true;
							editor->rv_dragging_object=o;
						} /* if */ 
					} /* if */ 

				} /* while */ 

				if (editor->rv_dragging_object!=0) {
					init_x=editor->rv_dragging_object->x;
					init_y=editor->rv_dragging_object->y;
					end_x=init_x;
					end_y=init_y;
					if (editor->rv_dragging_object->object->control==CTRL_TEXT) {
						int x1,y1,x2,y2;
						editor->rv_dragging_object->get_text_bbox(&x1,&y1,&x2,&y2);
						init_x=x1;
						init_y=y1;
						end_x=init_x;
						end_y=init_y;
						dx=x2-x1;
						dy=y2-y1;
					} else {
						if (editor->rv_dragging_object->object->control==CTRL_GAMEWINDOW) {
							int x1,y1,x2,y2;
							editor->rv_dragging_object->get_gw_bbox(&x1,&y1,&x2,&y2);
							dx=x2-x1;
							dy=y2-y1;
						} else {
							dx=editor->rv_dragging_object->object->tiles[0]->r.w;
							dy=editor->rv_dragging_object->object->tiles[0]->r.h;
						} /* if */ 
					} /* if */ 
					set_dirty();
				} /* if */ 

				if (!found) {
					init_x=-1;
					init_y=-1;
				} else {
					sprintf(cursor_text,"%i,%i",int(init_x),int(init_y));
				} /* if */ 	
			} else {
				grid=values[editor->rv_grid];
				mx=(local_mouse_x-(2-grid/2)-dx/2)/grid;
				my=(local_mouse_y-(2-grid/2)-dy/2)/grid;
				mx*=grid;
				my*=grid;			

				end_x=init_x=mx;
				end_y=init_y=my;

				if (editor->rv_dragging_object!=0) {
					if (editor->rv_dragging_object->object->control==CTRL_TEXT) {
						int x1,y1,x2,y2;
						editor->rv_dragging_object->get_text_bbox(&x1,&y1,&x2,&y2);
						dx=x2-x1;
						dy=y2-y1;
					} else {
						if (editor->rv_dragging_object->object->control==CTRL_GAMEWINDOW) {
							int x1,y1,x2,y2;
							editor->rv_dragging_object->get_gw_bbox(&x1,&y1,&x2,&y2);
							dx=x2-x1;
							dy=y2-y1;
						} else {
							dx=editor->rv_dragging_object->object->tiles[0]->r.w;
							dy=editor->rv_dragging_object->object->tiles[0]->r.h;
						} /* if */ 
					} /* if */ 					
				} /* if */ 

				sprintf(cursor_text,"TO %i,%i",int(init_x),int(init_y));
				set_dirty();
				if (!mouse_button&SDL_BUTTON(1)) {
					/* BUTTON RELEASED, FINISH DRAG: */ 
					if (editor->rv_dragging_object->object->control==CTRL_TEXT) {
						int x1,y1,x2,y2;
						editor->rv_dragging_object->get_text_bbox(&x1,&y1,&x2,&y2);
						editor->rv_dragging_object->x=init_x+(editor->rv_dragging_object->x-x1);
						editor->rv_dragging_object->y=init_y+(editor->rv_dragging_object->y-y1);
					} else {			
						editor->rv_dragging_object->x=init_x;
						editor->rv_dragging_object->y=init_y;
					} /* if */ 
					editor->iv_dragging_object=0;
				} /* if */ 
			} /* if */ 
		} /* if */ 

	} /* if */ 

	if (!mouse_button&SDL_BUTTON(1)) edit_state=0;

	return true;
} /* REditWnd::cycle */ 


void REditWnd::contens_size(int *dx,int *dy)
{
	int max_x=0,max_y=0;

	BWindow::contens_size(dx,dy);

	if (editor->room!=0) {
		max_x=editor->room->dx*TILESIZE+4+1;
		max_y=editor->room->dy*TILESIZE+4+1;
	} /* if */ 

	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* REditWnd::contens_size */ 


void REditWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int wdx,int wdy)
{
	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,wdx,wdy);

	if (editor->room!=0) {
		SDL_Rect r;

		if (stx>2) {
			r.x=0;
			if (2+long(editor->room->dx)*TILESIZE-stx>=sfc->w) r.w=sfc->w;
												          else r.w=2+long(editor->room->dx)*TILESIZE-stx;
		} else {
			r.x=2-stx;
			if (long(editor->room->dx)*TILESIZE>=sfc->w) r.w=sfc->w;
													else r.w=editor->room->dx*TILESIZE;
		} /* if */ 
		if (sty>2) {
			r.y=0;
			if (2+long(editor->room->dy)*TILESIZE-sty>=sfc->h) r.h=sfc->h;
														  else r.h=2+long(editor->room->dy)*TILESIZE-sty;
		} else {
			r.y=2-sty;
			if (long(editor->room->dy)*TILESIZE>=sfc->h) r.h=sfc->h;
												    else r.h=editor->room->dy*TILESIZE;
		} /* if */ 

		SDL_SetClipRect(sfc,&r);
		{
			int i;
			bool drawplane[MAX_OBJECT_PLANES];
			for(i=0;i<MAX_OBJECT_PLANES;i++) {
				drawplane[i]=editor->RToolW->lw->selected_p(i);
			} /* for */ 
			editor->room->draw(sfc,2-stx,2-sty,dx,dy,0,0,1.0,drawplane);
		}
		SDL_SetClipRect(sfc,0);

		draw_rectangle(sfc,0-stx,0-sty,4+editor->room->dx*TILESIZE,4+editor->room->dy*TILESIZE,0x00ffffff);

		if (init_x!=-1 && init_y!=-1 && dx!=0 && dy!=0) {
			int x1,y1,x2,y2;

			x1=init_x;
			y1=init_y;
			x2=end_x;
			y2=end_y;
			if (end_x<init_x) {
				int t=x1;
				x1=x2;
				x2=t;
			} /* if */ 
			if (end_y<init_y) {
				int t=y1;
				y1=y2;
				y2=t;
			} /* if */ 
			if (cursor_text[0]!=0) {
				print_left(cursor_text,font,255,255,255,sfc,2+x1-stx,2+y2+dy+1-sty);
			} /* if */ 
			draw_rectangle(sfc,2+x1-stx,2+y1-sty,(x2-x1)+dx,(y2-y1)+dy,0x00ffffff);
		} /* if */ 
	} /* if */ 

} /* REditWnd::draw_contens */ 


bool REditWnd::event(SDL_Event *event)
{
	if (active_window!=this) return true;

	if (event->type==SDL_KEYDOWN) {
		/* Keyboard event */ 
		if (event->key.keysym.sym==SDLK_UP) {
			old_sty-=TILESIZE;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_DOWN) {
			old_sty+=TILESIZE;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_LEFT) {
			old_stx-=TILESIZE;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_RIGHT) {
			old_stx+=TILESIZE;
		} /* if */ 

		if (event->key.keysym.sym==SDLK_PAGEUP) {
			old_sty-=TILESIZE*16;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_PAGEDOWN) {
			old_sty+=TILESIZE*16;
		} /* if */ 

		set_dirty();
    } /* if */ 

	return true;
} /* REditWnd::Event */ 