#ifdef _WIN32
#include "windows.h"
#include "direct.h"
#endif

#include "string.h"
#include "stdlib.h"
#include "math.h"

#include "SDL.h"
#include "SDL_net.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "mog2multiplayer.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activemog2.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BListWnd.h"
#include "BMultiListWnd.h"
#include "GamePreviewDlg.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BFileDlg.h"
#include "POVariableWnd.h"

#include "GameWnd.h"
#include "GameDebugWnd.h"
#include "networktest.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

const int MAX_CONNECTIONS=256;
const int CONNECTION_TIMEOUT=1000;
const int LISTENING_TIME=0;

const int MSG_MAXLEN=2048;

extern int buffer_fps,act_fps,fps_timer;
extern int buffer_fps_d,fps_d;

int TEST_REDRAWING_PERIOD=40;

BNetworkTestWnd::BNetworkTestWnd(int x,int y,int dx,int dy,NWMOG2GameInfo *gi,TTF_Font *font) : BActiveWindow(x,y,dx,dy,font)
{
	BButton *b;

	b=new BButton(20,dy-20,80,12,"BACK",1);
	buttons.Add(b);

	net=new NetworkMOG2Protocol(gi,0,0);

	time=SDL_GetTicks();

} /* BNetworkTestWnd::BNetworkTestWnd */ 


BNetworkTestWnd::~BNetworkTestWnd()
{
	delete net;

} /* BNetworkTestWnd::~BNetworkTestWnd */ 


bool BNetworkTestWnd::cycle(int stx,int sty)
{
	bool retval;
	int act_time;

	retval=BWindow::cycle(stx,sty);

	set_dirty();

	if (!net->listening_cycle()) retval=false;

	/* Game cycles: */ 
	act_time=SDL_GetTicks();
	if (act_time-time>=TEST_REDRAWING_PERIOD) {
		act_fps++;
		if ((act_time-fps_timer)>1000) {
			fps_timer=act_time;
			buffer_fps=act_fps;
			buffer_fps_d=fps_d;
			act_fps=0;
			fps_d=0;
		} /* if */ 

		time+=TEST_REDRAWING_PERIOD;
		if ((act_time-time)>8*TEST_REDRAWING_PERIOD) time=act_time;

		if (!net->execute_cycle()) retval=false;

	} /* if */ 
	fps_d++;

	return retval;
} /* BNetworkTestWnd::cycle */ 



void BNetworkTestWnd::contens_size(int *dx,int *dy)
{
	BActiveWindow::contens_size(dx,dy);

	if (old_dx>*dx) *dx=old_dx;
	if (old_dy>*dy) *dy=old_dy;
} /* BNetworkTestWnd::contens_size */ 


void BNetworkTestWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_Rect r;
	int i,j;
	char tmp[256];
	NWMOG2ClientInfo *ci;

	SDL_FillRect(sfc,0,0x0000007f);

	r.x=1-stx;
	r.y=1-sty;
	r.w=dx-2;
	r.h=dy-2;
	SDL_FillRect(sfc,&r,0x000000ff);

	BActiveWindow::draw_contens(sfc,stx,sty,dx,dy);

	if (net->gameinfo->server==net->gameinfo->local) {
		for(j=0,i=0;i<net->gameinfo->clients.Length()+1;i++) if (net->client_active[i]) j++;
		sprintf(tmp,"%i/%i SERVER, COMPUTERS: %i/%i",net->current_cycle,net->last_synchronized_cycle,
				j,net->gameinfo->clients.Length()+1);
		print_left(tmp,font,255,255,255,sfc,10-stx,10-sty);
	} else {
		sprintf(tmp,"%i/%i CLIENT, STATE: %i",net->current_cycle,net->last_synchronized_cycle,net->connection_state);
		print_left(tmp,font,255,255,255,sfc,10-stx,10-sty);
	} /* if */ 

	sprintf(tmp,"%i/%i",buffer_fps,buffer_fps_d);
	print_left(tmp,font,255,255,255,sfc,(dx-50)-stx,(dy-10)-sty);

	for(i=0;i<net->gameinfo->clients.Length()+1;i++) {
		ci=net->gameinfo->find_by_LVK_identifier(i);
		if (ci!=0) {
			sprintf(tmp,"%s (LVK %i): ",ci->computer_name,i);
			for(j=0;j<net->LVK[i]->k_size;j++) {
				if (net->LVK[i]->keyboard[j]) {
					sprintf(tmp+strlen(tmp),"%i ",j);
				} /* if */ 
			} /* for */ 
			print_left(tmp,font,255,255,255,sfc,10-stx,40+i*10-sty);
		} /* if */ 
	} /* for */ 

} /* BNetworkTestWnd::draw_contens */ 



bool BNetworkTestWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		return false;
		break;
	} /* switch */ 

	return true;
} /* BNetworkTestWnd::button_pressed */ 


bool BNetworkTestWnd::event(SDL_Event *event)
{
	if (active_window!=this) return true;

	net->event(event);

	return true;
} /* BNetworkTestWnd::Event */ 