#ifdef _WIN32
#include <windows.h>
#include <windowsx.h>
#else
#include <sys/time.h>
#include <time.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_net.h"
#include "SDL/SDL_mixer.h"
#include "SDL_sound.h"

#include "List.h"

#include "auxiliar.h"
#include "MOG2_auxiliar.h"
#include "reflection_effect.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activeMAP.h"
#include "activemog2.h"
#include "activemog2application.h"
#include "mog2multiplayer.h"

#include "BWindow.h"
#include "BActiveWindow.h"


/*						GLOBAL VARIABLES INITIALIZATION:							*/ 

int SCREEN_X=640;
int SCREEN_Y=480;
int COLOUR_DEPTH=32;
char *application_name="MoG2 engine";
bool SOUND_ENABLED=true;

#ifdef _WIN32
bool fullscreen=false;

//int extra_flags=SDL_HWSURFACE|SDL_DOUBLEBUF;
int extra_flags=0;
int surface_flags=SDL_HWSURFACE;
// int surface_flags=0;

#else
bool fullscreen=false;
int extra_flags=0;
int surface_flags=0;
#endif

/* Redrawing constant: */ 
int REDRAWING_PERIOD=20;
int buffer_fps,act_fps,fps_timer;
int buffer_fps_d,fps_d;
int n_collisions=0;
int acum_collisions[100]={0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0};

/* Surfaces: */ 
SDL_Surface *screen_sfc=0;


TTF_Font *font=0;


/*						AUXILIAR FUNCTION DEFINITION:							*/ 

SDL_Surface* initializeSDL(int moreflags)
{
	SDL_Surface *screen;

	//int flags = SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_HWPALETTE;
	int flags = extra_flags|moreflags;
	if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO)<0) return 0;
	atexit(SDL_Quit);
	SDL_WM_SetCaption(application_name, 0);
	SDL_ShowCursor(SDL_DISABLE);
	SDL_EnableUNICODE(1);

	screen = SDL_SetVideoMode(SCREEN_X, SCREEN_Y, COLOUR_DEPTH, flags);

	if (SOUND_ENABLED) {
		char SoundcardName[256];
		int audio_rate = 44100;
		int audio_channels = 2;
		int audio_bufsize = 2048;
		Uint16 audio_format = AUDIO_S16;
		SDL_version compile_version;

		fprintf (stderr, "Initializing SDL_mixer.\n");
		if (Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_bufsize)) {
			fprintf (stderr, "Unable to open audio: %s\n", Mix_GetError());
			return 0;
		} /* if */ 

		SDL_AudioDriverName (SoundcardName, sizeof (SoundcardName));
		Mix_QuerySpec (&audio_rate, &audio_format, &audio_channels);
		fprintf (stderr, "    opened %s at %d Hz %d bit %s, %d bytes audio buffer\n",
				 SoundcardName, audio_rate, audio_format & 0xFF,
				 audio_channels > 1 ? "stereo" : "mono", audio_bufsize);
		MIX_VERSION (&compile_version);
		fprintf (stderr, "    compiled with SDL_mixer version: %d.%d.%d\n",
				 compile_version.major,
				 compile_version.minor,
				 compile_version.patch);
		fprintf (stderr, "    running with SDL_mixer version: %d.%d.%d\n",
				 Mix_Linked_Version()->major,
				 Mix_Linked_Version()->minor,
				 Mix_Linked_Version()->patch);

		Sound_Init();
	}

	TTF_Init();

	reflection_init();

	return screen;
} /* initializeSDL */ 


void finalizeSDL()
{
	reflection_init();

	if (SOUND_ENABLED) {
		Sound_Quit();
		Mix_CloseAudio();
	} /* if */ 
	TTF_Quit();
	SDL_Quit();
} /* finalizeSDL */ 




#ifdef _WIN32
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR lpCmdLine, int nCmdShow)
{
#else
int main(int argc, char** argv)
{
#endif

	List<char> files;
	char gamefile[256]="";

	{
#ifdef _WIN32
		/* Look for files: */ 
		WIN32_FIND_DATA finfo;
		HANDLE h;
		char *pattern="*.mg2";
               
		files.Delete();	

		h=FindFirstFile(pattern,&finfo);
		if (h!=INVALID_HANDLE_VALUE) {
			if ((finfo.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)==0) {
				char *tmp;

				tmp=new char[strlen(finfo.cFileName)+1];
				strcpy(tmp,finfo.cFileName);
				files.Add(tmp);
			} /* if */ 

			while(FindNextFile(h,&finfo)==TRUE) {
				if ((finfo.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)==0) {
					char *tmp;

					tmp=new char[strlen(finfo.cFileName)+1];
					strcpy(tmp,finfo.cFileName);
					files.Add(tmp);
				} /* if */ 
			} /* while */ 
		} /* if */ 

		FindClose(h);
                
#else
		DIR *dp;
		char *tmp2=".mg2";
		char *pattern=".";
		struct dirent *ep;

		files.Delete();

                              
		dp = opendir (tmp);
		if (dp != NULL) {
		    while (ep = readdir (dp)) {
				if (ep->d_type!=DT_DIR) {
					if (strlen(tmp2)<=strlen(ep->d_name) &&
						strcmp(tmp2,ep->d_name+strlen(ep->d_name)-strlen(tmp2))==0) {
						char *tmp;

						tmp=new char[strlen(ep->d_name)+1];
						strcpy(tmp,ep->d_name);
						files.Add(tmp);
					} /* if */ 
				} /* if */ 
            } /* while */ 
            (void) closedir (dp);
        } /* if */ 
                                  
#endif
	}

	if (!files.EmptyP()) {

		screen_sfc = initializeSDL((fullscreen ? SDL_FULLSCREEN : 0));
		if (screen_sfc==0) return 0;
		buffer_fps=act_fps=buffer_fps_d=fps_d=0;
		fps_timer=SDL_GetTicks();

		if (files.Length()==1) {
			strcpy(gamefile,files[0]);
		} else {
			int time,act_time;
			SDL_Event event;
			bool quit = false;
			TTF_Font *font=0;

			font=TTF_OpenFont("fonts/lucon.ttf",10);

			files.Rewind();

			while (!quit) {
				while( SDL_PollEvent( &event ) ) {
					bool captured=false;

					switch( event.type ) {
						/* Keyboard event */ 
						case SDL_KEYDOWN:

							if (event.key.keysym.sym==SDLK_DOWN) {
								if (!files.LastP()) files.Next();
								captured=true;
							} /* if */ 

							if (event.key.keysym.sym==SDLK_UP) {
								if (!files.BeginP()) files.Prev();
								captured=true;
							} /* if */ 

							if (event.key.keysym.sym==SDLK_RETURN ||
								event.key.keysym.sym==SDLK_SPACE) {
								SDLMod modifiers=SDL_GetModState();

								if ((modifiers&KMOD_ALT)==0) {
									captured=true;
									strcpy(gamefile,files.GetObj());
									quit=true;
								} /* if */ 
							} /* if */ 

							if (event.key.keysym.sym==SDLK_F12) {
								captured=true;
								quit = true;
							} /* if */ 

							if (event.key.keysym.sym==SDLK_RETURN) {
								SDLMod modifiers=SDL_GetModState();

								if ((modifiers&KMOD_ALT)!=0) {
									/* Toogle FULLSCREEN mode: */ 
									captured=true;
									if (fullscreen) fullscreen=false;
											   else fullscreen=true;
//									SDL_QuitSubSystem(SDL_INIT_VIDEO);
//									SDL_InitSubSystem(SDL_INIT_VIDEO);
//									if (SDL_WasInit(SDL_INIT_VIDEO)) {
										screen_sfc = SDL_SetVideoMode(SCREEN_X, SCREEN_Y, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0)|extra_flags);
										SDL_WM_SetCaption(application_name, 0);
										SDL_ShowCursor(SDL_DISABLE);
										SDL_EnableUNICODE(1);
//									} else {
//										quit = true;
//									} /* if */ 
								} /* if */ 
							} /* if */ 
							break;

						/* SDL_QUIT event (window close) */ 
						case SDL_QUIT:
							quit = true;
							break;
					} /* switch */ 

				} /* while */ 

				act_time=SDL_GetTicks();
				if (act_time-time>=REDRAWING_PERIOD)
				{
					act_fps++;
					if ((act_time-fps_timer)>1000) {
						fps_timer=act_time;
						buffer_fps=act_fps;
						buffer_fps_d=fps_d;
						act_fps=0;
						fps_d=0;
					} /* if */ 

					time+=REDRAWING_PERIOD;
					if ((act_time-time)>8*REDRAWING_PERIOD) time=act_time;

				}

				if (!quit) {
					fps_d++;
					/* Draw the file list: */  
					{
						List<char> l;
						char *f;
						int y=64;

						print_centered("Select one game:",font,255,255,255,screen_sfc,SCREEN_X/2,y);
						y+=32;

						l.Instance(files);
						l.Rewind();
						while(l.Iterate(f)) {
							if (strcmp(f,files.GetObj())==0) {
								print_centered(f,font,255,255,255,screen_sfc,SCREEN_X/2,y);
							} else {
								print_centered(f,font,128,128,128,screen_sfc,SCREEN_X/2,y);
							} /* if */ 
							y+=16;
						} /* while */ 
					}

					SDL_Flip(screen_sfc);
				} /* if */ 
			}

			TTF_CloseFont(font);
			font=0;
		} /* if */ 

		if (strcmp(gamefile,"")!=0) {
			MOG2 *game;

			{
				FILE *fp;
				fp=fopen(gamefile,"r");

				if (fp!=0) {
					game=new MOG2();
					if (!game->load(fp)) {
						delete game;
						game=0;
					} /* if */ 
				} /* if */ 
			}

			if (game!=0) {
				ActiveMOG2Application *app=new ActiveMOG2Application(game);
				app->Execute();
				delete app;
			} /* if */ 

			delete game;
		} /* if */ 
		finalizeSDL();
	} /* if */ 

	return 0;
}















