#ifdef _WIN32
#include <windows.h>
#include <windowsx.h>
#else
#endif
#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "List.h"
#include "auxiliar.h"
#include "MOG2_auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "samples.h"

#include "valuescript.h"
#include "conditionscript.h"
#include "actionscript.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BKeySelectionDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "TileViewWnd.h"
#include "TileSelectionWnd.h"
#include "ActionScriptWnd.h"
#include "interfaces.h"
#include "VariableListWnd.h"
#include "VariableSelectorWnd.h"
#include "SampleSelectorDlg.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif


extern int SCREEN_X,SCREEN_Y;

int action_cmd_id[ACTION_MAX_CMD] = {ACTION_CMD_NONE,
									 ACTION_CMD_SEQ,
									 ACTION_CMD_PARALLEL,
									 ACTION_CMD_IF,
									 ACTION_CMD_WHILE,

									 ACTION_CMD_WHILE_ELSE,
									 ACTION_CMD_DO,
									 ACTION_CMD_PAUSE,
									 ACTION_CMD_ATOMIC,
									 ACTION_CMD_TERMINATE,

									 ACTION_CMD_TERMINATECHILD,
									 ACTION_CMD_STATE,
									 ACTION_CMD_SET,
									 ACTION_CMD_MOVE,
									 ACTION_CMD_CREATEOBJECT,

									 ACTION_CMD_CREATECHILD,
									 ACTION_CMD_CREATEGLUEDCHILD,
									 ACTION_CMD_DM_MODE_FACTOR,
									 ACTION_CMD_DM_BLINK,
									 ACTION_CMD_DM_COLOR1,

									 ACTION_CMD_DM_COLOR2,
									 ACTION_CMD_SET_INVINCIBLE,
									 ACTION_CMD_LEAVELADDER,
									 ACTION_CMD_CHANGEMAP,
									 ACTION_CMD_NEWVARIABLE,

									 ACTION_CMD_KILLVARIABLE,
									 ACTION_CMD_PLAY_SAMPLE,
									 ACTION_CMD_PLAY_SAMPLE_PAN,
									 ACTION_CMD_GETCHILD,
									 ACTION_CMD_PLAY_MUSIC,

									 ACTION_CMD_STOP_MUSIC,
									 ACTION_CMD_ADD_FOCUSING,
									 ACTION_CMD_DRAWPIXEL,
									 ACTION_CMD_DRAWLINE,
									 ACTION_CMD_DRAWBOX,

									 ACTION_CMD_DELETE_VIEWPORT,
									 ACTION_CMD_SUB_FOCUSING,
									 ACTION_CMD_RESETGAME,
									 ACTION_CMD_STOPGAME,
									 ACTION_CMD_RESUMEGAME,

									 ACTION_CMD_GETPLAYER,
									 ACTION_CMD_SETPLAYERNAME,
									 ACTION_CMD_SETPLAYERKEYBOARD,
									 ACTION_CMD_GAMEMODE,

									 ACTION_CMD_DRAWBITMAP,
									 ACTION_CMD_GETPARENT,
									 ACTION_CMD_GETROOT,
									 ACTION_CMD_CLEAR_FOCUSING,

									 ACTION_CMD_CASE,
									 ACTION_CMD_RANDOM_CASE,
									 ACTION_CMD_MENU_ADDOPTION,
									 ACTION_CMD_MENU_CLEAROPTIONS,

									 ACTION_CMD_MENU_SUBSTOPTION};

/*

  Tokens:
  %VS: Value script
  %CS: condition script
  %AS: action script
  %L: a list of all the action scripts
  %1L: a list of all the action scripts but the last one
  %VL: a list of pairs: VALUE - ACTION

*/

char *action_cmd_pat[ACTION_MAX_CMD] ={"NONE",
									   "SEQ( %L )",
									   "PARALLEL( %L )",
									   "IF %CS %L",
									   "WHILE( %CS ) { %L }",

									   "WHILE( %CS ) { %1L } ELSE %AS",
									   "DO { %L } WHILE( %CS )",
									   "PAUSE( %VS )",
									   "ATOMIC( %AS )",
									   "TERMINATE",

									   "TERMINATECHILD( %VS , %CS )",
									   "STATE( %VS )",
									   "%VS = %VS",
									   "MOVE( %VS , %VS )",
									   "CREATEOBJECT( %VS , %VS , %VS , %VS )",

									   "CREATECHILD( %VS , %VS , %VS , %VS , %VS , %VS )",
									   "CREATEGLUEDCHILD( %VS , %VS , %VS , %VS , %VS , %VS )",
									   "DM_MODE( %VS , %VS )",
									   "DM_BLINK( %VS , %VS , %VS )",
									   "DM_COLOR1( %VS , %VS , %VS , %VS )",

									   "DM_COLOR2( %VS , %VS , %VS , %VS )",
									   "SET_INVINCIBLE( %AS )",
									   "LEAVELADDER",
									   "CHANGEMAP( %VS , %VS , %VS )",
									   "NEWVARIABLE( %VS )",

									   "KILLVARIABLE( %VS )",
									   "PLAY_SAMPLE( %VS , %VS )",
									   "PLAY_SAMPLE_PAN( %VS , %VS , %VS )",
									   "GET_CHILD( %VS , %CS )",
									   "PLAY_MUSIC( %VS , %VS , %VS )",

									   "STOP_MUSIC",
									   "ADD_FOCUSING( %VS )",
									   "DRAWPIXEL( %VS , %VS , %VS , %VS , %VS , %VS )",
									   "DRAWLINE( %VS , %VS , %VS , %VS , %VS , %VS , %VS , %VS )",
									   "DRAWBOX( %VS , %VS , %VS , %VS , %VS , %VS , %VS , %VS )",

									   "DELETE_VIEWPORT( %VS )",
									   "SUB_FOCUSING( %VS )",
									   "RESET_GAME",
									   "STOP_GAME",
									   "RESUME_GAME",

									   "GETPLAYER( %VS )",
									   "SETPLAYERNAME( %VS , %VS )",
									   "SETPLAYERKEYBOARD( %VS , %VS , %VS , %VS , %VS , %VS , %VS , %VS , %VS )",
									   "GAMEMODE( %VS , %VS )",

									   "DRAWBITMAP( %VS , %VS , %VS , %VS )",
									   "GET_PARENT",
									   "GET_ROOT",
									   "CLEAR_FOCUSING",

									   "CASE( %VS ) { %VL }",
									   "RANDOM_CASE { %VL }",
									   "MENU_ADDOPTION( %VS , %AS )",
									   "MENU_CLEAROPTIONS",

									   "MENU_SUBSTIOPTION( %VS , %VS )"};



int condition_cmd_id[CONDITION_MAX_CMD] = {CONDITION_CMD_TRUE,
										   CONDITION_CMD_FALSE,
										   CONDITION_CMD_NOT,
										   CONDITION_CMD_AND,

										   CONDITION_CMD_OR,
										   CONDITION_CMD_TIMMER,
										   CONDITION_CMD_COLLISION,
										   CONDITION_CMD_COLLISION_WITH_OBJECT,

										   CONDITION_CMD_INSIDE_ROOM,
										   CONDITION_CMD_PARTIALLY_INSIDE_ROOM,
										   CONDITION_CMD_SENSE_OBJECT,
										   CONDITION_CMD_SENSE_SPECIFIC_OBJECT,

										   CONDITION_CMD_KEYPRESS,
										   CONDITION_CMD_KEYPRESSED,
										   CONDITION_CMD_VARIABLE_EQ,
										   CONDITION_CMD_VARIABLE_GR,

										   CONDITION_CMD_VARIABLE_GREQ,
										   CONDITION_CMD_VARIABLE_LW,
										   CONDITION_CMD_VARIABLE_LWEQ,
										   CONDITION_CMD_FIND_SUBSTRING,

										   CONDITION_CMD_INVINCIBLE,
										   CONDITION_CMD_INLADDER,
										   CONDITION_CMD_H_COLLISION,
										   CONDITION_CMD_H_COLLISION_WITH_OBJECT,

										   CONDITION_CMD_H_INSIDE_ROOM,
										   CONDITION_CMD_H_PARTIALLY_INSIDE_ROOM,
										   CONDITION_CMD_LOADGAME,
										   CONDITION_CMD_SAVEGAME,

										   CONDITION_CMD_VALID_OBJECT};

/*

  Tokens:
  %VS: Value script
  %CS: condition script
  %L: a list of all the condition scripts
*/

char *condition_cmd_pat[CONDITION_MAX_CMD] ={"TRUE",
											 "FALSE",
											 "NOT( %CS )",
											 "AND( %L )",
									 
											 "OR( %L )",
											 "TIMMER( %VS )",
											 "COLLISION( %VS , %VS )",
											 "COLLISION_WITH_OBJECT( %VS , %VS , %VS , %CS )",

											 "INSIDE_ROOM( %VS , %VS )",
											 "PARTIALLY_INSIDE_ROOM( %VS , %VS )",
											 "SENSE_OBJECT( %VS , %VS , %VS , %VS , %VS , %CS )",
											 "SENSE_SPECIFIC_OBJECT( %VS , %VS , %VS , %VS , %VS , %CS )",

											 "KEYPRESS( %VS )",
											 "KEYPRESSED( %VS )",
											 "TEST %VS = %VS",
											 "TEST %VS > %VS",

											 "TEST %VS >= %VS",
											 "TEST %VS < %VS",
											 "TEST %VS <= %VS",
											 "FIND_SUBSTRING( %VS , %VS )",
							
											 "INVINCIBLE",
											 "INLADDER",
											 "H_COLLISION( %VS , %VS )",
											 "H_COLLISION_WITH_OBJECT( %VS , %VS , %VS , %CS )",

											 "H_INSIDE_ROOM( %VS , %VS )",
											 "H_PARTIALLY_INSIDE_ROOM( %VS , %VS )",
											 "LOADGAME( %VS )",
											 "SAVEGAME( %VS )",

											 "VALID_OBJECT( %VS )"};



int value_cmd_id[VALUE_MAX_CMD] = {VALUE_CMD_INTEGER,
								   VALUE_CMD_STRING,
								   VALUE_CMD_KEY,
								   VALUE_CMD_VARIABLE,
								   
								   VALUE_CMD_RANDOM,
								   VALUE_CMD_ADD,
								   VALUE_CMD_SUB,
								   VALUE_CMD_MUL,

								   VALUE_CMD_DIV,
								   VALUE_CMD_COUNT,
								   VALUE_CMD_COUNTCHILDREN,
								   VALUE_CMD_MAKE_VARIABLE,
								   
								   VALUE_CMD_CONCATENATE,
								   VALUE_CMD_REMOVE_SUBSTRING,
								   VALUE_CMD_KEYPRESS_ASCII,
								   VALUE_CMD_SAMPLE,

								   VALUE_CMD_GETNPLAYERS,
								   VALUE_CMD_GETPLAYERNAME,
								   VALUE_CMD_GETPLAYERTYPE,
								   VALUE_CMD_GET_FOCUSING,

								   VALUE_CMD_NEW_VIEWPORT,
								   VALUE_CMD_KEYPRESS_KEY,
								   VALUE_CMD_KEYPRESS_NAME,
								   VALUE_CMD_ITOA,

								   VALUE_CMD_ATOI,
								   VALUE_CMD_DOT,
								   VALUE_CMD_ABS,
								   VALUE_CMD_NEWPLAYER,

								   VALUE_CMD_GET_NFOCUSING,
								   VALUE_CMD_DISTANCE};

/*

  Tokens:
  %I: an integer
  %S: a string
  %K: a key
  %V: a variable
  %VS: a Value script
  %CS: a Condition script

*/

char *value_cmd_pat[VALUE_MAX_CMD] ={"%I","%S","%K","%V",
									 
									 "RANDOM( %VS , %VS )",
									 "%VS + %VS",
									 "%VS - %VS",
									 "%VS * %VS",

									 "%VS / %VS",
									 "COUNT( %VS , %VS , %CS )",
									 "COUNTCHILDREN( %VS , %CS )",
									 "MAKE_VARIABLE( %VS , %VS )",

									 "CONCATENATE( %VS , %VS )",
									 "REMOVE_SUBSTRING( %VS , %VS )",
									 "KEYPRESS_ASCII",
									 "SAMPLE %SAM",

									 "GETNPLAYERS",
									 "GETPLAYERNAME( %VS )",
									 "GETPLAYERTYPE( %VS )",
									 "GET_FOCUSING",

									 "NEW_VIEWPORT( %VS , %VS , %VS , %VS , %VS , %VS )",
									 "KEYPRESS_KEY",
									 "KEYPRESS_NAME( %VS )",
									 "ITOA( %VS , %VS )",
									 "ATOI( %VS )",
									 "%VS . %VS",
									 "ABS( %VS )",
									 "NEW_PLAYER( %VS , %VS , %VS , %VS )",

									 "GET_NFOCUSING",
									 "DISTANCE( %VS , %VS )"};




BViewActionScriptWnd::BViewActionScriptWnd(int x,int y,int dx,int dy,ActionSCRIPT *a_s,ConditionSCRIPT *c_s,ValueSCRIPT *v_s,OBJECT *o,ROOM *r,MAP *m,MOG2 *g,INTERFACE *interfp,EDITOR *e,int lvl,TTF_Font *font) : BWindow(x,y,dx,dy,font)
{
	editor=e;
	a_script=a_s;
	c_script=c_s;
	v_script=v_s;
	object=o;
	room=r;
	map=m;
	game=g;
	interf=interfp;
	level=lvl;
} /* BViewActionScriptWnd::BViewActionScriptWnd */ 


BViewActionScriptWnd::~BViewActionScriptWnd()
{
} /* BViewActionScriptWnd::~BViewActionScriptWnd */ 


bool BViewActionScriptWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (local_mouse_x>old_stx &&
		local_mouse_y>old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy &&
		mouse_button&SDL_BUTTON(1) &&
		!(old_mouse_button&SDL_BUTTON(1))) {
		click_on_script();
		if (a_script!=0) a_script->simplify();
		if (c_script!=0) c_script->simplify();
		if (v_script!=0) v_script->simplify();
		if (a_script!=0) a_recompute_sizes(a_script);
		if (c_script!=0) c_recompute_sizes(c_script);
		if (v_script!=0) v_recompute_sizes(v_script);
		dirty=true;
	} /* if */ 


	if (local_mouse_x>old_stx &&
		local_mouse_y>old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy &&
		mouse_button&SDL_BUTTON(3) &&
		!(old_mouse_button&SDL_BUTTON(3))) {
		right_click_on_script();
		if (a_script!=0) a_script->simplify();
		if (c_script!=0) c_script->simplify();
		if (v_script!=0) v_script->simplify();
		if (a_script!=0) a_recompute_sizes(a_script);
		if (c_script!=0) c_recompute_sizes(c_script);
		if (v_script!=0) v_recompute_sizes(v_script);
		dirty=true;
	} /* if */ 


	return true;
} /* BViewActionScriptWnd::cycle */ 


void BViewActionScriptWnd::contens_size(int *dx,int *dy)
{
	int max_x=0;
	int max_y=0;

	if (a_script!=0) a_script_size(a_script,&max_x,&max_y);
	if (c_script!=0) c_script_size(c_script,&max_x,&max_y);
	if (v_script!=0) v_script_size(v_script,&max_x,&max_y);

	*dx=old_dx;
	*dy=old_dy;

	if (max_x+8>*dx) *dx=max_x+8;
	if (max_y+8>*dy) *dy=max_y+8;

} /* BViewActionScriptWnd::contens_size */ 


void BViewActionScriptWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{

	SDL_FillRect(sfc,0,0);

	a_selected_script=0;
	c_selected_script=0;
	v_selected_script=0;
	selected_object=0;

	if (a_script!=0) a_draw_script(sfc,a_script,4-stx,4-sty);
	if (c_script!=0) c_draw_script(sfc,c_script,4-stx,4-sty);
	if (v_script!=0) v_draw_script(sfc,v_script,4-stx,4-sty);

	if (a_selected_script!=0 ||
		c_selected_script!=0 ||
		v_selected_script!=0) draw_rectangle(sfc,selected.x,selected.y,selected.w,selected.h,0xffffff);

	set_dirty();

} /* BViewActionScriptWnd::draw_contens */ 


void BViewActionScriptWnd::a_draw_script(SDL_Surface *sfc,ActionSCRIPT *script,int stx,int sty)
{
	int i;
	int dx,dy;
	SDL_Rect r;
	char *pattern;

	a_script_size(script,&dx,&dy);
	r.x=stx;
	r.y=sty;
	r.w=dx+4;
	r.h=dy+4;

	if (local_mouse_x>=stx+old_stx && local_mouse_y>=sty+old_sty &&
		local_mouse_x<stx+dx+4+old_stx && local_mouse_y<sty+dy+4+old_sty) {
		selected.x=stx;
		selected.y=sty;
		selected.w=dx+4;
		selected.h=dy+4;
		a_selected_script=script;
		c_selected_script=0;
		v_selected_script=0;
	} /* if */ 
	SDL_FillRect(sfc,&r,0x0000A0);
	draw_rectangle(sfc,r.x,r.y,r.w,r.h,0x0000F0);

	pattern=0;
	for(i=0;i<ACTION_MAX_CMD;i++) {
		if (script->command==action_cmd_id[i]) pattern=action_cmd_pat[i];
	} /* for */ 

	if (pattern!=0) {
		int vcount=0;
		int acount=0;
		int x,y;
		int offset=0;
		char token[256];
		int dx2,dy2;
		int dx3,dy3;

		x=stx+2;
		y=sty+2;
		while(offset<int(strlen(pattern)) &&
			  1==sscanf(pattern
			  +offset,"%s",token)) {
			offset+=strlen(token)+1;

			if (strcmp(token,"%VS")==0) {

				v_script_size(script->vpar[vcount],&dx2,&dy2);
				v_draw_script(sfc,script->vpar[vcount],x,y);

				vcount++;
				x+=dx2+4+TTF_FontHeight(font);
			} else if(strcmp(token,"%CS")==0) {
				c_script_size(script->cond,&dx2,&dy2);
				c_draw_script(sfc,script->cond,x,y);

				x+=dx2+4+TTF_FontHeight(font);
			} else if(strcmp(token,"%AS")==0) {
				if (script->aparlist.Length()>acount) {
					a_script_size(script->aparlist[acount],&dx2,&dy2);
					a_draw_script(sfc,script->aparlist[acount],x,y);
					acount++;
					x+=dx2+4+TTF_FontHeight(font);
				} // if 
			} else if(strcmp(token,"%L")==0) {
				while(acount<script->aparlist.Length()) {
					a_script_size(script->aparlist[acount],&dx2,&dy2);
					a_draw_script(sfc,script->aparlist[acount],x,y);
					acount++;
					y+=dy2+5;
				} /* while */ 
			} else if(strcmp(token,"%1L")==0) {
				while(acount<(script->aparlist.Length()-1)) {
					a_script_size(script->aparlist[acount],&dx2,&dy2);
					a_draw_script(sfc,script->aparlist[acount],x,y);
					acount++;
					y+=dy2+5;
				} /* while */ 
			} else if(strcmp(token,"%VL")==0) {
				while(acount<script->aparlist.Length()) {
					v_script_size(script->vparlist[acount],&dx3,&dy3);
					v_draw_script(sfc,script->vparlist[acount],x,y);
					a_script_size(script->aparlist[acount],&dx2,&dy2);
					a_draw_script(sfc,script->aparlist[acount],x+dx3+5,y);
					acount++;
					y+=max(dy2,dy3)+5;
				} /* while */ 
			} else {
				print_left(token,font,255,255,255,sfc,x,y);
				x+=get_text_width(token,font)+TTF_FontHeight(font);
			} /* if */ 
		} /* while */ 
	} /* if */ 

} /* BViewActionScriptWnd::a_draw_script */ 


void BViewActionScriptWnd::c_draw_script(SDL_Surface *sfc,ConditionSCRIPT *script,int stx,int sty)
{
	int i;
	int dx,dy;
	SDL_Rect r;
	char *pattern;

	c_script_size(script,&dx,&dy);
	r.x=stx;
	r.y=sty;
	r.w=dx+4;
	r.h=dy+4;
	if (local_mouse_x>=stx+old_stx && local_mouse_y>=sty+old_sty &&
		local_mouse_x<stx+dx+4+old_stx && local_mouse_y<sty+dy+4+old_sty) {
		selected.x=stx;
		selected.y=sty;
		selected.w=dx+4;
		selected.h=dy+4;
		a_selected_script=0;
		c_selected_script=script;
		v_selected_script=0;
	} /* if */ 
	SDL_FillRect(sfc,&r,0xA000A0);
	draw_rectangle(sfc,r.x,r.y,r.w,r.h,0xF000F0);

	pattern=0;
	for(i=0;i<CONDITION_MAX_CMD;i++) {
		if (script->command==condition_cmd_id[i]) pattern=condition_cmd_pat[i];
	} /* for */ 

	if (pattern!=0) {
		int vcount=0;
		int ccount=0;
		int x,y;
		int offset=0;
		char token[256];

		x=stx+2;
		y=sty+2;
		while(offset<int(strlen(pattern)) &&
			  1==sscanf(pattern+offset,"%s",token)) {
			offset+=strlen(token)+1;

			if (strcmp(token,"%VS")==0) {
				int dx2,dy2;

				v_script_size(script->vpar[vcount],&dx2,&dy2);
				v_draw_script(sfc,script->vpar[vcount],x,y);

				vcount++;
				x+=dx2+4+TTF_FontHeight(font);
			} else if(strcmp(token,"%CS")==0) {
				int dx2,dy2;

				c_script_size(script->aparlist[ccount],&dx2,&dy2);
				c_draw_script(sfc,script->aparlist[ccount],x,y);

				ccount++;
				x+=dx2+4+TTF_FontHeight(font);
			} else if(strcmp(token,"%L")==0) {
				int dx2,dy2;

				while(ccount<script->aparlist.Length()) {
					c_script_size(script->aparlist[ccount],&dx2,&dy2);
					c_draw_script(sfc,script->aparlist[ccount],x,y);
					ccount++;
					y+=dy2+5;
				} /* while */ 
			} else {
				print_left(token,font,255,255,255,sfc,x,y);
				x+=get_text_width(token,font)+TTF_FontHeight(font);
			} /* if */ 
		} /* while */ 
	} /* if */ 

} /* BViewActionScriptWnd::c_draw_script */ 


void BViewActionScriptWnd::v_draw_script(SDL_Surface *sfc,ValueSCRIPT *script,int stx,int sty)
{
	int i;
	int dx,dy;
	SDL_Rect r;
	char tmp[256];
	char *pattern;

	v_script_size(script,&dx,&dy);
	r.x=stx;
	r.y=sty;
	r.w=dx+4;
	r.h=dy+4;
	if (local_mouse_x>=stx+old_stx && local_mouse_y>=sty+old_sty &&
		local_mouse_x<stx+dx+4+old_stx && local_mouse_y<sty+dy+4+old_sty) {
		selected.x=stx;
		selected.y=sty;
		selected.w=dx+4;
		selected.h=dy+4;
		a_selected_script=0;
		c_selected_script=0;
		v_selected_script=script;
	} /* if */ 
	if (script->command==VALUE_CMD_VARIABLE) SDL_FillRect(sfc,&r,0x770000);
										else SDL_FillRect(sfc,&r,0xAA0000);
	if (script->command==VALUE_CMD_VARIABLE) draw_rectangle(sfc,r.x,r.y,r.w,r.h,0xAA0000);
										else draw_rectangle(sfc,r.x,r.y,r.w,r.h,0xFF0000);

	pattern=0;
	for(i=0;i<VALUE_MAX_CMD;i++) {
		if (script->command==value_cmd_id[i]) pattern=value_cmd_pat[i];
	} /* for */ 

	if (pattern!=0) {
		int vcount=0;
		int x,y;
		int offset=0;
		char token[256];

		x=stx+2;
		y=sty+2;
		while(offset<int(strlen(pattern)) &&
			  1==sscanf(pattern+offset,"%s",token)) {
			offset+=strlen(token)+1;

			if (strcmp(token,"%I")==0) {
				sprintf(tmp,"%i",script->ipar);
				print_left(tmp,font,255,255,255,sfc,x,y);
				x+=get_text_width(tmp,font)+TTF_FontHeight(font);
			} else if(strcmp(token,"%S")==0) {
				print_left(script->spar,font,255,255,255,sfc,x,y);
				x+=get_text_width(script->spar,font)+TTF_FontHeight(font);
			} else if(strcmp(token,"%SAM")==0) {
				if (script->sample!=0) {
					print_left(script->sample->name,font,255,255,255,sfc,x,y);
					x+=get_text_width(script->sample->name,font)+TTF_FontHeight(font);
				} else {
					print_left("-",font,255,255,255,sfc,x,y);
					x+=get_text_width("-",font)+TTF_FontHeight(font);
				} /* if */ 
			} else if(strcmp(token,"%K")==0) {
				sprintf(tmp,input_key_name(script->ipar));
				print_left(tmp,font,255,255,255,sfc,x,y);
				x+=get_text_width(tmp,font)+TTF_FontHeight(font);
			} else if(strcmp(token,"%V")==0) {
				tmp[0]=0;
				if (script->vlocation==VALUE_VAR_OBJECT) sprintf(tmp,"%s(OBJECT)",script->vname);
				if (script->vlocation==VALUE_VAR_ROOM) sprintf(tmp,"%s(ROOM)",script->vname);
				if (script->vlocation==VALUE_VAR_MAP) sprintf(tmp,"%s(MAP)",script->vname);
				if (script->vlocation==VALUE_VAR_GAME) sprintf(tmp,"%s(GAME)",script->vname);
				if (script->vlocation==VALUE_VAR_INTERFACE) sprintf(tmp,"%s(INTERFACE)",script->vname);
				if (script->vlocation==VALUE_VAR_FOREIGN) sprintf(tmp,"%s(FOREIGN)",script->vname);
				if (script->vlocation==VALUE_VAR_APPLICATION) sprintf(tmp,"%s(APPLICATION)",script->vname);
				print_left(tmp,font,255,255,255,sfc,x,y);
				x+=get_text_width(tmp,font)+TTF_FontHeight(font);
			} else if(strcmp(token,"%VS")==0) {
				int dx2,dy2;

				v_script_size(script->vpar[vcount],&dx2,&dy2);
				v_draw_script(sfc,script->vpar[vcount],x,y);

				vcount++;
				x+=dx2+4+TTF_FontHeight(font);
			} else if(strcmp(token,"%CS")==0) {
				int dx2,dy2;

				c_script_size(script->cond,&dx2,&dy2);
				c_draw_script(sfc,script->cond,x,y);

				vcount++;
				x+=dx2+4+TTF_FontHeight(font);
			} else {
				print_left(token,font,255,255,255,sfc,x,y);
				x+=get_text_width(token,font)+TTF_FontHeight(font);
			} /* if */ 

		} /* while */ 

	} /* if */ 
} /* BViewActionScriptWnd::v_draw_script */ 




bool BViewActionScriptWnd::click_on_script(void)
{
	if (v_selected_script!=0) {
		int res;
		BVChoiceDlg *dlg;

		/* OBJECT SCRIPTS: */ 
		if (level==0) {
			if (object->control!=CTRL_GAMEWINDOW) {
				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","integer\nstring\nkey\nvariable\nrandom\n"
																"+\n-\n*\n/\ncount\ncountchildren\n"
																"concatenate\nremove_substring\n"
																"make_variable\nkeypress_ascii\n"
																"keypress_key\nkeypress_name\nsample\n"
																"getnplayers\ngetplayername\n"
																"getplayertype\n"
																"itoa\natoi\n.\nabs\ndistance\n",&res,font);
			} else {
				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","integer\nstring\nkey\nvariable\nrandom\n"
																"+\n-\n*\n/\ncount\ncountchildren\n"
																"concatenate\nremove_substring\n"
																"make_variable\nkeypress_ascii\n"
																"keypress_key\nkeypress_name\nsample\n"
																"getnplayers\ngetplayername\n"
																"getplayertype\n"
																"itoa\natoi\n.\nabs\ndistance\n"
																"get_focusing\nget_nfocusing\n",&res,font);
			} /* if */ 

			dlg->Execute();
			delete dlg;

			switch(res) {
			case 1:
				{
					BIntegerEdDlg *dlg;
					int y,x;

					v_selected_script->command=VALUE_CMD_INTEGER;
					delete []v_selected_script->spar;
					v_selected_script->spar=0;

					new_window_coordinates(140,60,&x,&y);

					dlg=new BIntegerEdDlg(x,y,140,60,"Enter value:",&v_selected_script->ipar,6,font);
					dlg->Execute();
					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 2:			
				{
					BStringEdDlg *dlg;
					int y,x;
					char tmp[128];

					v_selected_script->command=VALUE_CMD_STRING;
					v_selected_script->ipar=0;
					if (v_selected_script->spar==0) {
						v_selected_script->spar=new char[1];
						v_selected_script->spar[0]=0;
					} /* if */ 

					new_window_coordinates(int(SCREEN_X*0.8),60,&x,&y);
					strcpy(tmp,v_selected_script->spar);
					dlg=new BStringEdDlg(x,y,int(SCREEN_X*0.8),60,"Enter string:",tmp,128,"",font);
					dlg->Execute();
					delete dlg;

					delete []v_selected_script->spar;
					v_selected_script->spar=new char[strlen(tmp)+1];
					strcpy(v_selected_script->spar,tmp);

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;

				}
				break;
			case 3:
				{
					BKeyDlg *dlg;
					int y,x;

					v_selected_script->command=VALUE_CMD_KEY;
					delete v_selected_script->spar;
					v_selected_script->spar=0;

					x=mouse_x-125;
					y=mouse_y;
					if (y>(SCREEN_Y-100)) y=SCREEN_Y-100;
					if (x>(SCREEN_X-250)) x=SCREEN_X-250;
					if (x<0) x=0;

					dlg=new BKeyDlg(x,y,250,100,"Enter key:",&v_selected_script->ipar,font);
					dlg->Execute();
					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 4:
				{
					int x,y;
					BVariableSelectorWnd *dlg;

					new_window_coordinates(300,300,&x,&y);
					if (object==0) {
							dlg=new BVariableSelectorWnd(x,y,300,300,editor,0,room,map,game,interf,font);
					} else {
						dlg=new BVariableSelectorWnd(x,y,300,300,editor,object,room,map,game,interf,font);
					} /* if */ 
					dlg->Execute();

					if (dlg->vlw->selected!=0) {

						v_selected_script->command=VALUE_CMD_VARIABLE;
						v_selected_script->ipar=0;
						delete v_selected_script->spar;
						v_selected_script->spar=0;

						delete []v_selected_script->vname;
						v_selected_script->vname=new char[strlen(dlg->vlw->selected)+1];
						strcpy(v_selected_script->vname,dlg->vlw->selected);
//						strlwr(v_selected_script->vname);
						v_selected_script->vlocation=dlg->vlw->scope;
					} /* if */ 

					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 5:
				{
					v_selected_script->command=VALUE_CMD_RANDOM;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=1;
				}
				break;
			case 6:
				{
					if (v_selected_script->command!=VALUE_CMD_ADD &&
						v_selected_script->command!=VALUE_CMD_SUB &&
						v_selected_script->command!=VALUE_CMD_MUL &&
						v_selected_script->command!=VALUE_CMD_DIV) {
						if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
						if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
						v_selected_script->vpar[0]=new ValueSCRIPT();
						v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
						v_selected_script->vpar[0]->ipar=0;
						v_selected_script->vpar[1]=new ValueSCRIPT();
						v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
						v_selected_script->vpar[1]->ipar=0;
					} /* if */ 
					v_selected_script->command=VALUE_CMD_ADD;
				}
				break;
			case 7:
				{
					if (v_selected_script->command!=VALUE_CMD_ADD &&
						v_selected_script->command!=VALUE_CMD_SUB &&
						v_selected_script->command!=VALUE_CMD_MUL &&
						v_selected_script->command!=VALUE_CMD_DIV) {
						if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
						if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
						v_selected_script->vpar[0]=new ValueSCRIPT();
						v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
						v_selected_script->vpar[0]->ipar=0;
						v_selected_script->vpar[1]=new ValueSCRIPT();
						v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
						v_selected_script->vpar[1]->ipar=0;
					} /* if */ 
					v_selected_script->command=VALUE_CMD_SUB;
				}
				break;
			case 8:
				{
					if (v_selected_script->command!=VALUE_CMD_ADD &&
						v_selected_script->command!=VALUE_CMD_SUB &&
						v_selected_script->command!=VALUE_CMD_MUL &&
						v_selected_script->command!=VALUE_CMD_DIV) {
						if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
						if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
						v_selected_script->vpar[0]=new ValueSCRIPT();
						v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
						v_selected_script->vpar[0]->ipar=0;
						v_selected_script->vpar[1]=new ValueSCRIPT();
						v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
						v_selected_script->vpar[1]->ipar=0;
					} /* if */ 
					v_selected_script->command=VALUE_CMD_MUL;
				}
				break;
			case 9:
				{
					if (v_selected_script->command!=VALUE_CMD_ADD &&
						v_selected_script->command!=VALUE_CMD_SUB &&
						v_selected_script->command!=VALUE_CMD_MUL &&
						v_selected_script->command!=VALUE_CMD_DIV) {
						if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
						if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
						v_selected_script->vpar[0]=new ValueSCRIPT();
						v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
						v_selected_script->vpar[0]->ipar=0;
						v_selected_script->vpar[1]=new ValueSCRIPT();
						v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
						v_selected_script->vpar[1]->ipar=0;
					} /* if */ 
					v_selected_script->command=VALUE_CMD_DIV;
				}
				break;
			case 10:
				{
					v_selected_script->command=VALUE_CMD_COUNT;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"room");
				}
				break;
			case 11:
				{
					v_selected_script->command=VALUE_CMD_COUNTCHILDREN;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");

					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 12:
				{
					v_selected_script->command=VALUE_CMD_CONCATENATE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"none");
				}
				break;
			case 13:
				{
					v_selected_script->command=VALUE_CMD_REMOVE_SUBSTRING;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"none");
				}
				break;
			case 14:
				{
					v_selected_script->command=VALUE_CMD_MAKE_VARIABLE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[7];
					strcpy(v_selected_script->vpar[1]->spar,"object");
				}
				break;
			case 15:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_ASCII;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 16:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_KEY;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 17:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_NAME;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 18:
				{
					v_selected_script->command=VALUE_CMD_SAMPLE;
					v_selected_script->ipar=0;
					v_selected_script->spar=0;
					v_selected_script->sample=0;

					{
						int x,y;
						SAMPLE *s=0;
						SampleSelectorDlg *dlg;

						new_window_coordinates(200,200,&x,&y);
						dlg=new SampleSelectorDlg(x,y,200,200,editor,&s,font);
						dlg->Execute();
						delete dlg;

						if (s!=0) v_selected_script->sample=s;
					}

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 19:
				{
					v_selected_script->command=VALUE_CMD_GETNPLAYERS;
				}
				break;
			case 20:
				{
					v_selected_script->command=VALUE_CMD_GETPLAYERNAME;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
				}
				break;
			case 21:
				{
					v_selected_script->command=VALUE_CMD_GETPLAYERTYPE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
				}
				break;
			case 22:
				{
					v_selected_script->command=VALUE_CMD_ITOA;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=new ValueSCRIPT();
				}
				break;
			case 23:
				{
					v_selected_script->command=VALUE_CMD_ATOI;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[2];
					strcpy(v_selected_script->vpar[0]->spar,"0");
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 24:
				{
					v_selected_script->command=VALUE_CMD_DOT;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 25:
				{
					v_selected_script->command=VALUE_CMD_ABS;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
				}
				break;
			case 26:
				{
					v_selected_script->command=VALUE_CMD_DISTANCE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[3];
					strcpy(v_selected_script->vpar[1]->spar,"xy");
				}
				break;
			case 27:
				v_selected_script->command=VALUE_CMD_GET_FOCUSING;
				break;
			case 28:
				v_selected_script->command=VALUE_CMD_GET_NFOCUSING;
				break;
			} /* switch */ 
			return true;
		} /* if */ 



		/* ROOM/MAP/GAME SCRIPTS: */ 
		if (level==1 || level==2 || level==3) {
			dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","integer\nstring\nkey\nvariable\nrandom\n"
															"+\n-\n*\n/\ncount\n"
															"concatenate\nremove_substring\n"
															"make_variable\nkeypress_ascii\n"
															"keypress_key\nkeypress_name\nsample\n"
															"itoa\natoi\n.\nabs\n",&res,font);
			dlg->Execute();
			delete dlg;

			switch(res) {
			case 1:
				{
					BIntegerEdDlg *dlg;
					int y,x;

					v_selected_script->command=VALUE_CMD_INTEGER;
					delete []v_selected_script->spar;
					v_selected_script->spar=0;

					x=mouse_x-110;
					y=mouse_y;
					if (y>(SCREEN_Y-60)) y=SCREEN_Y-60;
					if (x>(SCREEN_X-140)) x=SCREEN_X-140;
					if (x<0) x=0;

					dlg=new BIntegerEdDlg(x,y,140,60,"Enter value:",&v_selected_script->ipar,6,font);
					dlg->Execute();
					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 2:			
				{
					BStringEdDlg *dlg;
					int y,x;
					char tmp[128];

					v_selected_script->command=VALUE_CMD_STRING;
					v_selected_script->ipar=0;
					if (v_selected_script->spar==0) {
						v_selected_script->spar=new char[1];
						v_selected_script->spar[0]=0;
					} /* if */ 

					new_window_coordinates(int(SCREEN_X*0.8),60,&x,&y);
					strcpy(tmp,v_selected_script->spar);
					dlg=new BStringEdDlg(x,y,int(SCREEN_X*0.8),60,"Enter string:",tmp,128,"",font);
					dlg->Execute();
					delete dlg;

					delete []v_selected_script->spar;
					v_selected_script->spar=new char[strlen(tmp)+1];
					strcpy(v_selected_script->spar,tmp);

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;

				}
				break;
			case 3:
				{
					BKeyDlg *dlg;
					int y,x;

					v_selected_script->command=VALUE_CMD_KEY;
					delete v_selected_script->spar;
					v_selected_script->spar=0;

					x=mouse_x-125;
					y=mouse_y;
					if (y>(SCREEN_Y-100)) y=SCREEN_Y-100;
					if (x>(SCREEN_X-250)) x=SCREEN_X-250;
					if (x<0) x=0;

					dlg=new BKeyDlg(x,y,250,100,"Enter key:",&v_selected_script->ipar,font);
					dlg->Execute();
					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 4:
				{
					int x,y;
					BVariableSelectorWnd *dlg;

					new_window_coordinates(300,300,&x,&y);
					dlg=new BVariableSelectorWnd(x,y,300,300,editor,0,room,map,game,interf,font);
					dlg->Execute();

					if (dlg->vlw->selected!=0) {

						v_selected_script->command=VALUE_CMD_VARIABLE;
						v_selected_script->ipar=0;
						delete v_selected_script->spar;
						v_selected_script->spar=0;

						delete v_selected_script->vname;
						v_selected_script->vname=new char[strlen(dlg->vlw->selected)+1];
						strcpy(v_selected_script->vname,dlg->vlw->selected);
//						strlwr(v_selected_script->vname);
						v_selected_script->vlocation=dlg->vlw->scope;
					} /* if */ 

					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 5:
				{
					v_selected_script->command=VALUE_CMD_RANDOM;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=1;
				}
				break;
			case 6:
				{
					v_selected_script->command=VALUE_CMD_ADD;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 7:
				{
					v_selected_script->command=VALUE_CMD_SUB;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 8:
				{
					v_selected_script->command=VALUE_CMD_MUL;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 9:
				{
					v_selected_script->command=VALUE_CMD_DIV;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 10:
				{
					v_selected_script->command=VALUE_CMD_COUNT;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"room");
				}
				break;
			case 11:
				{
					v_selected_script->command=VALUE_CMD_CONCATENATE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"none");
				}
				break;
			case 12:
				{
					v_selected_script->command=VALUE_CMD_REMOVE_SUBSTRING;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"none");
				}
				break;
			case 13:
				{
					v_selected_script->command=VALUE_CMD_MAKE_VARIABLE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[7];
					strcpy(v_selected_script->vpar[1]->spar,"object");
				}
				break;
			case 14:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_ASCII;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 15:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_KEY;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 16:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_NAME;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 17:
				{
					v_selected_script->command=VALUE_CMD_SAMPLE;
					v_selected_script->ipar=0;
					v_selected_script->spar=0;
					v_selected_script->sample=0;

					{
						int x,y;
						SAMPLE *s=0;
						SampleSelectorDlg *dlg;

						new_window_coordinates(200,200,&x,&y);
						dlg=new SampleSelectorDlg(x,y,200,200,editor,&s,font);
						dlg->Execute();
						delete dlg;

						if (s!=0) v_selected_script->sample=s;
					}

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 18:
				{
					v_selected_script->command=VALUE_CMD_ITOA;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=new ValueSCRIPT();
				}
				break;
			case 19:
				{
					v_selected_script->command=VALUE_CMD_ATOI;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[2];
					strcpy(v_selected_script->vpar[0]->spar,"0");
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 20:
				{
					v_selected_script->command=VALUE_CMD_DOT;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 21:
				{
					v_selected_script->command=VALUE_CMD_ABS;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
				}
				break;

			} /* switch */ 
			return true;
		} /* if */ 





		/* INTERFACE SCRIPTS: */ 
		if (level==4) {
			dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","integer\nstring\nkey\nvariable\nrandom\n"
															"+\n-\n*\n/\ncount\n"
															"concatenate\nremove_substring\n"
															"make_variable\nkeypress_ascii\n"
															"keypress_key\nkeypress_name\nsample\n"
															"getnplayers\ngetplayername\n"
															"getplayertype\n"
															"itoa\natoi\n.\nabs\n",&res,font);
			dlg->Execute();
			delete dlg;

			switch(res) {
			case 1:
				{
					BIntegerEdDlg *dlg;
					int y,x;

					v_selected_script->command=VALUE_CMD_INTEGER;
					delete []v_selected_script->spar;
					v_selected_script->spar=0;

					x=mouse_x-110;
					y=mouse_y;
					if (y>(SCREEN_Y-60)) y=SCREEN_Y-60;
					if (x>(SCREEN_X-140)) x=SCREEN_X-140;
					if (x<0) x=0;

					dlg=new BIntegerEdDlg(x,y,140,60,"Enter value:",&v_selected_script->ipar,6,font);
					dlg->Execute();
					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 2:			
				{
					BStringEdDlg *dlg;
					int y,x;
					char tmp[128];

					v_selected_script->command=VALUE_CMD_STRING;
					v_selected_script->ipar=0;
					if (v_selected_script->spar==0) {
						v_selected_script->spar=new char[1];
						v_selected_script->spar[0]=0;
					} /* if */ 

					new_window_coordinates(int(SCREEN_X*0.8),60,&x,&y);
					strcpy(tmp,v_selected_script->spar);
					dlg=new BStringEdDlg(x,y,int(SCREEN_X*0.8),60,"Enter string:",tmp,128,"",font);
					dlg->Execute();
					delete dlg;

					delete []v_selected_script->spar;
					v_selected_script->spar=new char[strlen(tmp)+1];
					strcpy(v_selected_script->spar,tmp);

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;

				}
				break;
			case 3:
				{
					BKeyDlg *dlg;
					int y,x;

					v_selected_script->command=VALUE_CMD_KEY;
					delete v_selected_script->spar;
					v_selected_script->spar=0;

					x=mouse_x-125;
					y=mouse_y;
					if (y>(SCREEN_Y-100)) y=SCREEN_Y-100;
					if (x>(SCREEN_X-250)) x=SCREEN_X-250;
					if (x<0) x=0;

					dlg=new BKeyDlg(x,y,250,100,"Enter key:",&v_selected_script->ipar,font);
					dlg->Execute();
					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 4:
				{
					int x,y;
					BVariableSelectorWnd *dlg;

					new_window_coordinates(300,300,&x,&y);
					dlg=new BVariableSelectorWnd(x,y,300,300,editor,0,room,map,game,interf,font);
					dlg->Execute();

					if (dlg->vlw->selected!=0) {

						v_selected_script->command=VALUE_CMD_VARIABLE;
						v_selected_script->ipar=0;
						delete v_selected_script->spar;
						v_selected_script->spar=0;

						delete v_selected_script->vname;
						v_selected_script->vname=new char[strlen(dlg->vlw->selected)+1];
						strcpy(v_selected_script->vname,dlg->vlw->selected);
//						strlwr(v_selected_script->vname);
						v_selected_script->vlocation=dlg->vlw->scope;
					} /* if */ 

					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 5:
				{
					v_selected_script->command=VALUE_CMD_RANDOM;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=1;
				}
				break;
			case 6:
				{
					v_selected_script->command=VALUE_CMD_ADD;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 7:
				{
					v_selected_script->command=VALUE_CMD_SUB;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 8:
				{
					v_selected_script->command=VALUE_CMD_MUL;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 9:
				{
					v_selected_script->command=VALUE_CMD_DIV;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 10:
				{
					v_selected_script->command=VALUE_CMD_COUNT;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"room");
				}
				break;
			case 11:
				{
					v_selected_script->command=VALUE_CMD_CONCATENATE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"none");
				}
				break;
			case 12:
				{
					v_selected_script->command=VALUE_CMD_REMOVE_SUBSTRING;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"none");
				}
				break;
			case 13:
				{
					v_selected_script->command=VALUE_CMD_MAKE_VARIABLE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[7];
					strcpy(v_selected_script->vpar[1]->spar,"object");
				}
				break;
			case 14:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_ASCII;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 15:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_KEY;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 16:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_NAME;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 17:
				{
					v_selected_script->command=VALUE_CMD_SAMPLE;
					v_selected_script->ipar=0;
					v_selected_script->spar=0;
					v_selected_script->sample=0;

					{
						int x,y;
						SAMPLE *s=0;
						SampleSelectorDlg *dlg;

						new_window_coordinates(200,200,&x,&y);
						dlg=new SampleSelectorDlg(x,y,200,200,editor,&s,font);
						dlg->Execute();
						delete dlg;

						if (s!=0) v_selected_script->sample=s;
					}

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;

			case 18:
				{
					v_selected_script->command=VALUE_CMD_GETNPLAYERS;
				}
				break;
			case 19:
				{
					v_selected_script->command=VALUE_CMD_GETPLAYERNAME;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
				}
				break;
			case 20:
				{
					v_selected_script->command=VALUE_CMD_GETPLAYERTYPE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
				}
				break;
			case 21:
				{
					v_selected_script->command=VALUE_CMD_ITOA;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=new ValueSCRIPT();
				}
				break;
			case 22:
				{
					v_selected_script->command=VALUE_CMD_ATOI;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[2];
					strcpy(v_selected_script->vpar[0]->spar,"0");
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 23:
				{
					v_selected_script->command=VALUE_CMD_DOT;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 24:
				{
					v_selected_script->command=VALUE_CMD_ABS;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
				}
				break;
			} /* switch */ 
			return true;
		} /* if */ 






		/* APPLICATION SCRIPTS: */ 
		if (level==5) {
			dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","integer\nstring\nkey\nvariable\nrandom\n"
															"+\n-\n*\n/\ncount\n"
															"concatenate\nremove_substring\n"
															"make_variable\nkeypress_ascii\n"
															"keypress_key\nkeypress_name\nsample\n"
															"getnplayers\ngetplayername\n"
															"getplayertype\n"
															"new_viewport\n"
															"itoa\natoi\n.\nabs\nnew_player\n",&res,font);
			dlg->Execute();
			delete dlg;

			switch(res) {
			case 1:
				{
					BIntegerEdDlg *dlg;
					int y,x;

					v_selected_script->command=VALUE_CMD_INTEGER;
					delete []v_selected_script->spar;
					v_selected_script->spar=0;

					x=mouse_x-110;
					y=mouse_y;
					if (y>(SCREEN_Y-60)) y=SCREEN_Y-60;
					if (x>(SCREEN_X-140)) x=SCREEN_X-140;
					if (x<0) x=0;

					dlg=new BIntegerEdDlg(x,y,140,60,"Enter value:",&v_selected_script->ipar,6,font);
					dlg->Execute();
					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 2:			
				{
					BStringEdDlg *dlg;
					int y,x;
					char tmp[128];

					v_selected_script->command=VALUE_CMD_STRING;
					v_selected_script->ipar=0;
					if (v_selected_script->spar==0) {
						v_selected_script->spar=new char[1];
						v_selected_script->spar[0]=0;
					} /* if */ 

					new_window_coordinates(int(SCREEN_X*0.8),60,&x,&y);
					strcpy(tmp,v_selected_script->spar);
					dlg=new BStringEdDlg(x,y,int(SCREEN_X*0.8),60,"Enter string:",tmp,128,"",font);
					dlg->Execute();
					delete dlg;

					delete []v_selected_script->spar;
					v_selected_script->spar=new char[strlen(tmp)+1];
					strcpy(v_selected_script->spar,tmp);

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;

				}
				break;
			case 3:
				{
					BKeyDlg *dlg;
					int y,x;

					v_selected_script->command=VALUE_CMD_KEY;
					delete v_selected_script->spar;
					v_selected_script->spar=0;

					new_window_coordinates(250,100,&x,&y);
					dlg=new BKeyDlg(x,y,250,100,"Enter key:",&v_selected_script->ipar,font);
					dlg->Execute();
					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 4:
				{
					int x,y;
					BVariableSelectorWnd *dlg;

					new_window_coordinates(300,300,&x,&y);
					dlg=new BVariableSelectorWnd(x,y,300,300,editor,0,room,map,game,interf,font);
					dlg->Execute();

					if (dlg->vlw->selected!=0) {

						v_selected_script->command=VALUE_CMD_VARIABLE;
						v_selected_script->ipar=0;
						delete v_selected_script->spar;
						v_selected_script->spar=0;

						delete v_selected_script->vname;
						v_selected_script->vname=new char[strlen(dlg->vlw->selected)+1];
						strcpy(v_selected_script->vname,dlg->vlw->selected);
//						strlwr(v_selected_script->vname);
						v_selected_script->vlocation=dlg->vlw->scope;
					} /* if */ 

					delete dlg;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 5:
				{
					v_selected_script->command=VALUE_CMD_RANDOM;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=1;
				}
				break;
			case 6:
				{
					v_selected_script->command=VALUE_CMD_ADD;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 7:
				{
					v_selected_script->command=VALUE_CMD_SUB;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 8:
				{
					v_selected_script->command=VALUE_CMD_MUL;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 9:
				{
					v_selected_script->command=VALUE_CMD_DIV;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 10:
				{
					v_selected_script->command=VALUE_CMD_COUNT;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"room");
				}
				break;
			case 11:
				{
					v_selected_script->command=VALUE_CMD_CONCATENATE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"none");
				}
				break;
			case 12:
				{
					v_selected_script->command=VALUE_CMD_REMOVE_SUBSTRING;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[5];
					strcpy(v_selected_script->vpar[1]->spar,"none");
				}
				break;
			case 13:
				{
					v_selected_script->command=VALUE_CMD_MAKE_VARIABLE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[5];
					strcpy(v_selected_script->vpar[0]->spar,"none");
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[1]->spar=new char[7];
					strcpy(v_selected_script->vpar[1]->spar,"object");
				}
				break;
			case 14:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_ASCII;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 15:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_KEY;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 16:
				{
					v_selected_script->command=VALUE_CMD_KEYPRESS_NAME;

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 17:
				{
					v_selected_script->command=VALUE_CMD_SAMPLE;
					v_selected_script->ipar=0;
					v_selected_script->spar=0;
					v_selected_script->sample=0;

					{
						int x,y;
						SAMPLE *s=0;
						SampleSelectorDlg *dlg;

						new_window_coordinates(200,200,&x,&y);
						dlg=new SampleSelectorDlg(x,y,200,200,editor,&s,font);
						dlg->Execute();
						delete dlg;

						if (s!=0) v_selected_script->sample=s;
					}

					delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=0;
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;

			case 18:
				{
					v_selected_script->command=VALUE_CMD_GETNPLAYERS;
				}
				break;
			case 19:
				{
					v_selected_script->command=VALUE_CMD_GETPLAYERNAME;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
				}
				break;
			case 20:
				{
					v_selected_script->command=VALUE_CMD_GETPLAYERTYPE;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
				}
				break;
			case 21:
				{
					v_selected_script->command=VALUE_CMD_NEW_VIEWPORT;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=new ValueSCRIPT();
					if (v_selected_script->vpar[2]!=0) delete v_selected_script->vpar[2];
					v_selected_script->vpar[2]=new ValueSCRIPT();
					if (v_selected_script->vpar[3]!=0) delete v_selected_script->vpar[3];
					v_selected_script->vpar[3]=new ValueSCRIPT();
					if (v_selected_script->vpar[3]!=0) delete v_selected_script->vpar[3];
					v_selected_script->vpar[3]=new ValueSCRIPT();
					v_selected_script->vpar[3]->ipar=100;
					if (v_selected_script->vpar[4]!=0) delete v_selected_script->vpar[3];
					v_selected_script->vpar[4]=new ValueSCRIPT();
					v_selected_script->vpar[4]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[4]->spar=new char[10];
					strcpy(v_selected_script->vpar[4]->spar,"interface");
				}
				break;
			case 22:
				{
					v_selected_script->command=VALUE_CMD_ITOA;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=new ValueSCRIPT();
				}
				break;
			case 23:
				{
					v_selected_script->command=VALUE_CMD_ATOI;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					v_selected_script->vpar[0]->spar=new char[2];
					strcpy(v_selected_script->vpar[0]->spar,"0");
					delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=0;
				}
				break;
			case 24:
				{
					v_selected_script->command=VALUE_CMD_DOT;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
					v_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[0]->ipar=0;
					if (v_selected_script->vpar[1]!=0) delete v_selected_script->vpar[1];
					v_selected_script->vpar[1]=new ValueSCRIPT();
					v_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					v_selected_script->vpar[1]->ipar=0;
				}
				break;
			case 25:
				{
					v_selected_script->command=VALUE_CMD_ABS;
					if (v_selected_script->vpar[0]!=0) delete v_selected_script->vpar[0];
					v_selected_script->vpar[0]=new ValueSCRIPT();
				}
				break;

			case 26:a_selected_script->command=VALUE_CMD_NEWPLAYER;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
 					strcpy(a_selected_script->vpar[0]->spar,"player");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[3]=new ValueSCRIPT();
					a_selected_script->vpar[3]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[3]->spar=new char[4];
 					strcpy(a_selected_script->vpar[3]->spar,"map");
					break;
			} /* switch */ 
			return true;
		} /* if */ 
	} /* if */ 


	if (c_selected_script!=0) {
		int res;
		BVChoiceDlg *dlg;

		/* OBJECT SCRIPTS: */ 
		if (level==0) {
			dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","true\nfalse\nnot\nand\nor\ntimmer\ncollision\n"
															  "collision_with_object\ninside_room\n"
															  "partially_inside_room\nsense_object\n"
															  "sense_specific_object\n"
															  "keypress\nkeypressed\neq\ngr\ngreq\n"
															  "lw\nlweq\nfind_substring\n"
															  "invincible\ninladder\n"
															  "h_collision\nh_collision_with_object\n"
															  "h_inside_room\nh_partially_inside_room\n"
															  "valid_object\n",&res,font);
			dlg->Execute();
			delete dlg;

			switch(res) {
			case 1:c_selected_script->command=CONDITION_CMD_TRUE;
				break;
			case 2:c_selected_script->command=CONDITION_CMD_FALSE;
				break;

			case 3:c_selected_script->aparlist.Delete();
				   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				   c_selected_script->command=CONDITION_CMD_NOT;
				break;
			case 4:if (c_selected_script->command!=CONDITION_CMD_OR &&
					   c_selected_script->command!=CONDITION_CMD_AND) {
					   c_selected_script->aparlist.Delete();
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				   } /* if */ 
				   c_selected_script->command=CONDITION_CMD_AND;
				break;
			case 5:if (c_selected_script->command!=CONDITION_CMD_OR &&
					   c_selected_script->command!=CONDITION_CMD_AND) {
					   c_selected_script->aparlist.Delete();
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				   } /* if */ 
				   c_selected_script->command=CONDITION_CMD_OR;
				break;

			case 6:c_selected_script->command=CONDITION_CMD_TIMMER;
				   delete c_selected_script->vpar[0];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				break;
			case 7:if (c_selected_script->command!=CONDITION_CMD_COLLISION &&
					   c_selected_script->command!=CONDITION_CMD_H_COLLISION) {
					   delete c_selected_script->vpar[0];
					   delete c_selected_script->vpar[1];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_COLLISION;
				break;
			case 8:if (c_selected_script->command!=CONDITION_CMD_COLLISION_WITH_OBJECT &&
					   c_selected_script->command!=CONDITION_CMD_H_COLLISION_WITH_OBJECT) {
					   delete c_selected_script->vpar[0];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					   c_selected_script->vpar[0]->spar=new char[5];
					   strcpy(c_selected_script->vpar[0]->spar,"none");
					   c_selected_script->vpar[1]=new ValueSCRIPT();
					   c_selected_script->vpar[2]=new ValueSCRIPT();
					   c_selected_script->aparlist.Delete();
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_COLLISION_WITH_OBJECT;
				break;
			case 9:if (c_selected_script->command!=CONDITION_CMD_INSIDE_ROOM &&
					   c_selected_script->command!=CONDITION_CMD_H_INSIDE_ROOM) {
					   delete c_selected_script->vpar[0];
					   delete c_selected_script->vpar[1];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[1]=new ValueSCRIPT();
				   } /* if */ 
				   c_selected_script->command=CONDITION_CMD_INSIDE_ROOM;				 
				break;
			case 10:if (c_selected_script->command!=CONDITION_CMD_PARTIALLY_INSIDE_ROOM &&
						c_selected_script->command!=CONDITION_CMD_H_PARTIALLY_INSIDE_ROOM) {
					   delete c_selected_script->vpar[0];
					   delete c_selected_script->vpar[1];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
				   c_selected_script->command=CONDITION_CMD_PARTIALLY_INSIDE_ROOM;
				break;

			case 11:c_selected_script->command=CONDITION_CMD_SENSE_OBJECT;
				   delete c_selected_script->vpar[0];
				   delete c_selected_script->vpar[1];
				   delete c_selected_script->vpar[2];
				   delete c_selected_script->vpar[3];
				   delete c_selected_script->vpar[4];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				   c_selected_script->vpar[0]->command=VALUE_CMD_STRING;
				   c_selected_script->vpar[0]->spar=new char[5];
				   strcpy(c_selected_script->vpar[0]->spar,"none");
				   c_selected_script->vpar[1]=new ValueSCRIPT();
				   c_selected_script->vpar[2]=new ValueSCRIPT();
				   c_selected_script->vpar[3]=new ValueSCRIPT();
				   c_selected_script->vpar[4]=new ValueSCRIPT();
				   c_selected_script->aparlist.Delete();
				   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				break;

			case 12:c_selected_script->command=CONDITION_CMD_SENSE_SPECIFIC_OBJECT;
				   delete c_selected_script->vpar[0];
				   delete c_selected_script->vpar[1];
				   delete c_selected_script->vpar[2];
				   delete c_selected_script->vpar[3];
				   delete c_selected_script->vpar[4];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				   c_selected_script->vpar[0]->command=VALUE_CMD_STRING;
				   c_selected_script->vpar[0]->spar=new char[5];
				   strcpy(c_selected_script->vpar[0]->spar,"none");
				   c_selected_script->vpar[1]=new ValueSCRIPT();
				   c_selected_script->vpar[2]=new ValueSCRIPT();
				   c_selected_script->vpar[3]=new ValueSCRIPT();					   
				   c_selected_script->vpar[4]=new ValueSCRIPT();
				   c_selected_script->aparlist.Delete();
				   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				break;

			case 13:if (c_selected_script->command!=CONDITION_CMD_KEYPRESS &&
					   c_selected_script->command!=CONDITION_CMD_KEYPRESSED) {
					   delete c_selected_script->vpar[0];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[0]->ipar=0;
				   } /* if */ 
				   c_selected_script->command=CONDITION_CMD_KEYPRESS;			   
				break;
			case 14:if (c_selected_script->command!=CONDITION_CMD_KEYPRESS &&
						c_selected_script->command!=CONDITION_CMD_KEYPRESSED) {
					   delete c_selected_script->vpar[0];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[0]->ipar=0;
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_KEYPRESSED;
				break;

			case 15:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_EQ;
					
				break;
			case 16:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_GR;
					
				break;
			case 17:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_GREQ;
					
				break;
			case 18:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_LW;
				break;

			case 19:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_LWEQ;
				break;

			case 20:c_selected_script->command=CONDITION_CMD_FIND_SUBSTRING;
				   delete c_selected_script->vpar[0];
				   delete c_selected_script->vpar[1];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				   c_selected_script->vpar[1]=new ValueSCRIPT();
				break;

			case 21:c_selected_script->command=CONDITION_CMD_INVINCIBLE;
				break;

			case 22:c_selected_script->command=CONDITION_CMD_INLADDER;
				break;

			case 23:if (c_selected_script->command!=CONDITION_CMD_COLLISION &&
						c_selected_script->command!=CONDITION_CMD_H_COLLISION) {
					   delete c_selected_script->vpar[0];
					   delete c_selected_script->vpar[1];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_H_COLLISION;				
				break;
			case 24:if (c_selected_script->command!=CONDITION_CMD_COLLISION_WITH_OBJECT &&
						c_selected_script->command!=CONDITION_CMD_H_COLLISION_WITH_OBJECT) {
					   delete c_selected_script->vpar[0];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					   c_selected_script->vpar[0]->spar=new char[5];
					   strcpy(c_selected_script->vpar[0]->spar,"none");
					   c_selected_script->vpar[1]=new ValueSCRIPT();
					   c_selected_script->vpar[2]=new ValueSCRIPT();
					   c_selected_script->aparlist.Delete();
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
					} /* if */ 
				   c_selected_script->command=CONDITION_CMD_H_COLLISION_WITH_OBJECT;				   
				break;
			case 25:if (c_selected_script->command!=CONDITION_CMD_INSIDE_ROOM &&
						c_selected_script->command!=CONDITION_CMD_H_INSIDE_ROOM) {
					   delete c_selected_script->vpar[0];
					   delete c_selected_script->vpar[1];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_H_INSIDE_ROOM;
				break;
			case 26:if (c_selected_script->command!=CONDITION_CMD_PARTIALLY_INSIDE_ROOM &&
						c_selected_script->command!=CONDITION_CMD_H_PARTIALLY_INSIDE_ROOM) {
					   delete c_selected_script->vpar[0];
					   delete c_selected_script->vpar[1];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
				   c_selected_script->command=CONDITION_CMD_H_PARTIALLY_INSIDE_ROOM;
				break;
			case 27:delete c_selected_script->vpar[0];
					c_selected_script->vpar[0]=new ValueSCRIPT();
					c_selected_script->command=CONDITION_CMD_VALID_OBJECT;
					break;

			} /* switch */ 
			return true;
		} /* if */ 


		/* ROOM/INTERFACE SCRIPTS: */ 
		if (level==1 || level==4) {
			if (level==4) {
				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","true\nfalse\nnot\nand\nor\ntimmer\n"
																  "sense_object\n"
																  "sense_specific_object\n"
																  "keypress\nkeypressed\neq\ngr\ngreq\n"
																  "lw\nlweq\nfind_substring\n"
																  "valid_object\n"
																  "loadgame\nsavegame\n",&res,font);
			} else {
				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","true\nfalse\nnot\nand\nor\ntimmer\n"
																  "sense_object\n"
																  "sense_specific_object\n"
																  "keypress\nkeypressed\neq\ngr\ngreq\n"
																  "lw\nlweq\nfind_substring\n"
																  "valid_object\n",&res,font);
			} /* if */ 
			dlg->Execute();
			delete dlg;

			switch(res) {
			case 1:c_selected_script->command=CONDITION_CMD_TRUE;
				break;
			case 2:c_selected_script->command=CONDITION_CMD_FALSE;
				break;

			case 3:c_selected_script->aparlist.Delete();
				   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				   c_selected_script->command=CONDITION_CMD_NOT;
				break;
			case 4:if (c_selected_script->command!=CONDITION_CMD_OR &&
					   c_selected_script->command!=CONDITION_CMD_AND) {
					   c_selected_script->aparlist.Delete();
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				   } /* if */ 
				   c_selected_script->command=CONDITION_CMD_AND;
				break;
			case 5:if (c_selected_script->command!=CONDITION_CMD_OR &&
					   c_selected_script->command!=CONDITION_CMD_AND) {
					   c_selected_script->aparlist.Delete();
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				   } /* if */ 
				   c_selected_script->command=CONDITION_CMD_OR;
				break;

			case 6:c_selected_script->command=CONDITION_CMD_TIMMER;
				   delete c_selected_script->vpar[0];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				break;

			case 7:c_selected_script->command=CONDITION_CMD_SENSE_OBJECT;
				   delete c_selected_script->vpar[0];
				   delete c_selected_script->vpar[1];
				   delete c_selected_script->vpar[2];
				   delete c_selected_script->vpar[3];
				   delete c_selected_script->vpar[4];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				   c_selected_script->vpar[0]->command=VALUE_CMD_STRING;
				   c_selected_script->vpar[0]->spar=new char[5];
				   strcpy(c_selected_script->vpar[0]->spar,"none");
				   c_selected_script->vpar[1]=new ValueSCRIPT();
				   c_selected_script->vpar[2]=new ValueSCRIPT();
				   c_selected_script->vpar[3]=new ValueSCRIPT();
				   c_selected_script->vpar[4]=new ValueSCRIPT();
				   c_selected_script->aparlist.Delete();
				   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				break;

			case 8:c_selected_script->command=CONDITION_CMD_SENSE_SPECIFIC_OBJECT;
				   delete c_selected_script->vpar[0];
				   delete c_selected_script->vpar[1];
				   delete c_selected_script->vpar[2];
				   delete c_selected_script->vpar[3];
				   delete c_selected_script->vpar[4];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				   c_selected_script->vpar[0]->command=VALUE_CMD_STRING;
				   c_selected_script->vpar[0]->spar=new char[5];
				   strcpy(c_selected_script->vpar[0]->spar,"none");
				   c_selected_script->vpar[1]=new ValueSCRIPT();
				   c_selected_script->vpar[2]=new ValueSCRIPT();
				   c_selected_script->vpar[3]=new ValueSCRIPT();
				   c_selected_script->vpar[4]=new ValueSCRIPT();
				   c_selected_script->aparlist.Delete();
				   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				break;

			case 9:if (c_selected_script->command!=CONDITION_CMD_KEYPRESS &&
					   c_selected_script->command!=CONDITION_CMD_KEYPRESSED) {
					   delete c_selected_script->vpar[0];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[0]->ipar=0;
				   } /* if */ 
				   c_selected_script->command=CONDITION_CMD_KEYPRESS;			   
				break;
			case 10:if (c_selected_script->command!=CONDITION_CMD_KEYPRESS &&
						c_selected_script->command!=CONDITION_CMD_KEYPRESSED) {
					   delete c_selected_script->vpar[0];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[0]->ipar=0;
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_KEYPRESSED;
				break;

			case 11:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_EQ;
					
				break;
			case 12:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_GR;
					
				break;
			case 13:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_GREQ;
					
				break;
			case 14:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_LW;
				break;

			case 15:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_LWEQ;
				break;

			case 16:c_selected_script->command=CONDITION_CMD_FIND_SUBSTRING;
				   delete c_selected_script->vpar[0];
				   delete c_selected_script->vpar[1];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				   c_selected_script->vpar[1]=new ValueSCRIPT();
				break;

			case 17:delete c_selected_script->vpar[0];
					c_selected_script->vpar[0]=new ValueSCRIPT();
					c_selected_script->command=CONDITION_CMD_VALID_OBJECT;
					break;

			case 18:c_selected_script->command=CONDITION_CMD_LOADGAME;
				   delete c_selected_script->vpar[0];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				   c_selected_script->vpar[0]->command=VALUE_CMD_STRING;
				   c_selected_script->vpar[0]->spar=new char[13];
				   strcpy(c_selected_script->vpar[0]->spar,"savegame.sav");
				break;
			case 19:c_selected_script->command=CONDITION_CMD_SAVEGAME;
				   delete c_selected_script->vpar[0];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				   c_selected_script->vpar[0]->command=VALUE_CMD_STRING;
				   c_selected_script->vpar[0]->spar=new char[13];
				   strcpy(c_selected_script->vpar[0]->spar,"savegame.sav");
				break;
			} /* switch */ 
			return true;
		} /* if */ 


		/* MAP/GAME/APPLICATION SCRIPTS: */ 
		if (level==2 || level==3 || level==5) {
			if (level==5) {
				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","true\nfalse\nnot\nand\nor\ntimmer\n"
																  "keypress\nkeypressed\neq\ngr\ngreq\n"
																  "lw\nlweq\nfind_substring\n"
																  "valid_object\n"
																  "loadgame\nsavegame\n",&res,font);
			} else {
				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","true\nfalse\nnot\nand\nor\ntimmer\n"
																  "keypress\nkeypressed\neq\ngr\ngreq\n"
																  "lw\nlweq\nfind_substring\n"
																  "valid_object\n",&res,font);
			} /* if */ 
			dlg->Execute();
			delete dlg;

			switch(res) {
			case 1:c_selected_script->command=CONDITION_CMD_TRUE;
				break;
			case 2:c_selected_script->command=CONDITION_CMD_FALSE;
				break;

			case 3:c_selected_script->aparlist.Delete();
				   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				   c_selected_script->command=CONDITION_CMD_NOT;
				break;
			case 4:if (c_selected_script->command!=CONDITION_CMD_OR &&
					   c_selected_script->command!=CONDITION_CMD_AND) {
					   c_selected_script->aparlist.Delete();
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				   } /* if */ 
				   c_selected_script->command=CONDITION_CMD_AND;
				break;
			case 5:if (c_selected_script->command!=CONDITION_CMD_OR &&
					   c_selected_script->command!=CONDITION_CMD_AND) {
					   c_selected_script->aparlist.Delete();
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
					   c_selected_script->aparlist.Add(new ConditionSCRIPT());
				   } /* if */ 
				   c_selected_script->command=CONDITION_CMD_OR;
				break;

			case 6:c_selected_script->command=CONDITION_CMD_TIMMER;
				   delete c_selected_script->vpar[0];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				break;

			case 7:if (c_selected_script->command!=CONDITION_CMD_KEYPRESS &&
					   c_selected_script->command!=CONDITION_CMD_KEYPRESSED) {
					   delete c_selected_script->vpar[0];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[0]->ipar=0;
				   } /* if */ 
				   c_selected_script->command=CONDITION_CMD_KEYPRESS;			   
				break;
			case 8:if (c_selected_script->command!=CONDITION_CMD_KEYPRESS &&
						c_selected_script->command!=CONDITION_CMD_KEYPRESSED) {
					   delete c_selected_script->vpar[0];
					   c_selected_script->vpar[0]=new ValueSCRIPT();
					   c_selected_script->vpar[0]->ipar=0;
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_KEYPRESSED;
				break;

			case 9:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_EQ;
					
				break;
			case 10:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_GR;
					
				break;
			case 11:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_GREQ;
					
				break;
			case 12:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_LW;
				break;

			case 13:if (c_selected_script->command!=CONDITION_CMD_VARIABLE_EQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GR &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_GREQ &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LW &&
						c_selected_script->command!=CONDITION_CMD_VARIABLE_LWEQ) {
						c_selected_script->vpar[0]=new ValueSCRIPT();
						c_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */ 
					c_selected_script->command=CONDITION_CMD_VARIABLE_LWEQ;
				break;

			case 14:c_selected_script->command=CONDITION_CMD_FIND_SUBSTRING;
				   delete c_selected_script->vpar[0];
				   delete c_selected_script->vpar[1];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				   c_selected_script->vpar[1]=new ValueSCRIPT();
				break;

			case 15:delete c_selected_script->vpar[0];
					c_selected_script->vpar[0]=new ValueSCRIPT();
					c_selected_script->command=CONDITION_CMD_VALID_OBJECT;
					break;

			case 16:c_selected_script->command=CONDITION_CMD_LOADGAME;
				   delete c_selected_script->vpar[0];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				   c_selected_script->vpar[0]->command=VALUE_CMD_STRING;
				   c_selected_script->vpar[0]->spar=new char[13];
				   strcpy(c_selected_script->vpar[0]->spar,"savegame.sav");
				break;
			case 17:c_selected_script->command=CONDITION_CMD_SAVEGAME;
				   delete c_selected_script->vpar[0];
				   c_selected_script->vpar[0]=new ValueSCRIPT();
				   c_selected_script->vpar[0]->command=VALUE_CMD_STRING;
				   c_selected_script->vpar[0]->spar=new char[13];
				   strcpy(c_selected_script->vpar[0]->spar,"savegame.sav");
				break;

			} /* switch */ 
			return true;		
		} /* if */ 
	} /* if */ 

	if (a_selected_script!=0) {
		int res;
		BVChoiceDlg *dlg;

		/* OBJECT SCRIPTS: */ 
		if (level==0) {
			if (object->control!=CTRL_GAMEWINDOW &&
				object->control!=CTRL_MENU) {
				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","none\nseq\nparallel\nif\nwhile\nwhile_else\ndo\npause\n"
																"atomic\nterminate\nterminatechild\nstate\n"
																"set\nmove\ncreateobject\ncreatechild\ncreategluedchild\n"
																"dm_mode\ndm_blink\ndm_color1\ndm_color2\nset_invincible\n"
																"leaveladder\n"
																"newvariable\nkillvariable\n"
																"play_sample\nplay_sample_pan\nget_child\n"
																"get_parent\nget_root\n"
																"drawpixel\ndrawline\ndrawbox\ndrawbitmap\n"
																"changemap\ngetplayer\n"
																"setplayername\n"
																"case\nrandom_case\n",&res,font);
			} else {
				if (object->control==CTRL_GAMEWINDOW) 
					dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","none\nseq\nparallel\nif\nwhile\nwhile_else\ndo\npause\n"
																	"atomic\nterminate\nterminatechild\nstate\n"
																	"set\nmove\ncreateobject\ncreatechild\ncreategluedchild\n"
																	"dm_mode\ndm_blink\ndm_color1\ndm_color2\nset_invincible\n"
																	"leaveladder\n"
																	"newvariable\nkillvariable\n"
																	"play_sample\nplay_sample_pan\nget_child\n"
																	"get_parent\nget_root\n"
																	"drawpixel\ndrawline\ndrawbox\ndrawbitmap\n"
																	"changemap\ngetplayer\n"
																	"setplayername\n"
																	"case\nrandom_case\n"
																	"add_focusing\nsub_focusing\nclear_focusing\n",&res,font);
				else
					dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","none\nseq\nparallel\nif\nwhile\nwhile_else\ndo\npause\n"
																	"atomic\nterminate\nterminatechild\nstate\n"
																	"set\nmove\ncreateobject\ncreatechild\ncreategluedchild\n"
																	"dm_mode\ndm_blink\ndm_color1\ndm_color2\nset_invincible\n"
																	"leaveladder\n"
																	"newvariable\nkillvariable\n"
																	"play_sample\nplay_sample_pan\nget_child\n"
																	"get_parent\nget_root\n"
																	"drawpixel\ndrawline\ndrawbox\ndrawbitmap\n"
																	"changemap\ngetplayer\n"
																	"setplayername\n"
																	"case\nrandom_case\n"
																	"menu_addoption\nmenu_clearoptions\nmenu_substoption\n",&res,font);

			} /* if */ 
			dlg->Execute();
			delete dlg;

			switch(res) {
			case 1:a_selected_script->command=ACTION_CMD_NONE;
				break;
			case 2:if (a_selected_script->command!=ACTION_CMD_SEQ &&
					   a_selected_script->command!=ACTION_CMD_PARALLEL) {
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_SEQ;
				break;
			case 3:if (a_selected_script->command!=ACTION_CMD_SEQ &&
					   a_selected_script->command!=ACTION_CMD_PARALLEL) {
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_PARALLEL;
				break;
			case 4:a_selected_script->command=ACTION_CMD_IF;
				   a_selected_script->cond->command=CONDITION_CMD_TRUE;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				break;
			case 5:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_WHILE;			   
				break;
			case 6:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } else {
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 		   
				   a_selected_script->command=ACTION_CMD_WHILE_ELSE;
				break;
			case 7:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_DO;			   
				break;
			case 8:a_selected_script->command=ACTION_CMD_PAUSE;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				break;
			case 9:a_selected_script->command=ACTION_CMD_ATOMIC;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				break;
			case 10:a_selected_script->command=ACTION_CMD_TERMINATE;
				break;
			case 11:a_selected_script->command=ACTION_CMD_TERMINATECHILD;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
   				   a_selected_script->cond->command=CONDITION_CMD_TRUE;
				   break;

			case 12:a_selected_script->command=ACTION_CMD_STATE;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				break;
			case 13:a_selected_script->command=ACTION_CMD_SET;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->vpar[1]=new ValueSCRIPT();
				break;
			case 14:a_selected_script->command=ACTION_CMD_MOVE;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->vpar[1]=new ValueSCRIPT();
				break;
			case 15:if (a_selected_script->command!=ACTION_CMD_CREATEOBJECT &&
						a_selected_script->command!=ACTION_CMD_CREATECHILD &&
						a_selected_script->command!=ACTION_CMD_CREATEGLUEDCHILD) {
						if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
 						if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
 						if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
						a_selected_script->vpar[0]=new ValueSCRIPT();
						a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
						a_selected_script->vpar[0]->spar=new char[5];
						strcpy(a_selected_script->vpar[0]->spar,"none");
						a_selected_script->vpar[1]=new ValueSCRIPT();
						a_selected_script->vpar[2]=new ValueSCRIPT();
					} /* if */ 
 					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					a_selected_script->vpar[3]=new ValueSCRIPT();
					a_selected_script->command=ACTION_CMD_CREATEOBJECT;	
				break;
			case 16:if (a_selected_script->command!=ACTION_CMD_CREATEOBJECT &&
						a_selected_script->command!=ACTION_CMD_CREATECHILD &&
						a_selected_script->command!=ACTION_CMD_CREATEGLUEDCHILD) {
						if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
 						if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
 						if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
						a_selected_script->vpar[0]=new ValueSCRIPT();
						a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
						a_selected_script->vpar[0]->spar=new char[5];
						strcpy(a_selected_script->vpar[0]->spar,"none");
						a_selected_script->vpar[1]=new ValueSCRIPT();
						a_selected_script->vpar[2]=new ValueSCRIPT();
					} /* if */ 
					if (a_selected_script->command!=ACTION_CMD_CREATECHILD &&
						a_selected_script->command!=ACTION_CMD_CREATEGLUEDCHILD) {
						if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
						if (a_selected_script->vpar[4]!=0) delete a_selected_script->vpar[4];
						if (a_selected_script->vpar[5]!=0) delete a_selected_script->vpar[5];
						a_selected_script->vpar[3]=new ValueSCRIPT();
						a_selected_script->vpar[4]=new ValueSCRIPT();
						a_selected_script->vpar[5]=new ValueSCRIPT();
					} /* if */ 
					a_selected_script->command=ACTION_CMD_CREATECHILD;			
				break;
			case 17:if (a_selected_script->command!=ACTION_CMD_CREATEOBJECT &&
						a_selected_script->command!=ACTION_CMD_CREATECHILD &&
						a_selected_script->command!=ACTION_CMD_CREATEGLUEDCHILD) {
						if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
 						if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
 						if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
						a_selected_script->vpar[0]=new ValueSCRIPT();
						a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
						a_selected_script->vpar[0]->spar=new char[5];
						strcpy(a_selected_script->vpar[0]->spar,"none");
						a_selected_script->vpar[1]=new ValueSCRIPT();
						a_selected_script->vpar[2]=new ValueSCRIPT();
					} /* if */ 
					if (a_selected_script->command!=ACTION_CMD_CREATECHILD &&
						a_selected_script->command!=ACTION_CMD_CREATEGLUEDCHILD) {
						if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
						if (a_selected_script->vpar[4]!=0) delete a_selected_script->vpar[4];
						if (a_selected_script->vpar[5]!=0) delete a_selected_script->vpar[5];
						a_selected_script->vpar[3]=new ValueSCRIPT();
						a_selected_script->vpar[4]=new ValueSCRIPT();
						a_selected_script->vpar[5]=new ValueSCRIPT();
					} /* if */ 
					a_selected_script->command=ACTION_CMD_CREATEGLUEDCHILD;
				break;
			case 18:a_selected_script->command=ACTION_CMD_DM_MODE_FACTOR;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"normal");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->ipar=100;
					break;
			case 19:if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->ipar=0;
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->ipar=100;
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[2]->ipar=1;
					a_selected_script->command=ACTION_CMD_DM_BLINK;
					break;
			case 20:if (a_selected_script->command!=ACTION_CMD_DM_COLOR1 &&
						a_selected_script->command!=ACTION_CMD_DM_COLOR2) {
						if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
						if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
						if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
						if (a_selected_script->vpar[4]!=0) delete a_selected_script->vpar[4];
						a_selected_script->vpar[0]=new ValueSCRIPT();
						a_selected_script->vpar[0]->ipar=255;
						a_selected_script->vpar[1]=new ValueSCRIPT();
						a_selected_script->vpar[1]->ipar=255;
						a_selected_script->vpar[2]=new ValueSCRIPT();
						a_selected_script->vpar[2]->ipar=255;
						a_selected_script->vpar[3]=new ValueSCRIPT();
						a_selected_script->vpar[3]->ipar=-1;
					} /* if */ 
					a_selected_script->command=ACTION_CMD_DM_COLOR1;							
					break;
			case 21:if (a_selected_script->command!=ACTION_CMD_DM_COLOR1 &&
						a_selected_script->command!=ACTION_CMD_DM_COLOR2) {
						if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
						if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
						if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
						if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
						a_selected_script->vpar[0]=new ValueSCRIPT();
						a_selected_script->vpar[0]->ipar=0;
						a_selected_script->vpar[1]=new ValueSCRIPT();
						a_selected_script->vpar[1]->ipar=0;
						a_selected_script->vpar[2]=new ValueSCRIPT();
						a_selected_script->vpar[2]->ipar=0;
						a_selected_script->vpar[3]=new ValueSCRIPT();
						a_selected_script->vpar[3]->ipar=-1;
					} /* if */ 
					a_selected_script->command=ACTION_CMD_DM_COLOR2;						
					break;
			case 22:a_selected_script->command=ACTION_CMD_SET_INVINCIBLE;	
					a_selected_script->aparlist.Delete();
					a_selected_script->aparlist.Add(new ActionSCRIPT());
					break;
			case 23:a_selected_script->command=ACTION_CMD_LEAVELADDER;
					break;

			case 24:a_selected_script->command=ACTION_CMD_NEWVARIABLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					break;
			case 25:a_selected_script->command=ACTION_CMD_KILLVARIABLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					break;
			case 26:a_selected_script->command=ACTION_CMD_PLAY_SAMPLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					a_selected_script->vpar[1]->ipar=100;
					break;
			case 27:a_selected_script->command=ACTION_CMD_PLAY_SAMPLE_PAN;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					a_selected_script->vpar[1]->ipar=100;
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[2]->command=VALUE_CMD_INTEGER;
					break;
			case 28:a_selected_script->command=ACTION_CMD_GETCHILD;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->cond->command=CONDITION_CMD_TRUE;
				   break;
			case 29:a_selected_script->command=ACTION_CMD_GETPARENT;
				   break;
			case 30:a_selected_script->command=ACTION_CMD_GETROOT;
				   break;

			case 31:a_selected_script->command=ACTION_CMD_DRAWPIXEL;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					if (a_selected_script->vpar[4]!=0) delete a_selected_script->vpar[4];
					if (a_selected_script->vpar[5]!=0) delete a_selected_script->vpar[5];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[3]=new ValueSCRIPT();
					a_selected_script->vpar[4]=new ValueSCRIPT();
					a_selected_script->vpar[5]=new ValueSCRIPT();
					break;
			case 32:a_selected_script->command=ACTION_CMD_DRAWLINE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					if (a_selected_script->vpar[4]!=0) delete a_selected_script->vpar[4];
					if (a_selected_script->vpar[5]!=0) delete a_selected_script->vpar[5];
					if (a_selected_script->vpar[6]!=0) delete a_selected_script->vpar[6];
					if (a_selected_script->vpar[7]!=0) delete a_selected_script->vpar[7];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[3]=new ValueSCRIPT();
					a_selected_script->vpar[4]=new ValueSCRIPT();
					a_selected_script->vpar[5]=new ValueSCRIPT();
					a_selected_script->vpar[6]=new ValueSCRIPT();
					a_selected_script->vpar[7]=new ValueSCRIPT();
					break;
			case 33:a_selected_script->command=ACTION_CMD_DRAWBOX;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					if (a_selected_script->vpar[4]!=0) delete a_selected_script->vpar[4];
					if (a_selected_script->vpar[5]!=0) delete a_selected_script->vpar[5];
					if (a_selected_script->vpar[6]!=0) delete a_selected_script->vpar[6];
					if (a_selected_script->vpar[7]!=0) delete a_selected_script->vpar[7];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[3]=new ValueSCRIPT();
					a_selected_script->vpar[4]=new ValueSCRIPT();
					a_selected_script->vpar[5]=new ValueSCRIPT();
					a_selected_script->vpar[6]=new ValueSCRIPT();
					a_selected_script->vpar[7]=new ValueSCRIPT();
					break;
			case 34:a_selected_script->command=ACTION_CMD_DRAWBITMAP;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[3]=new ValueSCRIPT();
					break;

			case 35:a_selected_script->command=ACTION_CMD_CHANGEMAP;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
				   a_selected_script->vpar[0]->spar=new char[8];
				   strcpy(a_selected_script->vpar[0]->spar,"mapname");
				   if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[1]=new ValueSCRIPT();
				   if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[2]=new ValueSCRIPT();
				   break;

			case 36:a_selected_script->command=ACTION_CMD_GETPLAYER;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   break;

			case 37:a_selected_script->command=ACTION_CMD_SETPLAYERNAME;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[1]=new ValueSCRIPT();
				   a_selected_script->vpar[1]->command=VALUE_CMD_STRING;
				   a_selected_script->vpar[1]->spar=new char[8];
				   strcpy(a_selected_script->vpar[1]->spar,"none");
				   break;

			case 38:a_selected_script->command=ACTION_CMD_CASE;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->vparlist.Delete();
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist[1]->ipar=1;

				   break;

			case 39:a_selected_script->command=ACTION_CMD_RANDOM_CASE;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->vparlist.Delete();
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist[0]->ipar=1;
				   a_selected_script->vparlist[1]->ipar=1;
				   break;

			case 40:if (object->control==CTRL_GAMEWINDOW) {
						a_selected_script->command=ACTION_CMD_ADD_FOCUSING;
						if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
						a_selected_script->vpar[0]=new ValueSCRIPT();
					} else {
						a_selected_script->command=ACTION_CMD_MENU_ADDOPTION;
						if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
						a_selected_script->vpar[0]=new ValueSCRIPT();
					} /* if */ 
				    break;

			case 41:if (object->control==CTRL_GAMEWINDOW) {
						a_selected_script->command=ACTION_CMD_SUB_FOCUSING;
						if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
						a_selected_script->vpar[0]=new ValueSCRIPT();
					} else {
						a_selected_script->command=ACTION_CMD_MENU_CLEAROPTIONS;
					} /* if */ 
				   break;

			case 42:if (object->control==CTRL_GAMEWINDOW) {
						a_selected_script->command=ACTION_CMD_CLEAR_FOCUSING;
					} else {
						a_selected_script->command=ACTION_CMD_MENU_SUBSTOPTION;
						if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
						if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
						a_selected_script->vpar[0]=new ValueSCRIPT();
						a_selected_script->vpar[1]=new ValueSCRIPT();
					} /* if */  
				    break;

			} /* switch */ 
			return true;	
		} /* if */ 




		/* ROOM/INTERFACE SCRIPTS: */ 
		if (level==1 || level==4) {
			if (interf==0) {
				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","none\nseq\nparallel\nif\nwhile\nwhile_else\ndo\npause\n"
																  "atomic\n"
																  "set\ncreateobject\n"
																  "newvariable\nkillvariable\n"
																  "play_sample\nplay_sample_pan\n"
																  "play_music\nstop_music\n"
																  "drawpixel\ndrawline\ndrawbox\ndrawbitmap\n"
																  "changemap\n"
																  "case\nrandom_case\n",&res,font);
			} else {
				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","none\nseq\nparallel\nif\nwhile\nwhile_else\ndo\npause\n"
																  "atomic\n"
																  "set\ncreateobject\n"
																  "newvariable\nkillvariable\n"
																  "play_sample\nplay_sample_pan\n"
																  "play_music\nstop_music\n"
																  "drawpixel\ndrawline\ndrawbox\ndrawbitmap\n"
																  "changemap\n"
																  "case\nrandom_case\n"
																  "getplayer\nsetplayername\n"
																  "setplayerkeyboard\ngamemode\n"
																  "reset_game\nstop_game\nresume_game\n",&res,font);
			} /* if */ 

			dlg->Execute();
			delete dlg;

			switch(res) {
			case 1:a_selected_script->command=ACTION_CMD_NONE;
				break;
			case 2:if (a_selected_script->command!=ACTION_CMD_SEQ &&
					   a_selected_script->command!=ACTION_CMD_PARALLEL) {
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_SEQ;
				break;
			case 3:if (a_selected_script->command!=ACTION_CMD_SEQ &&
					   a_selected_script->command!=ACTION_CMD_PARALLEL) {
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_PARALLEL;
				break;
			case 4:a_selected_script->command=ACTION_CMD_IF;
				   a_selected_script->cond->command=CONDITION_CMD_TRUE;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				break;
			case 5:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_WHILE;			   
				break;
			case 6:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } else {
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 		   
				   a_selected_script->command=ACTION_CMD_WHILE_ELSE;
				break;
			case 7:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_DO;			   
				break;
			case 8:a_selected_script->command=ACTION_CMD_PAUSE;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				break;
			case 9:a_selected_script->command=ACTION_CMD_ATOMIC;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				break;

			case 10:a_selected_script->command=ACTION_CMD_SET;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->vpar[1]=new ValueSCRIPT();
				break;
			case 11:if (a_selected_script->command!=ACTION_CMD_CREATEOBJECT &&
						a_selected_script->command!=ACTION_CMD_CREATECHILD &&
						a_selected_script->command!=ACTION_CMD_CREATEGLUEDCHILD) {
						if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
 						if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
 						if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
						a_selected_script->vpar[0]=new ValueSCRIPT();
						a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
						a_selected_script->vpar[0]->spar=new char[5];
						strcpy(a_selected_script->vpar[0]->spar,"none");
						a_selected_script->vpar[1]=new ValueSCRIPT();
						a_selected_script->vpar[2]=new ValueSCRIPT();
					} /* if */ 
 					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					a_selected_script->vpar[3]=new ValueSCRIPT();
					a_selected_script->command=ACTION_CMD_CREATEOBJECT;	
				break;
			case 12:a_selected_script->command=ACTION_CMD_NEWVARIABLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					break;
			case 13:a_selected_script->command=ACTION_CMD_KILLVARIABLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					break;
			case 14:a_selected_script->command=ACTION_CMD_PLAY_SAMPLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					a_selected_script->vpar[1]->ipar=100;
					break;
			case 15:a_selected_script->command=ACTION_CMD_PLAY_SAMPLE_PAN;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					a_selected_script->vpar[1]->ipar=100;
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[2]->command=VALUE_CMD_INTEGER;
					break;
			case 16:a_selected_script->command=ACTION_CMD_PLAY_MUSIC;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					a_selected_script->vpar[1]->ipar=100;
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[2]->command=VALUE_CMD_INTEGER;
					break;
			case 17:a_selected_script->command=ACTION_CMD_STOP_MUSIC;
					break;

			case 18:a_selected_script->command=ACTION_CMD_DRAWPIXEL;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					if (a_selected_script->vpar[4]!=0) delete a_selected_script->vpar[4];
					if (a_selected_script->vpar[5]!=0) delete a_selected_script->vpar[5];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[3]=new ValueSCRIPT();
					a_selected_script->vpar[4]=new ValueSCRIPT();
					a_selected_script->vpar[5]=new ValueSCRIPT();
					break;
			case 19:a_selected_script->command=ACTION_CMD_DRAWLINE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					if (a_selected_script->vpar[4]!=0) delete a_selected_script->vpar[4];
					if (a_selected_script->vpar[5]!=0) delete a_selected_script->vpar[5];
					if (a_selected_script->vpar[6]!=0) delete a_selected_script->vpar[6];
					if (a_selected_script->vpar[7]!=0) delete a_selected_script->vpar[7];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[3]=new ValueSCRIPT();
					a_selected_script->vpar[4]=new ValueSCRIPT();
					a_selected_script->vpar[5]=new ValueSCRIPT();
					a_selected_script->vpar[6]=new ValueSCRIPT();
					a_selected_script->vpar[7]=new ValueSCRIPT();
					break;
			case 20:a_selected_script->command=ACTION_CMD_DRAWBOX;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					if (a_selected_script->vpar[4]!=0) delete a_selected_script->vpar[4];
					if (a_selected_script->vpar[5]!=0) delete a_selected_script->vpar[5];
					if (a_selected_script->vpar[6]!=0) delete a_selected_script->vpar[6];
					if (a_selected_script->vpar[7]!=0) delete a_selected_script->vpar[7];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[3]=new ValueSCRIPT();
					a_selected_script->vpar[4]=new ValueSCRIPT();
					a_selected_script->vpar[5]=new ValueSCRIPT();
					a_selected_script->vpar[6]=new ValueSCRIPT();
					a_selected_script->vpar[7]=new ValueSCRIPT();
					break;
			case 21:a_selected_script->command=ACTION_CMD_DRAWBITMAP;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[3]=new ValueSCRIPT();
					break;

			case 22:a_selected_script->command=ACTION_CMD_CHANGEMAP;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
				   a_selected_script->vpar[0]->spar=new char[8];
				   strcpy(a_selected_script->vpar[0]->spar,"mapname");
				   if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[1]=new ValueSCRIPT();
				   if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[2]=new ValueSCRIPT();
				   break;

			case 23:a_selected_script->command=ACTION_CMD_CASE;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->vparlist.Delete();
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist[1]->ipar=1;

				   break;

			case 24:a_selected_script->command=ACTION_CMD_RANDOM_CASE;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->vparlist.Delete();
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist[0]->ipar=1;
				   a_selected_script->vparlist[1]->ipar=1;
				   break;

			case 25:a_selected_script->command=ACTION_CMD_GETPLAYER;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   break;

			case 26:a_selected_script->command=ACTION_CMD_SETPLAYERNAME;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[1]=new ValueSCRIPT();
				   a_selected_script->vpar[1]->command=VALUE_CMD_STRING;
				   a_selected_script->vpar[1]->spar=new char[8];
				   strcpy(a_selected_script->vpar[1]->spar,"none");
				   break;

			case 27:a_selected_script->command=ACTION_CMD_SETPLAYERKEYBOARD;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					if (a_selected_script->vpar[4]!=0) delete a_selected_script->vpar[4];
					if (a_selected_script->vpar[5]!=0) delete a_selected_script->vpar[5];
					if (a_selected_script->vpar[6]!=0) delete a_selected_script->vpar[6];
					if (a_selected_script->vpar[7]!=0) delete a_selected_script->vpar[7];
					if (a_selected_script->vpar[8]!=0) delete a_selected_script->vpar[8];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[3]=new ValueSCRIPT();
					a_selected_script->vpar[4]=new ValueSCRIPT();
					a_selected_script->vpar[5]=new ValueSCRIPT();
					a_selected_script->vpar[6]=new ValueSCRIPT();
					a_selected_script->vpar[7]=new ValueSCRIPT();
					a_selected_script->vpar[8]=new ValueSCRIPT();
					break;

			case 28:a_selected_script->command=ACTION_CMD_GAMEMODE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[1]->spar=new char[12];
 					strcpy(a_selected_script->vpar[1]->spar,"default.rpl");
					break;

			case 29:a_selected_script->command=ACTION_CMD_RESETGAME;								
					break;
			case 30:a_selected_script->command=ACTION_CMD_STOPGAME;								
					break;
			case 31:a_selected_script->command=ACTION_CMD_RESUMEGAME;								
					break;

			} /* switch */ 
			return true;	
		} /* if */ 

		/* MAP/GAME SCRIPTS: */ 
		if (level==2 || level==3) {
			dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","none\nseq\nparallel\nif\nwhile\nwhile_else\ndo\npause\n"
															  "atomic\n"
															  "set\n"
															  "newvariable\nkillvariable\n"
															  "play_sample\nplay_sample_pan\n"
															  "play_music\nstop_music\n"
															  "case\nrandom_case\n",&res,font);
			dlg->Execute();
			delete dlg;

			switch(res) {
			case 1:a_selected_script->command=ACTION_CMD_NONE;
				break;
			case 2:if (a_selected_script->command!=ACTION_CMD_SEQ &&
					   a_selected_script->command!=ACTION_CMD_PARALLEL) {
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_SEQ;
				break;
			case 3:if (a_selected_script->command!=ACTION_CMD_SEQ &&
					   a_selected_script->command!=ACTION_CMD_PARALLEL) {
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_PARALLEL;
				break;
			case 4:a_selected_script->command=ACTION_CMD_IF;
				   a_selected_script->cond->command=CONDITION_CMD_TRUE;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				break;
			case 5:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_WHILE;
				break;
			case 6:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } else {
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 		   
				   a_selected_script->command=ACTION_CMD_WHILE_ELSE;
				break;
			case 7:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_DO;			   
				break;
			case 8:a_selected_script->command=ACTION_CMD_PAUSE;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				break;
			case 9:a_selected_script->command=ACTION_CMD_ATOMIC;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				break;

			case 10:a_selected_script->command=ACTION_CMD_SET;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->vpar[1]=new ValueSCRIPT();
				break;
			case 11:a_selected_script->command=ACTION_CMD_NEWVARIABLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					break;
			case 12:a_selected_script->command=ACTION_CMD_KILLVARIABLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					break;
			case 13:a_selected_script->command=ACTION_CMD_PLAY_SAMPLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					a_selected_script->vpar[1]->ipar=100;
					break;
			case 14:a_selected_script->command=ACTION_CMD_PLAY_SAMPLE_PAN;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					a_selected_script->vpar[1]->ipar=100;
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[2]->command=VALUE_CMD_INTEGER;
					break;
			case 15:a_selected_script->command=ACTION_CMD_PLAY_MUSIC;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					a_selected_script->vpar[1]->ipar=100;
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[2]->command=VALUE_CMD_INTEGER;
					break;
			case 16:a_selected_script->command=ACTION_CMD_STOP_MUSIC;
					break;

			case 17:a_selected_script->command=ACTION_CMD_CASE;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->vparlist.Delete();
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist[1]->ipar=1;

				   break;

			case 18:a_selected_script->command=ACTION_CMD_RANDOM_CASE;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->vparlist.Delete();
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist[0]->ipar=1;
				   a_selected_script->vparlist[1]->ipar=1;
				   break;

			} /* switch */ 
			return true;	
		} /* if */ 


		/* APPLICATION SCRIPTS: */ 
		if (level==5) {
			dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","none\nseq\nparallel\nif\nwhile\nwhile_else\ndo\npause\n"
															  "atomic\n"
															  "set\n"
															  "newvariable\nkillvariable\n"
															  "play_sample\nplay_sample_pan\n"
															  "play_music\nstop_music\n"
															  "delete_viewport\nadd_player\n"
															  "reset_game\nstop_game\nresume_game\nterminate\n"
															  "getplayer\nsetplayername\n"
															  "setplayerkeyboard\ngamemode\n"
															  "case\nrandom_case\n",&res,font);
			dlg->Execute();
			delete dlg;

			switch(res) {
			case 1:a_selected_script->command=ACTION_CMD_NONE;
				break;
			case 2:if (a_selected_script->command!=ACTION_CMD_SEQ &&
					   a_selected_script->command!=ACTION_CMD_PARALLEL) {
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_SEQ;
				break;
			case 3:if (a_selected_script->command!=ACTION_CMD_SEQ &&
					   a_selected_script->command!=ACTION_CMD_PARALLEL) {
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_PARALLEL;
				break;
			case 4:a_selected_script->command=ACTION_CMD_IF;
				   a_selected_script->cond->command=CONDITION_CMD_TRUE;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				break;
			case 5:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_WHILE;
				break;
			case 6:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } else {
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 		   
				   a_selected_script->command=ACTION_CMD_WHILE_ELSE;
				break;
			case 7:if (a_selected_script->command!=ACTION_CMD_WHILE &&
					   a_selected_script->command!=ACTION_CMD_DO) {
					   a_selected_script->cond->command=CONDITION_CMD_TRUE;
					   a_selected_script->aparlist.Delete();
					   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   } /* if */ 
				   a_selected_script->command=ACTION_CMD_DO;			   
				break;
			case 8:a_selected_script->command=ACTION_CMD_PAUSE;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				break;
			case 9:a_selected_script->command=ACTION_CMD_ATOMIC;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				break;

			case 10:a_selected_script->command=ACTION_CMD_SET;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->vpar[1]=new ValueSCRIPT();
				break;
			case 11:a_selected_script->command=ACTION_CMD_NEWVARIABLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					break;
			case 12:a_selected_script->command=ACTION_CMD_KILLVARIABLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					break;
			case 13:a_selected_script->command=ACTION_CMD_PLAY_SAMPLE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					a_selected_script->vpar[1]->ipar=100;
					break;
			case 14:a_selected_script->command=ACTION_CMD_PLAY_SAMPLE_PAN;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					a_selected_script->vpar[1]->ipar=100;
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[2]->command=VALUE_CMD_INTEGER;
					break;
			case 15:a_selected_script->command=ACTION_CMD_PLAY_MUSIC;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[0]->spar=new char[7];
					strcpy(a_selected_script->vpar[0]->spar,"noname");
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_INTEGER;
					a_selected_script->vpar[1]->ipar=100;
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[2]->command=VALUE_CMD_INTEGER;
					break;
			case 16:a_selected_script->command=ACTION_CMD_STOP_MUSIC;
					break;
			case 17:a_selected_script->command=ACTION_CMD_DELETE_VIEWPORT;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[0]->command=VALUE_CMD_INTEGER;
					break;
/*
			case 18:
					break;
*/

			case 19:a_selected_script->command=ACTION_CMD_RESETGAME;								
					break;
			case 20:a_selected_script->command=ACTION_CMD_STOPGAME;								
					break;
			case 21:a_selected_script->command=ACTION_CMD_RESUMEGAME;								
					break;
			case 22:a_selected_script->command=ACTION_CMD_TERMINATE;
					break;

			case 23:a_selected_script->command=ACTION_CMD_GETPLAYER;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   break;

			case 24:a_selected_script->command=ACTION_CMD_SETPLAYERNAME;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[1]=new ValueSCRIPT();
				   a_selected_script->vpar[1]->command=VALUE_CMD_STRING;
				   a_selected_script->vpar[1]->spar=new char[8];
				   strcpy(a_selected_script->vpar[1]->spar,"none");
				   break;

			case 25:a_selected_script->command=ACTION_CMD_SETPLAYERKEYBOARD;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					if (a_selected_script->vpar[2]!=0) delete a_selected_script->vpar[2];
					if (a_selected_script->vpar[3]!=0) delete a_selected_script->vpar[3];
					if (a_selected_script->vpar[4]!=0) delete a_selected_script->vpar[4];
					if (a_selected_script->vpar[5]!=0) delete a_selected_script->vpar[5];
					if (a_selected_script->vpar[6]!=0) delete a_selected_script->vpar[6];
					if (a_selected_script->vpar[7]!=0) delete a_selected_script->vpar[7];
					if (a_selected_script->vpar[8]!=0) delete a_selected_script->vpar[8];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[2]=new ValueSCRIPT();
					a_selected_script->vpar[3]=new ValueSCRIPT();
					a_selected_script->vpar[4]=new ValueSCRIPT();
					a_selected_script->vpar[5]=new ValueSCRIPT();
					a_selected_script->vpar[6]=new ValueSCRIPT();
					a_selected_script->vpar[7]=new ValueSCRIPT();
					a_selected_script->vpar[8]=new ValueSCRIPT();
					break;

			case 26:a_selected_script->command=ACTION_CMD_GAMEMODE;
					if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
					if (a_selected_script->vpar[1]!=0) delete a_selected_script->vpar[1];
					a_selected_script->vpar[0]=new ValueSCRIPT();
					a_selected_script->vpar[1]=new ValueSCRIPT();
					a_selected_script->vpar[1]->command=VALUE_CMD_STRING;
					a_selected_script->vpar[1]->spar=new char[12];
 					strcpy(a_selected_script->vpar[1]->spar,"default.rpl");
					break;

			case 27:a_selected_script->command=ACTION_CMD_CASE;
				   if (a_selected_script->vpar[0]!=0) delete a_selected_script->vpar[0];
				   a_selected_script->vpar[0]=new ValueSCRIPT();
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->vparlist.Delete();
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist[1]->ipar=1;

				   break;

			case 28:a_selected_script->command=ACTION_CMD_RANDOM_CASE;
				   a_selected_script->aparlist.Delete();
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->aparlist.Add(new ActionSCRIPT());
				   a_selected_script->vparlist.Delete();
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist.Add(new ValueSCRIPT());
				   a_selected_script->vparlist[0]->ipar=1;
				   a_selected_script->vparlist[1]->ipar=1;
				   break;

			} /* switch */ 
			return true;	
		} /* if */ 

	} /* if */ 


	return false;
} /* BViewActionScriptWnd::click_on_script */ 



bool BViewActionScriptWnd::right_click_on_script(void)
{

	if (v_selected_script!=0) {
		int res;
		BVChoiceDlg *dlg;

		dlg=new BVChoiceDlg(-1,-1,0,0,"Delete value?:","Delete\ncopy\ncut\npaste\n",&res,font);
		dlg->Execute();
		delete dlg;

		if (res==1) {
			v_selected_script->command=VALUE_CMD_INTEGER;
			v_selected_script->ipar=0;
		} /* if */ 

		if (res==2) {
			/* COPY */ 
			editor->se_clipboard_content=4;
			if (editor->se_clipboard_value!=0) delete editor->se_clipboard_value;
			editor->se_clipboard_value=new ValueSCRIPT(*v_selected_script);
		} /* if */ 

		if (res==3) {
			/* CUT */ 
			editor->se_clipboard_content=4;
			if (editor->se_clipboard_value!=0) delete editor->se_clipboard_value;
			editor->se_clipboard_value=new ValueSCRIPT(*v_selected_script);
			v_selected_script->command=ACTION_CMD_DELETE;
		} /* if */ 

		if (res==4) {
			/* PASTE */ 
			if (editor->se_clipboard_content==4) {
				v_selected_script->copy_state(editor->se_clipboard_value);
			} else {
				BChoiceDlg *dlg2=new BChoiceDlg(40,40,250,50,"The clipboard doesn't contain a value","Accept",&res,editor->font);
				dlg2->Execute();
				delete dlg2;
			} /* if */ 
		} /* if */ 

		return true;
	} /* if */ 

	if (c_selected_script!=0) {
		int res;
		BVChoiceDlg *dlg;

		dlg=new BVChoiceDlg(-1,-1,0,0,"Operation:","Delete\nAnd after\nAnd before\nOr after\nOr before\nNegate\ncopy\ncut\npaste\n",&res,font);
		dlg->Execute();
		delete dlg;

		if (res==1) {
			c_selected_script->aparlist.Delete();
			c_selected_script->command=CONDITION_CMD_DELETE;
		} /* if */ 

		if (res==2) {
			ConditionSCRIPT *tmp=new ConditionSCRIPT(*c_selected_script);
			c_selected_script->aparlist.Delete();
			c_selected_script->aparlist.Add(tmp);
			c_selected_script->aparlist.Add(new ConditionSCRIPT());
			c_selected_script->command=CONDITION_CMD_AND;
		} /* uf */ 
		if (res==3) {
			ConditionSCRIPT *tmp=new ConditionSCRIPT(*c_selected_script);
			c_selected_script->aparlist.Delete();
			c_selected_script->aparlist.Add(new ConditionSCRIPT());
			c_selected_script->aparlist.Add(tmp);
			c_selected_script->command=CONDITION_CMD_AND;
		} /* if */ 

		if (res==4) {
			ConditionSCRIPT *tmp=new ConditionSCRIPT(*c_selected_script);
			c_selected_script->aparlist.Delete();
			c_selected_script->aparlist.Add(tmp);
			c_selected_script->aparlist.Add(new ConditionSCRIPT());
			c_selected_script->command=CONDITION_CMD_OR;
		} /* if */ 
		if (res==5) {
			ConditionSCRIPT *tmp=new ConditionSCRIPT(*c_selected_script);
			c_selected_script->aparlist.Delete();
			c_selected_script->aparlist.Add(new ConditionSCRIPT());
			c_selected_script->aparlist.Add(tmp);
			c_selected_script->command=CONDITION_CMD_OR;
		} /* if */ 

		if (res==6) {
			ConditionSCRIPT *tmp;
			tmp=new ConditionSCRIPT(*c_selected_script);

			c_selected_script->aparlist.Delete();
			c_selected_script->aparlist.Add(tmp);
			c_selected_script->command=CONDITION_CMD_NOT;
		} /* if */ 

		if (res==7) {
			/* COPY */ 
			editor->se_clipboard_content=3;
			if (editor->se_clipboard_condition!=0) delete editor->se_clipboard_condition;
			editor->se_clipboard_condition=new ConditionSCRIPT(*c_selected_script);
		} /* if */ 

		if (res==8) {
			/* CUT */ 
			editor->se_clipboard_content=3;
			if (editor->se_clipboard_condition!=0) delete editor->se_clipboard_condition;
			editor->se_clipboard_condition=new ConditionSCRIPT(*c_selected_script);
			c_selected_script->command=CONDITION_CMD_DELETE;
		} /* if */ 

		if (res==9) {
			/* PASTE */ 
			if (editor->se_clipboard_content==3) {
				c_selected_script->copy_state(editor->se_clipboard_condition);
			} else {
				BChoiceDlg *dlg2=new BChoiceDlg(40,40,250,50,"The clipboard doesn't contain a condition","Accept",&res,editor->font);
				dlg2->Execute();
				delete dlg2;
			} /* if */ 
		} /* if */ 

		return true;
	} /* if */ 

	if (a_selected_script!=0) {
		int res;
		BVChoiceDlg *dlg;

		if (a_selected_script->command==ACTION_CMD_CASE ||
			a_selected_script->command==ACTION_CMD_RANDOM_CASE) {
			dlg=new BVChoiceDlg(-1,-1,0,0,"Operation","Delete\nInsert before\nInsert after\nInsert in parallel\ncopy\ncut\npaste\nAdd Case\nSub Case\n",&res,font);
		} else {
			dlg=new BVChoiceDlg(-1,-1,0,0,"Operation","Delete\nInsert before\nInsert after\nInsert in parallel\ncopy\ncut\npaste\n",&res,font);
		} /* if */ 
		dlg->Execute();
		delete dlg;

		if (res==1) {
			a_selected_script->command=ACTION_CMD_DELETE;
		} /* if */ 

		if (res==2) {
			ActionSCRIPT *tmp=new ActionSCRIPT(*a_selected_script);
			a_selected_script->aparlist.Delete();
			a_selected_script->aparlist.Add(new ActionSCRIPT());
			a_selected_script->aparlist.Add(tmp);
			a_selected_script->command=ACTION_CMD_SEQ;
		} /* if */ 
		if (res==3) {
			ActionSCRIPT *tmp=new ActionSCRIPT(*a_selected_script);
			a_selected_script->aparlist.Delete();
			a_selected_script->aparlist.Add(tmp);
			a_selected_script->aparlist.Add(new ActionSCRIPT());
			a_selected_script->command=ACTION_CMD_SEQ;
		} /* if */ 

		if (res==4) {
			ActionSCRIPT *tmp=new ActionSCRIPT(*a_selected_script);
			a_selected_script->aparlist.Delete();
			a_selected_script->aparlist.Add(tmp);
			a_selected_script->aparlist.Add(new ActionSCRIPT());
			a_selected_script->command=ACTION_CMD_PARALLEL;
		} /* if */ 

		if (res==5) {
			/* COPY */ 
			editor->se_clipboard_content=2;
			if (editor->se_clipboard_action!=0) delete editor->se_clipboard_action;
			editor->se_clipboard_action=new ActionSCRIPT(*a_selected_script);
		} /* if */ 

		if (res==6) {
			/* CUT */ 
			editor->se_clipboard_content=2;
			if (editor->se_clipboard_action!=0) delete editor->se_clipboard_action;
			editor->se_clipboard_action=new ActionSCRIPT(*a_selected_script);
			a_selected_script->command=ACTION_CMD_DELETE;
		} /* if */ 

		if (res==7) {
			/* PASTE */ 
			if (editor->se_clipboard_content==2) {
				a_selected_script->copy_state(editor->se_clipboard_action);
			} else {
				BChoiceDlg *dlg2=new BChoiceDlg(40,40,250,50,"The clipboard doesn't contain an action","Accept",&res,editor->font);
				dlg2->Execute();
				delete dlg2;
			} /* if */ 
		} /* if */ 

		if (res==8) {
			a_selected_script->aparlist.Add(new ActionSCRIPT());
			a_selected_script->vparlist.Add(new ValueSCRIPT());
		} /* if */ 

		if (res==9) {
			delete a_selected_script->aparlist.Extract();
			delete a_selected_script->vparlist.Extract();
		} /* if */ 

		return true;
	} /* if */ 

	return false;
} /* BViewActionScriptWnd::right_click_on_script */ 



void BViewActionScriptWnd::a_script_size(ActionSCRIPT *script,int *dx,int *dy)
{
	int i;
	char *pattern;

	if (script->dx!=0 &&
		script->dy!=0) {
		*dx=script->dx;
		*dy=script->dy;
		return;
	} /* if */ 

	*dx=0;
	*dy=0;
	pattern=0;
	for(i=0;i<ACTION_MAX_CMD;i++) {
		if (script->command==action_cmd_id[i]) pattern=action_cmd_pat[i];
	} /* for */ 

	if (pattern!=0) {
		int vcount=0;
		int acount=0;
		int x,y;
		int offset=0;
		char token[256];
		int dx2,dy2;
		int dx3,dy3;

		x=0;
		y=0;
		while(offset<int(strlen(pattern)) &&
			  1==sscanf(pattern+offset,"%s",token)) {
			offset+=strlen(token)+1;

			if (strcmp(token,"%VS")==0) {
				v_script_size(script->vpar[vcount],&dx2,&dy2);
				if (dx2+x+4>*dx) *dx=dx2+x+4;
				if (dy2+y+4>*dy) *dy=dy2+y+4;
				vcount++;
				x+=dx2+4+TTF_FontHeight(font);
			} else if(strcmp(token,"%CS")==0) {
				c_script_size(script->cond,&dx2,&dy2);
				if (dx2+x+4>*dx) *dx=dx2+x+4;
				if (dy2+y+4>*dy) *dy=dy2+y+4;
				x+=dx2+4+TTF_FontHeight(font);
			} else if(strcmp(token,"%AS")==0) {
				if (script->aparlist.Length()>acount) {
					a_script_size(script->aparlist[acount],&dx2,&dy2);
					if (dx2+x+4>*dx) *dx=dx2+x+4;
					if (dy2+y+4>*dy) *dy=dy2+y+4;
					acount++;
					x+=dx2+4+TTF_FontHeight(font);
				} /* if */ 
			} else if(strcmp(token,"%L")==0) {
				while(acount<script->aparlist.Length()) {
					a_script_size(script->aparlist[acount],&dx2,&dy2);
					if (dx2+x+4>*dx) *dx=dx2+x+4;
					if (dy2+y+4>*dy) *dy=dy2+y+4;
					acount++;
					y+=dy2+5;
				} // while  
			} else if(strcmp(token,"%1L")==0) {
				while(acount<(script->aparlist.Length()-1)) {
					a_script_size(script->aparlist[acount],&dx2,&dy2);
					if (dx2+x+4>*dx) *dx=dx2+x+4;
					if (dy2+y+4>*dy) *dy=dy2+y+4;
					acount++;
					y+=dy2+5;
				} // while 
			} else if(strcmp(token,"%VL")==0) {
				while(acount<script->aparlist.Length()) {
					v_script_size(script->vparlist[acount],&dx3,&dy3);
					a_script_size(script->aparlist[acount],&dx2,&dy2);
					if (dx2+5+dx3+x+4>*dx) *dx=dx2+5+dx3+x+4;
					if (max(dy2,dy3)+y+4>*dy) *dy=max(dy2,dy3)+y+4;
					acount++;
					y+=max(dy2,dy3)+5;
				} /* while */ 
			} else {
				dx2=get_text_width(token,font);
				dy2=TTF_FontHeight(font);
				if (dx2+x>*dx) *dx=dx2+x;
				if (dy2+y>*dy) *dy=dy2+y;
				x+=dx2+TTF_FontHeight(font);
			} // if 

		} // while 
	} // if 

	script->dx=*dx;
	script->dy=*dy;
} /* BViewActionScriptWnd::a_script_size */ 


void BViewActionScriptWnd::c_script_size(ConditionSCRIPT *script,int *dx,int *dy)
{
	int i;
	char *pattern;

	if (script->dx!=0 &&
		script->dy!=0) {
		*dx=script->dx;
		*dy=script->dy;
		return;
	} /* if */ 

	*dx=0;
	*dy=0;
	pattern=0;
	for(i=0;i<CONDITION_MAX_CMD;i++) {
		if (script->command==condition_cmd_id[i]) pattern=condition_cmd_pat[i];
	} /* for */ 

	if (pattern!=0) {
		int dx2,dy2;
		int vcount=0;
		int ccount=0;
		int x,y;
		int offset=0;
		char token[256];

		x=0;
		y=0;
		while(offset<int(strlen(pattern)) &&
			  1==sscanf(pattern+offset,"%s",token)) {
			offset+=strlen(token)+1;

			if (strcmp(token,"%VS")==0) {

				v_script_size(script->vpar[vcount],&dx2,&dy2);

				if (dx2+x+4>*dx) *dx=dx2+x+4;
				if (dy2+y+4>*dy) *dy=dy2+y+4;
				vcount++;
				x+=dx2+4+TTF_FontHeight(font);
			} else if(strcmp(token,"%CS")==0) {

				c_script_size(script->aparlist[ccount],&dx2,&dy2);
				if (dx2+x+4>*dx) *dx=dx2+x+4;
				if (dy2+y+4>*dy) *dy=dy2+y+4;
				ccount++;
				x+=dx2+4+TTF_FontHeight(font);
			} else if(strcmp(token,"%L")==0) {

				while(ccount<script->aparlist.Length()) {
					c_script_size(script->aparlist[ccount],&dx2,&dy2);
					if (dx2+x+4>*dx) *dx=dx2+x+4;
					if (dy2+y+4>*dy) *dy=dy2+y+4;
					ccount++;
					y+=dy2+5;
				} /* while */ 
			} else {
				dx2=get_text_width(token,font);
				dy2=TTF_FontHeight(font);
				if (dx2+x>*dx) *dx=dx2+x;
				if (dy2+y>*dy) *dy=dy2+y;
				x+=dx2+TTF_FontHeight(font);
			} /* if */ 
		} /* while */ 
	} /* if */ 

	script->dx=*dx;
	script->dy=*dy;
} /* BViewActionScriptWnd::c_script_size */ 


void BViewActionScriptWnd::v_script_size(ValueSCRIPT *script,int *dx,int *dy)
{
	int i;
	char *pattern;
	char tmp[256];

	if (script->dx!=0 &&
		script->dy!=0) {
		*dx=script->dx;
		*dy=script->dy;
		return;
	} /* if */ 

	*dx=0;
	*dy=0;
	pattern=0;
	for(i=0;i<VALUE_MAX_CMD;i++) {
		if (script->command==value_cmd_id[i]) pattern=value_cmd_pat[i];
	} /* for */ 

	if (pattern!=0) {
		int dx2,dy2;
		int vcount=0;
		int x,y;
		int offset=0;
		char token[256];


		x=0;
		y=0;
		while(offset<int(strlen(pattern)) &&
			  1==sscanf(pattern+offset,"%s",token)) {
			offset+=strlen(token)+1;

			if (strcmp(token,"%I")==0) {
				sprintf(tmp,"%i",script->ipar);
				dx2=get_text_width(tmp,font);
				dy2=TTF_FontHeight(font);
				if (dx2+x>*dx) *dx=dx2+x;
				if (dy2+y>*dy) *dy=dy2+y;
				x+=get_text_width(tmp,font)+TTF_FontHeight(font);
			} else if(strcmp(token,"%S")==0) {
				dx2=get_text_width(script->spar,font);
				dy2=TTF_FontHeight(font);
				if (dx2+x>*dx) *dx=dx2+x;
				if (dy2+y>*dy) *dy=dy2+y;
				x+=get_text_width(script->spar,font)+TTF_FontHeight(font);
			} else if(strcmp(token,"%SAM")==0) {
				if (script->sample!=0) {
					dx2=get_text_width(script->sample->name,font);
					dy2=TTF_FontHeight(font);
					if (dx2+x>*dx) *dx=dx2+x;
					if (dy2+y>*dy) *dy=dy2+y;
					x+=get_text_width(script->sample->name,font)+TTF_FontHeight(font);
				} else {
					dx2=get_text_width("-",font);
					dy2=TTF_FontHeight(font);
					if (dx2+x>*dx) *dx=dx2+x;
					if (dy2+y>*dy) *dy=dy2+y;
					x+=get_text_width("-",font)+TTF_FontHeight(font);
				} /* if */ 
			} else if(strcmp(token,"%K")==0) {
				sprintf(tmp,input_key_name(script->ipar));
				dx2=get_text_width(tmp,font);
				dy2=TTF_FontHeight(font);
				if (dx2+x>*dx) *dx=dx2+x;
				if (dy2+y>*dy) *dy=dy2+y;
				x+=get_text_width(tmp,font)+TTF_FontHeight(font);
			} else if(strcmp(token,"%V")==0) {
				tmp[0]=0;
				if (script->vlocation==VALUE_VAR_OBJECT) sprintf(tmp,"%s(OBJECT)",script->vname);
				if (script->vlocation==VALUE_VAR_ROOM) sprintf(tmp,"%s(ROOM)",script->vname);
				if (script->vlocation==VALUE_VAR_MAP) sprintf(tmp,"%s(MAP)",script->vname);
				if (script->vlocation==VALUE_VAR_GAME) sprintf(tmp,"%s(GAME)",script->vname);
				if (script->vlocation==VALUE_VAR_INTERFACE) sprintf(tmp,"%s(INTERFACE)",script->vname);
				if (script->vlocation==VALUE_VAR_FOREIGN) sprintf(tmp,"%s(FOREIGN)",script->vname);
				if (script->vlocation==VALUE_VAR_APPLICATION) sprintf(tmp,"%s(APPLICATION)",script->vname);
				dx2=get_text_width(tmp,font);
				dy2=TTF_FontHeight(font);
				if (dx2+x>*dx) *dx=dx2+x;
				if (dy2+y>*dy) *dy=dy2+y;
				x+=get_text_width(tmp,font)+TTF_FontHeight(font);
			} else if(strcmp(token,"%VS")==0) {

				v_script_size(script->vpar[vcount],&dx2,&dy2);

				if (dx2+x+4>*dx) *dx=dx2+x+4;
				if (dy2+y+4>*dy) *dy=dy2+y+4;

				vcount++;
				x+=dx2+4+TTF_FontHeight(font);;
			} else if(strcmp(token,"%CS")==0) {

				c_script_size(script->cond,&dx2,&dy2);

				if (dx2+x+4>*dx) *dx=dx2+x+4;
				if (dy2+y+4>*dy) *dy=dy2+y+4;

				vcount++;
				x+=dx2+4+TTF_FontHeight(font);;
			} else {
				dx2=get_text_width(token,font);
				dy2=TTF_FontHeight(font);
				if (dx2+x>*dx) *dx=dx2+x;
				if (dy2+y>*dy) *dy=dy2+y;
				x+=get_text_width(token,font)+TTF_FontHeight(font);
			} /* if */ 
		} /* while */ 
	} /* if */ 

	script->dx=*dx;
	script->dy=*dy;
} /* BViewActionScriptWnd::v_script_size */ 



void BViewActionScriptWnd::a_recompute_sizes(ActionSCRIPT *script)
{
	int i;
	ActionSCRIPT *as;
	List<ActionSCRIPT> l;

	for(i=0;i<MAX_ACTION_PARAMETERS;i++) {
		if (script->vpar[i]!=0) {
			v_recompute_sizes(script->vpar[i]);
		} /* if */ 
	} /* for */ 
	if (script->cond!=0) c_recompute_sizes(script->cond);

	l.Instance(script->aparlist);
	l.Rewind();
	while(l.Iterate(as)) {
		a_recompute_sizes(as);
	} /* while */ 
	script->dx=0;
	script->dy=0;
} /* BViewActionScriptWnd::a_recompute_sizes */ 


void BViewActionScriptWnd::c_recompute_sizes(ConditionSCRIPT *script)
{
	int i;
	ConditionSCRIPT *as;
	List<ConditionSCRIPT> l;

	for(i=0;i<MAX_CONDITION_PARAMETERS;i++) {
		if (script->vpar[i]!=0) {
			v_recompute_sizes(script->vpar[i]);
		} /* if */ 
	} /* for */ 

	l.Instance(script->aparlist);
	l.Rewind();
	while(l.Iterate(as)) {
		c_recompute_sizes(as);
	} /* while */ 
	script->dx=0;
	script->dy=0;
} /* BViewActionScriptWnd::c_recompute_sizes */ 


void BViewActionScriptWnd::v_recompute_sizes(ValueSCRIPT *script)
{
	if (script->vpar[0]!=0) v_recompute_sizes(script->vpar[0]);
	if (script->vpar[1]!=0) v_recompute_sizes(script->vpar[1]);
	script->dx=0;
	script->dy=0;

	if (script->cond!=0) c_recompute_sizes(script->cond);
} /* BViewActionScriptWnd::v_recompute_sizes */ 


void BViewActionScriptWnd::save_bmp(char *file)
{
	int dx=0,dy=0;
	SDL_Surface *sfc;

	if (a_script!=0) a_script_size(a_script,&dx,&dy);
	if (c_script!=0) c_script_size(c_script,&dx,&dy);
	if (v_script!=0) v_script_size(v_script,&dx,&dy);
	sfc=SDL_CreateRGBSurface(SDL_SWSURFACE,dx,dy,32,0,0,0,0);

	if (a_script!=0) a_draw_script(sfc,a_script,0,0);
	if (c_script!=0) c_draw_script(sfc,c_script,0,0);
	if (v_script!=0) v_draw_script(sfc,v_script,0,0);

	SDL_SaveBMP(sfc,file);

	SDL_FreeSurface(sfc);
} /* BViewActionScriptWnd::save_bmp */ 
